/*
 ***** BEGIN LICENSE BLOCK *****
 * Version: EPL 2.0/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Eclipse Public
 * License Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.eclipse.org/legal/epl-v10.html
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * Copyright (C) 2013-2017 The JRuby Team (jruby@jruby.org)
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the EPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the EPL, the GPL or the LGPL.
 ***** END LICENSE BLOCK *****/
package org.jruby.ext.ripper;

public class YyTables {
   private static short[] combine(short[] t1, short[] t2, 
                                  short[] t3, short[] t4) {
      short[] t = new short[t1.length + t2.length + t3.length + t4.length];
      int index = 0;
      System.arraycopy(t1, 0, t, index, t1.length);
      index += t1.length;
      System.arraycopy(t2, 0, t, index, t2.length);
      index += t2.length;
      System.arraycopy(t3, 0, t, index, t3.length);
      index += t3.length;
      System.arraycopy(t4, 0, t, index, t4.length);
      return t;
   }

   public static final short[] yyTable() {
      return combine(yyTable1(), yyTable2(), yyTable3(), yyTable4());
   }

   public static final short[] yyCheck() {
      return combine(yyCheck1(), yyCheck2(), yyCheck3(), yyCheck4());
   }
   private static final short[] yyTable1() {
      return new short[] {

          308,  421,  311,  423,  310,   88,   88,  134,  134,  305, 
          490,  713,  253,  253,  253,  304,  381,  279,  253,  253, 
          357,  667,  253,  619,  133,  133,  647,  648,  107,  633, 
           83,   83,  468,  263,  133,  529,  217,  673,  632,  531, 
          686,  291,  291,  327,  355,  291,  257,  260,   88,  634, 
          218,  218,  353,  433,  253,  635,  295,  295,  231,  430, 
          295,  710,    8,  275,  233,   83,  646,  685,  606,  133, 
          685,  218,    8,  635,  434,  602,  646,  610,  231,  686, 
          615,  277,  363,  646,  233,  646,  307,  379,  334,  338, 
          582,  259,  438,  785,  436,  646,  218,  294,  294,  259, 
          435,  294,  353,  756,  647,  937,  304,  307, 1000,  805, 
          646,  662,  427, 1030,  517,  646,   83,  695,  602,  133, 
          610,    8,  427,  814,  582,  676,  263,  574,  646,  333, 
          435,  253,  646,   88,  647,  927,  748,  758,  443,  582, 
          258,  581,  390, 1023,  844,  592,  580,  276,  258,  379, 
          778,  259,  605,  396,  826,  945,  566,  300,  346,  646, 
          641,  574,  566,  788,  526,  790,  464,  303,  647,  380, 
          317,  427,  526,  997,  751,  581,  574,  646,  641,  592, 
          580,  739,  411,   74,  707,   73,  600,  313,  307,  590, 
          581,  390,  478,  647,  592,  580,  307,  566,  525,  411, 
          258,  605,  396,  313,  566,  798,  528,  346,  752,  469, 
          303,  566,  383,  490,  845,  808, 1030,   86,  880,  784, 
          600,  626, 1062,  590,  887, 1000,  566,  503,  512,  789, 
          748,  380,  748,  749,   72,  600,   71,  839,  590,  646, 
           88,  686,  106,  106,  376,  381,  566, 1023, 1043,  106, 
          106,  106,  452,  384,  439,  106,  106,  379,  318,  106, 
          453,  382,  793,  377,  937,  380,  537,  253,  253,  378, 
          685,  685,  519,  635,  574,  739,   75,  739,  447,  447, 
           81,   86,  263,   53,  447,  106,   79,  554,  646,  646, 
          635,  106,  555,  520, 1074,  504,  505,  551,   72,  646, 
          473,  474,  893,  253,  748,  253,  253,  822,  574,  253, 
          605,  253,  533,  561,   76,  368,   88,  441,  562,  327, 
          895,  442,   88,  574,   88, 1088,  291,   75,  291,  291, 
          479,   81,  870,  253,   83,  479,  415,   79,  635,  481, 
          370,  295,  374,  295,  481,  263,  551,  515,  646,  739, 
          539,  540,  797,  376,  470,  353,  459,  542,  647,  605, 
          426,  647,  670,  427,  646,   76,  387,  388,  106,  646, 
          106,  619,  133,  635,  427,  427,  427,  425,  918,  427, 
          427,  427,  294,  427,  294,  253,  253,  253,  253,   88, 
          253,  253,  633,  427,  616,  275,  620,   83,  275,  818, 
          630,  634,  427,  427,  824,  427,  427,  427,  427,  427, 
          431,  646,  634,  277,  569,  612,  277,  646,  565,  566, 
          567,  568,  501,  253,  628,    8,  327,  748,  253,  748, 
          253,   88,  353,  432,  253,  253,  646,   88,  306,  582, 
          427,  580,  612,  811,  521,  522,  291,  427,  427,  133, 
          253,  646,   88,  253,  427,  645,  102,  102,  612,  636, 
          646,  306,   83,   88,  989,  611,  253,  991,  611,   84, 
          686,   88,  612,  291,  968,  427,  574,  106,  580,  276, 
          253,  600,  276,  693,  590,   92,   92,  135,  135,  291, 
          581,  390,  611,  646,  592,  672,  672,  280,  533,  102, 
          685,  605,  396,  291,  106,  106,  612,  427,  611,  625, 
          253,  580,   88,  625, 1086,   74,   72,   73,  600,  313, 
           84,  590,  611,   88,   77,  444,  870,  684,   92,  566, 
          870,  612,  354,  935,  818,  600,  566,   83,  590,  253, 
          106,  353,  106,  106,  383,   75,  106, 1033,  106,   81, 
          218,  600,  668,  106,  590,   79,  611,  231,  133,  106, 
          455,  106,  291,  233,  387,  388,  703,  458,   17,  574, 
          106,  757,  539,  540,  465,   77,  376,  381,  466,  542, 
          764,  611,  354,   76,  102,  384,   82,  533,  748,  379, 
          762,  470,  551,  382,   80,  377,  761,  380,  768,   16, 
          467,  378,   78,  771,  767,  605,  574,  687,   75,  767, 
           91,  472,   81,   92,  479,  691,  692,   17,   79,  475, 
          764,  448,  106,  106,  106,  106,  106,  106,  106,  551, 
          353,  792,  935,  700,  870,  485,  764,   82,  761,  574, 
          486,  767,  605,  253,   88,   80,   76,  133,   16,  489, 
          764,   95,   95,   78,   88,  288,  509,  514,  800,  797, 
          106,  782,  551,  281,  825,  106,  855,  106,  106,  389, 
          448,  106,  106,  513,  106,  605,  783,  828,  829,  986, 
          830,  764,  253,  791, 1067,  988,  514,  106,  935,  106, 
          106,  102,  646,  647,   95,  727,  728,  218,  218,  259, 
          106,  646,   88,  106,  518,  231,  524,  353,  106,   96, 
          530,  233,  532,  721,  889,  815,  538,  106,  722,  549, 
           92,  420,  646,  553,  133,  646,  871,   83,  876,  878, 
          819,  646,  646,  810,  883,  885,  935,  625,   84,  533, 
          322,  646,  777,  107,  620,   59,   60,  106,  258,  106, 
          646,  727,  728,  557,  332,  336,  560,   88,  629,   88, 
          106,   88,  253,  253,  637,  787,  507,  102,  253,  253, 
          650,  646,  655,  102,  646,  102,  106,  107,  253,   95, 
          659,  923,  353,  949,  539,  540,  544,  657,  953,  669, 
          544,  542,  688,   77,  107,  729,   92,  689,  690,  133, 
           88,   84,   92,  253,   92,  897,  389,  696,  699,  680, 
          820,  704,  901,  871,   88,   88,   88,  715,  823,  750, 
           17,   17,   17,  759,  316,  322,   17,   17,  760,   17, 
          765,  848,  850,  871,  447,  354,  910,  672,  404,  544, 
          102,  766,  406,  407,  775,  110,  617,   88,   88,  776, 
          964,   16,   16,   16,  779,   82,   77,   16,   16,  646, 
           16, 1013,  463,   80,  786,  803,  307,  417,  103,   92, 
          835,   78,  957,  797,  418,  422,  116,  809,  448,  804, 
          106,  106,  102,  806,  253,  807,   95,  821,  102,  133, 
          842,  106,  840,  841,  514,  847, 1037,  849,   88,   88, 
           17,  514,  514,  102,  899,  851,  983,  617,  646,  853, 
           88,   92,  354,  854,  102,  438,  857,   92,   82,  106, 
          122,   17,  102,  133,  882,  116,   80,  898,  617,  874, 
          514,   16,   92,  881,   78,  477,  903,  900,  646,  106, 
         1056,   97,  907,   92,  292,  298,  912,  911,  299,  646, 
          448,   92,   16,   17,  926,  646,  459,  520,  417,  646, 
          646, 1026,   95,  102,  514,  418,  419,  913,   95,  122, 
           95,  919,   88,  920,  102,  646,  646,  925,  133,  514, 
          835,  924,  646,   91,   16,  943,  946,  646,  520,   88, 
          948,   88,   92,  950,  106,  952,  106,   88,  106,  106, 
          106,  438, 1063,   92,  958,  106,  106,  959,  510,  960, 
          496,  514,  107,  539,  540,  106,  322,   91,  961,  963, 
          542,  354, 1003,  992,  993,  547,  998, 1007, 1009, 1001, 
         1011,  612, 1012,  517,   91,   95, 1002,  106,  996,  253, 
          106,  764, 1004,  716,   88,  717,  718,  719,  720, 1005, 
         1006,  106,  106,  106,  646, 1008,  251,  251,  251, 1010, 
         1016,  316,  291, 1031,  514, 1032, 1024,  517,  316,  316, 
          835,  417,  835, 1034,  621,  107,  624,   95,  418,  446, 
         1042,  611,  517,   95,  106,  106,  544, 1064,  544,  544, 
          544,  544,  544,  335, 1050,  102, 1052,  544,   95,  417, 
          463, 1054, 1057,  122,  438,  102,  418,  462, 1075,   95, 
          354,  646,  646,  646,  517, 1077, 1072,   95,  646, 1087, 
          544,  106, 1091,  322,   92,  633,  646,  496,  544,  544, 
          544,  544,  110,  116,   92,  106,  106,  539,  540,  541, 
         1076, 1078, 1079, 1080,  542,  835, 1045,  106,   61,   62, 
           63,   64,  438,  102,  496,  658,  536,  646,   95,  438, 
          438,  635,  716,  666,  717,  718,  719,  720,  721,   95, 
          496,  634,  417,  722,  544,  440,  116,  122,  635,  418, 
          476,  110,   92, 1095,  496,  639,  438,  354,  639,  835, 
          438,  835,  646,  641,  520,  832,  723,  646,  646,  646, 
          617,  520,  520,  635,  725,  726,  727,  728,  102,  106, 
          102,  635,  102,  514,  378,  716,   96,  717,  718,  719, 
          720,  366,  835,  858,  483,  520,  106,  137,  106,  491, 
          447,  494,  520,  520,  106,   97, 1036,   92,  438,   92, 
          316,   92,  559,  496, 1071,  438,  438,  701,  514,  833, 
          729,  102,   91,  617,  220,  514,  514,  624,  772,  103, 
          969,  617,  354,  934,  861,  102,  102,  102, 1018,   97, 
          755,  755,  438,  939,  316, 1019,  106,  888,  282,  514, 
           92,  106,  617, 1070, 1044,  922,   97, 1085,    0,  316, 
           95,  624,    0,  103,   92,   92,   92,    0,  102,  102, 
           95,  514,    0,  618,    0,  755,  624,    0,  514,  514, 
          103,  251,  251,    0,  617,   91,  520,  770,  933,    0, 
          936,  316,  770,  865,    0,    0,    0,   92,   92,  480, 
          481,  482,    0,    0,  484,  617,   51,  417,  624,    0, 
          438,    0,    0,    0,  418,  511,   51,    0,   95,  102, 
          102,  517,  135,  500,  770,  502,    0,    0,  517,  517, 
          389,  102,  796,  799,  618,  799,    0,  539,  540,  544, 
            0,    0,    0,  799,  542,    0,    0,  523,   92,   92, 
            0,   96,  517,  290,  290,  618,  984,  290,    0,  110, 
           92,    0,  404,  417,    0,   51,  406,  407,  408,  409, 
          418,  535,  617,   95,    0,   95,   96,   95,    0,  314, 
          316,    0,    0,    0,    0,  290,  290, 1017,  356,  358, 
            0, 1021,  940,  102,  624,  940,    0,    0,  519,  251, 
          251,  251,  251,    0,  570,  571,    0,  539,  540,  546, 
          102,  135,  102,    0,  542,    0,   95,    0,  102,  872, 
          514,  873,   92,  875,    0,    0,    0,  514,  514,    0, 
           95,   95,   95,  617,    0,    0,    0,  417,    0,   92, 
            0,   92,  623,  417,  418, 1025,  633,   92,    0, 1047, 
          418, 1035,    0,    0,  617,    0,    0,    0,  755,  539, 
          540,  550,  906,   95,   95,  102,  542,  649,    0,    0, 
            0,    0,  905,    0,   97,  909,  915,  916,  917,    0, 
          205,  539,  540,  558,  865,  623,    0,  865,  542,  940, 
          868,  865,    0,    0,   92, 1082,    0,  624,  103,  438, 
          681,  683,    0,    0,  335,  485,  485,  485,    0,  931, 
          932,    0,  485,    0,   95,   95,  716,    0,  717,  718, 
          719,  720,  985,  520,  694,    0,   95,    0,  316,  205, 
          520,  520,  467,  467,  467,  316,  316,   97,    0,  467, 
          683,  621,    0,  335,    0,  624,    0,  438,    0,  473, 
          473,  473,  624,  624,  438,  438,  473,    0,  620,  316, 
          967,  103,  107,    0,   98,    0,    0,    0,   51,   51, 
           51,    0,  987,   51,   51,   51,  624,   51,    0,  624, 
         1027,  438,  717,  718,  719,  720,  115,   51,   95,  941, 
            0,  865,  799,  865,    0,  865,   51,   51,  865,   51, 
           51,   51,   51,   51,    0,   95,    0,   95,    0,  316, 
            0,  521,    0,   95,  382,  383,  384,  385,  386,  438, 
            0,    0,  290,  290,  290,  358,    0,  290,    0,  438, 
            0,    0,  865,  773,    0,  519,    0,    0,  290,    0, 
          290,  290,  519,  519,    0,    0,    0,    0,   51,  508, 
            0, 1038,    0, 1039,    0,    0,    0,  251,    0, 1040, 
           95,    0,    0,  438,    0,    0,    0,    0,    0,   51, 
          794,    0,    0,    0,    0,    0,    0,  438,  438,    0, 
          122,  868,  438,  389,  868,    0, 1020,    0,  868,    0, 
            0,    0,    0,    0,    0,  816,  251,  817,  869,  402, 
          403,   51,    0,    0,  683,  335, 1073,    0,  126,    0, 
          438,    0,    0,  496,    0,  404,  799,  405,    0,  406, 
          407,  408,  409,    0,   99,    0,   96,  572,  573,  574, 
          575,  576,  577,  578,  579,  580,  581,  582,  583,  584, 
          585,  586,  587,  588,  589,  590,  591,  592,  593,  594, 
          595,  596,  597,    0,    0,    0,    0,  126,  290,    0, 
           96,    0,  856,  617,    0,  621,    0,    0,    0,  290, 
          635,  522,    0,    0,    0,    0,  205,   96,  205,  205, 
          205,  205,  205,    0,    0,  290,  246,  205,  868,  316, 
          868,  892,  868,    0,    0,  868,  246,  126,  520,  117, 
            0,  290,    0,  617,  665,  635,  520,  519,  290,    0, 
          205,  205,    0,    0,    0,  290,    0,  251,  205,  205, 
          205,  205,    0,  290,  290,  914,    0,  290,    0,  868, 
          246,  716,  860,  717,  718,  719,  720,    0,    0,    0, 
            0,    0,  115,    0,  246,  246,  316,    0,  521,  246, 
            0,  697,  698,  316,  316,  521,  521,    0,  293,  621, 
            0,    0,    0,  290,  205,  833,  290,    0,  293,    0, 
            0,    0,  942,  834,  290,    0,    0,    0,    0,  117, 
            0,  438,  438,  438,    0,    0,  438,  438,  438,  869, 
          438,  115,  869,    0,  869,    0,  869,    0,    0,  438, 
          438,  438,  288,    0,    0,    0,  966,    0,  118,  438, 
          438,    0,  438,  438,  438,  438,  438,  293,  120,    0, 
            0,    0,  928,  519,  438,    0,    0,    0,  117,  938, 
            0,    0,    0,    0,  335,    0,  438,  438,  438,  438, 
          438,  438,  438,  438,  438,  438,  438,  438,  438,  438, 
            0,    0,  438,  438,  438,  438,  290,  118,  438,  118, 
          521,  438,    0,    0,  438,  126,  438,  120,  438,    0, 
          438,    0,  438,  438,  438,  438,  438,  438,  438,    0, 
          438,  438,  438,    0,    0,    0,    0,    0,    0,  522, 
            0,    0,    0,  290,  438,   96,  869,    0,  869,  524, 
          869,  514,    0,  869,    0,    0,    0,    0,  522,    0, 
            0,  514,    0,    0,  438,  522,  522,    0,  290,    0, 
          290,    0,    0, 1014,    0,  947,  316,  290,  290,  951, 
         1022,    0,    0,  316,  316,    0,    0,  869,    0,  621, 
            0,    0,    0,    0,  519,  514,    0,    0,  246,  246, 
          246,  519,  519,  246,  246,  246,    0,  246,   96,  514, 
          514,    0,  110,    0,  514,    0,    0,  246,  246,    0, 
            0,    0,    0,    0,    0,    0,  246,  246,    0,  246, 
          246,  246,  246,  246,    0,  290, 1046,    0, 1048,    0, 
         1049,  246,  514,    0,    0,    0,    0,  877,  879,  115, 
            0,    0,    0,  884,  886,    0,    0,    0,    0,    0, 
          246,  246,  890,  665,  290,  246,  246,    0,  894,    0, 
          293,  293,  293,    0,    0,  246,  293,  293,  246,  293, 
            0,  246, 1081,  246, 1083,    0,  117, 1084,    0,    0, 
          517,  877,  879,    0,  884,  886,    0,    0,  290,  246, 
          517,  293,  293,  293,  293,  293,    0, 1089,    0,    0, 
          519,  246,    0,    0,    0,  118,    0,  519,  519,    0, 
            0, 1096,    0,    0,    0,  120,    0,    0,    0, 1051, 
            0,  246, 1053, 1055,  517,    0, 1058, 1059,    0,    0, 
            0,    0,    0,    0,    0,  290,    0,  521,  517,  517, 
          293,  113,    0,  517,  521,  521,    0,    0,    0,    0, 
            0,  716,  962,  717,  718,  719,  720,  721,    0,  965, 
            0,  293,  722,    0,    0,    0,  522,    0,    0,  290, 
            0,  517,    0,  522,  522,    0,  524,    0,    0,    0, 
            0,    0,    0,  524,  524,  723, 1090, 1092, 1093, 1094, 
          965,    0,  198,  293,    0,  727,  728,  290,  617,  617, 
          617, 1097,  617,  514,  514,  514,  617,  617,  514,  514, 
          514,  617,  514,  617,  617,  617,  617,  617,  617,  617, 
            0,  617,  514,  514,  617,  617,  617,  617,  617,  617, 
          617,  514,  514,  617,  514,  514,  514,  514,  514,  729, 
          617,  198,    0,  617,  617,  617,  514,  617,  617,  617, 
          617,  617,  617,  617,  617,  617,  617,  617,  514,  514, 
          514,  514,  514,  514,  514,  514,  514,  514,  514,  514, 
          514,  514,   94,    0,  514,  514,  514,  514,  617,  526, 
          514,  617,  617,  514,  617,  617,  514,  617,  514,  617, 
          514,  617,  514,  617,  514,  514,  514,  514,  514,  514, 
          514,  617,  514,  617,  514,    0,  617,  617,  617,  617, 
          617,  617,  105,    0,  290,  617,  514,  617,  617,  517, 
          617,  617,  617,  617,    0,    0,    0,    0,    0,    0, 
            0,  119,    0,  617,  617,  617,  514,  618,  618,  618, 
            0,  618,  517,  517,  517,  618,  618,  517,  517,  517, 
          618,  517,  618,  618,  618,  618,  618,  618,  618,  624, 
          618,  517,  517,  618,  618,  618,  618,  618,  618,  618, 
          517,  517,  618,  517,  517,  517,  517,  517,    0,  618, 
          119,    0,  618,  618,  618,  517,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  517,  517,  517, 
          517,  517,  517,  517,  517,  517,  517,  517,  517,  517, 
          517,  101,  523,  517,  517,  517,  517,  618,  529,  517, 
          618,  618,  517,  618,  618,  517,  618,  517,  618,  517, 
          618,  517,  618,  517,  517,  517,  517,  517,  517,  517, 
          618,  517,  618,  517,    0,  618,  618,  618,  618,  618, 
          618,    0,    0,  624,  618,  517,  618,  618,  524,  618, 
          618,  618,  618,  624,    0,    0,    0,  545,    0,  548, 
            0,  551,  618,  618,  618,  517,    0,  551,  633,  633, 
          633,    0,    0,    0,  633,  633,    0,  633,  198,    0, 
          198,  198,  198,  198,  198,    0,    0,  624,    0,  198, 
          716,    0,  717,  718,  719,  720,    0,  113,  618,    0, 
          519,  624,  624,    0,  124,    0,  624,    0,  519,  634, 
          634,  634,  198,  198,    0,  634,  634,    0,  634,    0, 
          198,  198,  198,  198,  833,    0,  389,    0,    0,    0, 
            0,    0,  999,    0,  624,    0,    0,  124,  624,    0, 
            0,    0,  402,  403,    0,    0,  517,    0,  633,    0, 
            0,    0,    0,  517,  517,    0,    0,    0,  404,  618, 
          405,    0,  406,  407,  408,  409,  198,    0,  412,  633, 
          413,    0,    0,    0,    0,  545,    0,    0,  119,    0, 
          618,    0,  620,    0,    0,    0,  624,    0,    0,  634, 
            0,  389,  620,  624,  624,    0,  394,  395,    0,  620, 
            0,  633,    0,    0,    0,    0,    0,  402,  403,    0, 
          634,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          624,    0,    0,  404,    0,  405,  620,  406,  407,  408, 
          409,  410,  411,  412,    0,  413,  120,    0,    0,  523, 
          620,  620,  634,  123,  714,  620,  523,  523,    0,    0, 
            0,  714,    0,  716,    0,  717,  718,  719,  720,  721, 
            0,    0,    0,    0,  722,    0,    0,    0,    0,    0, 
            0,    0,    0,  620,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  524,    0,  723,  724,    0, 
            0,    0,  524,  524,  208,  725,  726,  727,  728,    0, 
          620,  620,  620,    0,  620,  624,  624,  624,  620,  620, 
          624,  624,  624,  620,  624,  620,  620,  620,  620,  620, 
          620,  620,    0,  624,  624,  624,  620,  620,  620,  620, 
          620,  620,  620,  624,  624,  620,  624,  624,  624,  624, 
          624,  729,  620,  208,    0,  620,  620,  620,  624,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          624,  624,  624,  624,  624,  624,  624,  624,  624,  624, 
          624,  624,  624,  624,  104,    0,  624,  624,  624,  624, 
          620,  193,  624,  620,  620,  624,  620,  620,  624,  620, 
          624,  620,  624,  620,  624,  620,  624,  624,  624,  624, 
          624,  624,  624,  620,  624,  624,  624,    0,  620,  620, 
          620,  620,  620,  620,  100,    0,    0,  620,  624,  620, 
          620,  620,  620,  620,  620,  620,    0,    0,    0,    0, 
          193,    0,    0,  121,    0,  620,  620,  620,  624,  620, 
          620,  620,    0,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  523,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
            0,  620,  121,    0,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  102,  525,  620,  620,  620,  620,  620, 
            0,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,    0,  620,  620,  620, 
          620,  620,  620,    0,    0,  621,  620,  620,  620,  620, 
          525,  620,  620,  620,  620,  621,  389,    0,    0,    0, 
            0,    0,    0,    0,  620,  620,  620,  620,    0,    0, 
            0,    0,  402,  403,    0,    0,    0,    0,    0,    0, 
          208,    0,  208,  208,  208,  208,  208,    0,  404,  621, 
            0,  208,  406,  407,  408,  409,    0,    0,    0,  123, 
          620,    0,  521,  621,  621,    0,  125,    0,  621,    0, 
          521,    0,    0,    0,  208,  208,    0,    0,    0,    0, 
            0,    0,  208,  208,  208,  208,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  621,    0,    0,  119, 
            0,    0,    0,    0,    0,    0,    0,    0,  620,    0, 
            0,    0,    0,    0,    0,  620,  620,    0,    0,    0, 
            0,  620,    0,    0,    0,    0,    0,  193,  208,  193, 
          193,  193,  193,  193,    0,    0,    0,    0,  193,    0, 
          121,    0,  620,    0,  307,    0,    0,    0,  523,  522, 
            0,    0,    0,    0,  307,  523,  523,  522,    0,    0, 
            0,  193,  193,    0,    0,    0,    0,    0,    0,  193, 
          193,  193,  193,    0,    0,    0,    0,    0,    0,    0, 
          389,  390,  391,  392,  393,  394,  395,  396,  307,  398, 
          399,    0,    0,    0,    0,    0,  402,  403,  121,    0, 
            0,  525,  307,  307,    0,    0,    0,  307,  525,  525, 
            0,    0,  404,    0,  405,  193,  406,  407,  408,  409, 
          410,  411,  412,    0,  413,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  307,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  525,    0,    0, 
            0,    0,    0,    0,  525,  525,  175,    0,    0,    0, 
            0,    0,  621,  621,  621,    0,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,    0,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,  621, 
          621,  621,  621,    0,  621,  175,    0,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,  106,    0,  621,  621, 
          621,  621,  621,  176,  621,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,    0, 
          621,  621,  621,  621,  621,  621,   93,    0,    0,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,    0,    0, 
            0,    0,  176,    0,    0,  112,    0,  621,  621,  621, 
          621,  619,  619,  619,    0,  619,  307,  307,  307,  619, 
          619,  307,  307,  307,  619,  307,  619,  619,  619,  619, 
          619,  619,  619,  516,  619,  307,  307,  619,  619,  619, 
          619,  619,  619,  619,  307,  307,  619,  307,  307,  307, 
          307,  307,    0,  619,  112,    0,  619,  619,  619,  307, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  307,  307,  307,  307,  307,  307,  307,  307,  307, 
          307,  307,  307,  307,  307,   92,  516,  307,  307,  307, 
          307,  619,    0,  307,  619,  619,  307,  619,  619,  307, 
          619,  307,  619,  307,  619,  307,  619,  307,  307,  307, 
          307,  307,  307,  307,  619,  307,  619,  307,    0,  619, 
          619,  619,  619,  619,  619,    0,    0,  625,  619,  307, 
          619,  619,  515,  619,  619,  619,  619,  625,    0,    0, 
            0,    0,    0,    0,    0,    0,  619,  619,  619,  307, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  175,    0,  175,  175,  175,  175,  175,    0, 
            0,  625,    0,  175,    0,    0,    0,    0,    0,    0, 
            0,  125,  621,    0,  524,  625,  625,    0,    0,    0, 
          625,    0,  524,    0,    0,    0,  175,  175,    0,    0, 
            0,    0,    0,    0,  175,  175,  175,  175,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  625,    0, 
            0,  112,    0,    0,    0,    0,    0,    0,    0,    0, 
          621,    0,    0,    0,    0,    0,    0,  621,  621,    0, 
            0,    0,    0,  621,    0,    0,    0,    0,    0,  176, 
          175,  176,  176,  176,  176,  176,    0,    0,    0,    0, 
          176,    0,  112,    0,  621,    0,  626,    0,    0,    0, 
          516,  523,    0,    0,    0,    0,  626,  516,  516,  523, 
            0,    0,    0,  176,  176,    0,    0,    0,    0,    0, 
            0,  176,  176,  176,  176,    0,    0,    0,    0,    0, 
            0,    0,  389,  390,  391,  392,  393,  394,  395,    0, 
          626,  398,  399,    0,    0,    0,    0,    0,  402,  403, 
          111,    0,    0,  516,  626,  626,    0,    0,    0,  626, 
          516,  516,    0,    0,  404,    0,  405,  176,  406,  407, 
          408,  409,  410,  411,  412,    0,  413,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  626,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  515, 
            0,    0,    0,    0,    0,    0,  515,  515,  177,    0, 
            0,    0,    0,    0,  622,  622,  622,    0,  622,  625, 
          625,  625,  622,  622,  625,  625,  625,  622,  625,  622, 
          622,  622,  622,  622,  622,  622,    0,  625,  625,  625, 
          622,  622,  622,  622,  622,  622,  622,  625,  625,  622, 
          625,  625,  625,  625,  625,    0,  622,  177,    0,  622, 
          622,  622,  625,  622,  622,  622,  622,  622,  622,  622, 
          622,  622,  622,  622,  625,  625,  625,  625,  625,  625, 
          625,  625,  625,  625,  625,  625,  625,  625,   95,    0, 
          625,  625,  625,  625,  622,  134,  625,  622,  622,  625, 
          622,  622,  625,  622,  625,  622,  625,  622,  625,  622, 
          625,  625,  625,  625,  625,  625,  625,  622,  625,  625, 
          625,    0,  622,  622,  622,  622,  622,  622,  109,    0, 
            0,  622,  625,  622,  622,  518,  622,  622,  622,  622, 
            0,    0,    0,    0,  134,    0,    0,  111,    0,  622, 
          622,  622,  625,  623,  623,  623,    0,  623,  626,  626, 
          626,  623,  623,  626,  626,  626,  623,  626,  623,  623, 
          623,  623,  623,  623,  623,  306,  626,  626,  626,  623, 
          623,  623,  623,  623,  623,  623,  626,  626,  623,  626, 
          626,  626,  626,  626,    0,  623,  111,    0,  623,  623, 
          623,  626,  623,  623,  623,  623,  623,  623,  623,  623, 
          623,  623,  623,  626,  626,  626,  626,  626,  626,  626, 
          626,  626,  626,  626,  626,  626,  626,  108,  515,  626, 
          626,  626,  626,  623,    0,  626,  623,  623,  626,  623, 
          623,  626,  623,  626,  623,  626,  623,  626,  623,  626, 
          626,  626,  626,  626,  626,  626,  623,  626,  626,  626, 
            0,  623,  623,  623,  623,  623,  623,    0,    0,  316, 
          623,  626,  623,  623,  317,  623,  623,  623,  623,  316, 
            0,    0,    0,    0,    0,    0,    0,    0,  623,  623, 
          623,  626,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  177,    0,  177,  177,  177,  177, 
          177,    0,    0,  316,    0,  177,    0,    0,    0,    0, 
            0,    0,    0,  114,    0,    0,  525,  316,  316,    0, 
          126,    0,  316,    0,  525,    0,    0,    0,  177,  177, 
            0,    0,    0,    0,    0,    0,  177,  177,  177,  177, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          316,    0,    0,  128,    0,    0,    0,    0,    0,    0, 
            0,    0,  518,    0,    0,    0,    0,    0,    0,  518, 
          518,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  134,  177,  134,  134,  134,  134,  134,    0,    0, 
            0,    0,  134,    0,  111,    0,    0,    0,  437,    0, 
            0,    0,  306,  514,    0,    0,    0,    0,  437,  306, 
          306,  514,    0,    0,    0,  134,  134,    0,    0,    0, 
            0,    0,    0,  134,  134,  134,  134,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  437,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  127,    0,    0,  515,    0,  437,    0,    0, 
            0,  437,  515,  515,    0,    0,    0,    0,    0,  134, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  437, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  317,    0,    0,    0,    0,    0,    0,  317,  317, 
          135,    0,    0,    0,    0,    0,  621,  621,  621,    0, 
          621,  316,  316,  316,  621,  621,  316,  316,  316,  621, 
          316,  621,  621,  621,  621,  621,  621,  621,    0,    0, 
          316,  316,  621,  621,  621,  621,  621,  621,  621,  316, 
          316,  621,  316,  316,  316,  316,  316,    0,  621,  135, 
            0,  621,  621,  621,  316,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,  316,  316,  316,  316, 
          316,  316,  316,  316,  316,  316,  316,  316,  316,  316, 
            0,    0,  316,  316,  316,  316,  621,    0,  316,  621, 
          621,  316,  621,  621,  316,  621,  316,  621,  316,  621, 
          316,  621,  316,  316,  316,  316,  316,  316,  316,  621, 
          316,    0,  316,    0,  621,  621,  621,  621,  621,  621, 
            0,    0,    0,  621,  316,  621,  621,    0,  621,  621, 
          621,  621,  352,    0,    0,    0,    0,    0,    0,    0, 
            0,  621,  621,  621,  316,  283,  283,  283,    0,  283, 
          437,  437,  437,  283,  283,  437,  437,  437,  283,  437, 
          283,  283,  283,  283,  283,  283,  283,    0,  437,  437, 
          437,  283,  283,  283,  283,  283,  283,  283,  437,  437, 
          283,  437,  437,  437,  437,  437,    0,  283,    0,    0, 
          283,  283,  283,    0,  283,  283,  283,  283,  283,  283, 
          283,  283,  283,  283,  283,  437,  437,  437,  437,  437, 
          437,  437,  437,  437,  437,  437,  437,  437,  437,    0, 
            0,  437,  437,  437,  437,  283,    0,  437,  283,    0, 
          437,  283,  283,  437,  283,  437,  283,  437,  283,  437, 
          283,  437,  437,  437,  437,  437,  437,  437,  283,  437, 
          437,  437,    0,  283,  283,  283,  283,  283,  283,    0, 
            0,  646,  283,    0,  283,  283,    0,  283,  283,  283, 
          283,  646,    0,    0,    0,    0,    0,    0,    0,    0, 
          283,  283,  283,  437,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  135,    0,  135,  135, 
          135,  135,  135,    0,    0,  646,    0,  135,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  517,    0, 
          646,    0,    0,    0,  646,    0,  517,    0,    0,    0, 
          135,  135,    0,    0,    0,    0,    0,    0,  135,  135, 
          135,  135,    0,    0,    0,    4,    5,    6,    0,    8, 
            0,    0,  646,    9,   10,    0,    0,    0,   11,    0, 
           12,   13,   14,  108,  109,   17,   18,    0,    0,    0, 
            0,  110,  111,  112,  339,  340,  341,  342,    0,    0, 
          117,    0,    0,    0,  135,    0,    0,  118,    0,    0, 
          343,  344,  345,    0,  346,   35,  347,  348,  349,  350, 
          323,   40,    0,    0,  127,    0,    0,    0,    0,    0, 
          323,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  351,    0,    0,  130,    0, 
            0,   46,   47,    0,   48,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  323,    0,    0,    0,    0,    0, 
            0,    0,    0,   52,   53,   54,   55,   56,   57,  323, 
            0,    0,   58,  323,   59,   60,    0,   61,   62,   63, 
           64,  716,    0,  717,  718,  719,  720,  721,    0,    0, 
           65,   66,  722,    0,    0,    0,    0,    0,    0,    0, 
            0,  323,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  723,    0,    0,    0,    0, 
            0,    0,    0,  725,  726,  727,  728,    0,  283,  283, 
          283,    0,  283,  646,  646,  646,  283,  283,  646,  646, 
          646,  283,  646,  283,  283,  283,  283,  283,  283,  283, 
            0,  646,  646,  646,  283,  283,  283,  283,  283,  283, 
          283,  646,  646,  283,  646,  646,  646,  646,  646,  729, 
          283,    0,    0,  283,  283,  283,    0,  283,  283,  283, 
          283,  283,  283,  283,  283,  283,  283,  283,  646,  646, 
          646,  646,  646,  646,  646,  646,  646,  646,  646,  646, 
          646,  646,  262,    0,  646,  646,  646,  646,  283,    0, 
          646,  283,  262,  646,  283,  283,  646,  283,  646,  283, 
          646,  283,  646,  283,  646,  646,  646,  646,  646,  646, 
          646,  283,  646,  646,  646,    0,  283,  283,  283,  283, 
          283,  283,    0,    0,    0,  283,  262,  283,  283,    0, 
          283,  283,  283,  283,    0,    0,    0,    0,    0,    0, 
          262,  262,    0,  283,  283,  283,  646,  283,  283,  283, 
            0,  283,  323,  323,  323,  283,  283,  323,  323,  323, 
          283,  323,  283,  283,  283,  283,  283,  283,  283,    0, 
            0,  323,  323,  283,  283,  283,  283,  283,  283,  283, 
          323,  323,  283,  323,  323,  323,  323,  323,    0,  283, 
            0,    0,  283,  283,  283,    0,  283,  283,  283,  283, 
          283,  283,  283,  283,  283,  283,  283,  323,  323,  323, 
          323,  323,  323,  323,  323,  323,  323,  323,  323,  323, 
          323,    0,    0,  323,  323,  323,  323,  283,    0,  323, 
          283,    0,  323,  283,  283,  323,  283,  323,  283,  323, 
          283,  323,  283,  323,  323,  323,  323,  323,  323,  323, 
          283,  323,  514,  323,    0,  283,  283,  283,  283,  283, 
          283,    0,  514,    0,  283,    0,  283,  283,    0,  283, 
          283,  283,  283,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  283,  283,  283,  323,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,   91,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  514,    0,  110,    0,  514,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  514,  262,  262,  262,    0,    0,  262, 
          262,  262,    0,  262,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  262,  262,    0,    0,    0,    0,    0, 
            0,    0,  262,  262,    0,  262,  262,  262,  262,  262, 
            0,    0,    0,    0,    0,    0,    0,  262,    0,    0, 
            0,  517,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  517,  389,  390,  391,  392,  393,  394,  395,  396, 
          397,  398,  399,    0,    0,    0,    0,    0,  402,  403, 
            0,  262,    0,    0,  262,    0,    0,  262,    0,  262, 
            0,    0,    0,    0,  404,   94,  405,    0,  406,  407, 
          408,  409,  410,  411,  412,  262,  413,  613,  608,    0, 
          517,  614,  113,    0,  517,  187,  188,  262,  189,  190, 
          191,  192,  193,  194,  195,    0,    0,  196,  197,    0, 
            0,    0,  198,  199,  200,  201,    0,  262,    0,    0, 
            0,    0,  517,    0,    0,    0,    0,    0,    0,  203, 
          204,    0,  205,  206,  207,  208,  209,  210,  211,  212, 
          213,  214,  215,    0,    0,  216,    0,    0,    0,  617, 
          617,  617,    0,  617,  514,  514,  514,  617,  617,    0, 
          514,  514,  617,  514,  617,  617,  617,  617,  617,  617, 
          617,    0,  617,    0,    0,  617,  617,  617,  617,  617, 
          617,  617,  514,  514,  617,  514,  514,  514,  514,  514, 
            0,  617,    0,    0,  617,  617,  617,    0,  617,  617, 
          617,  617,  617,  617,  617,  617,  617,  617,  617,  514, 
          514,  514,  514,  514,  514,  514,  514,  514,  514,  514, 
          514,  514,  514,  414,    0,  514,  514,  514,  514,  617, 
          526,   94,  617,  617,  514,  617,  617,    0,  617,    0, 
          617,  514,  617,  514,  617,  514,  514,  514,  514,  514, 
          514,  514,  617,  514,  617,  514,    0,  617,  617,  617, 
          617,  617,  617,    0,    0,   94,  617,    0,  617,  617, 
            0,  617,  617,  617,  617,    0,    0,    0,    0,    0, 
            0,    0,   94,    0,  617,  617,  617,  514,  618,  618, 
          618,    0,  618,  517,  517,  517,  618,  618,    0,  517, 
          517,  618,  517,  618,  618,  618,  618,  618,  618,  618, 
            0,  618,  517,    0,  618,  618,  618,  618,  618,  618, 
          618,  517,  517,  618,  517,  517,  517,  517,  517,    0, 
          618,    0,    0,  618,  618,  618,    0,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  517,  517, 
          517,  517,  517,  517,  517,  517,  517,  517,  517,  517, 
          517,  517,  414,    0,  517,  517,  517,  517,  618,  529, 
            0,  618,  618,  517,  618,  618,    0,  618,    0,  618, 
          517,  618,  517,  618,  517,  517,  517,  517,  517,  517, 
          517,  618,  517,  618,  517,    0,  618,  618,  618,  618, 
          618,  618,    0,    0,  624,  618,    0,  618,  618,    0, 
          618,  618,  618,  618,  624,    0,    0,    0,    0,    0, 
            0,    0,    0,  618,  618,  618,  517,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  105,    0, 
            0,    0,    0,    0,    0,    0,    0,  774,    0,    0, 
            0,    0,    0,  624,    0,  124,    0,  624,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  389,  390,  391, 
          392,  393,  394,  395,  396,  397,  398,  399,    0,  400, 
          401,    0,    0,  402,  403,  624,    0,    0,    0,    0, 
           94,  618,    0,    0,    0,    0,    0,    0,    0,  404, 
            0,  405,    0,  406,  407,  408,  409,  410,  411,  412, 
            0,  413,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  620,    0,    0,    0,    0,    0,  517, 
            0,    0,    0,  620,    0,    0,  517,  517,    0,    0, 
            0,    0,  618,   94,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  618,    0,    0,    0,  104,    0,    0, 
            0,    0,    0,    0,    0,    0,  780,    0,    0,    0, 
            0,    0,  620,    0,  123,    0,  620,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  389,  390,  391,  392, 
          393,  394,  395,  396,  397,  398,  399,    0,  400,  401, 
            0,    0,  402,  403,  620,    0,    0,    0,    0,    0, 
            0,    0,    0,    0, 
      };
   }

   private static final short[] yyTable2() {
      return new short[] {

            0,    0,    0,    0,  404,    0,  405,    0,  406,  407, 
          408,  409,  410,  411,  412,    0,  413,  620,  620,  620, 
            0,  620,  624,  624,  624,  620,  620,    0,  624,  624, 
          620,  624,  620,  620,  620,  620,  620,  620,  620,    0, 
          624,    0,    0,  620,  620,  620,  620,  620,  620,  620, 
          624,  624,  620,  624,  624,  624,  624,  624,    0,  620, 
            0,    0,  620,  620,  620,    0,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  624,  624,  624, 
          624,  624,  624,  624,  624,  624,  624,  624,  624,  624, 
          624,  414,    0,  624,  624,  624,  624,  620,    0,  105, 
          620,  620,  624,  620,  620,    0,  620,    0,  620,  624, 
          620,  624,  620,  624,  624,  624,  624,  624,  624,  624, 
          620,  624,  624,  624,    0,  620,  620,  620,  620,  620, 
          620,    0,    0,  105,  620,    0,  620,  620,    0,  620, 
          620,  620,  620,    0,    0,    0,    0,    0,    0,    0, 
          105,    0,  620,  620,  620,  624,  620,  620,  620,    0, 
          620,  620,  620,  620,  620,  620,    0,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,    0,  620, 
          624,    0,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,    0,  620,    0, 
            0,  620,  620,  620,    0,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
            0,    0,  620,  620,  620,  620,  620,    0,    0,  620, 
          620,  620,  620,  620,    0,  620,    0,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,    0,  620,  620,  620,  620,  620,  620, 
            0,    0,  621,  620,    0,  620,  620,    0,  620,  620, 
          620,  620,  621,    0,    0,   98,    0,    0,    0,    0, 
            0,  620,  620,  620,  620,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  106,    0,    0,   98, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  621,    0,  125,    0,  621,   98,    0,    0,    0, 
            0,    0,    0,    0,    0,  389,  390,  391,  392,  393, 
          394,  395,  396,  397,  398,  399,    0,  400,  401,    0, 
            0,  402,  403,  621,    0,    0,  521,    0,  105,  624, 
            0,    0,    0,    0,    0,    0,    0,  404,    0,  405, 
            0,  406,  407,  408,  409,  410,  411,  412,    0,  413, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  316,    0,    0,    0,    0,    0,  624,    0,    0, 
            0,  316,    0,    0,  624,  624,    0,    0,    0,    0, 
          620,  105,    0,  599,  600,    0,    0,  601,    0,    0, 
            0,  187,  188,    0,  189,  190,  191,  192,  193,  194, 
          195,  624,    0,  196,  197,  107,    0,    0,  198,  199, 
          200,  201,    0,    0,    0,    0,    0,    0,  303,    0, 
          316,    0,  126,    0,  316,  203,  204,    0,  205,  206, 
          207,  208,  209,  210,  211,  212,  213,  214,  215,    0, 
            0,  216,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  316,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  621, 
          621,  621,    0,  621,  621,  621,  621,  621,  621,    0, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,  621, 
          621,    0,  621,    0,   98,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,  621, 
            0,  621,    0,    0,  621,  621,  621,    0,  621,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,  621, 
          621,  621,  621,  521,    0,  621,  621,  621,  621,  621, 
          521,  521,  621,  621,  621,  621,  621,   98,  621,    0, 
          621,  621,  621,  621,  621,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,    0,  621,  621,  621, 
          621,  621,  621,    0,    0,    0,  621,    0,  621,  621, 
          646,  621,  621,  621,  621,    0,    0,    0,    0,    0, 
          646,    0,    0,    0,  621,  621,  621,  621,  621,  621, 
          621,    0,  621,  316,  316,  316,  621,  621,    0,  316, 
          316,  621,  316,  621,  621,  621,  621,  621,  621,  621, 
            0,    0,    0,    0,  621,  621,  621,  621,  621,  621, 
          621,  316,  316,  621,  316,  316,  316,  316,  316,  646, 
          621,    0,    0,  621,  621,  621,    0,  621,  621,  621, 
          621,  621,  621,  621,  621,  621,  621,  621,  316,  316, 
          316,  316,  316,  316,  316,  316,  316,  316,  316,  316, 
          316,  316,    0,    0,  316,  316,  316,  316,  621,    0, 
            0,  621,  621,  316,  621,  621,    0,  621,    0,  621, 
          316,  621,  316,  621,  316,  316,  316,  316,  316,  316, 
          316,  621,  316,    0,  316,    0,  621,  621,  621,  621, 
          621,  621,    0,    0,  437,  621,    0,  621,  621,    0, 
          621,  621,  621,  621,  437,    0,    0,    0,    0,    0, 
            0,    0,    0,  621,  621,  621,  316,    0,    0,  607, 
          608,    0,    0,  609,    0,    0,    0,  187,  188,    0, 
          189,  190,  191,  192,  193,  194,  195,    0,  437,  196, 
          197,    0,    0,    0,  198,  199,  200,  201,    0,    0, 
            0,    0,  437,  437,  303,    0,    0,  437,    0,    0, 
            0,  203,  204,    0,  205,  206,  207,  208,  209,  210, 
          211,  212,  213,  214,  215,    0,    0,  216,    0,    0, 
            0,    0,    0,    0,    0,  437,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   52, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   52, 
            0,    0,    0,    0,    0,    0,    0,  283,  283,  283, 
            0,  283,  646,  646,  646,  283,  283,  646,  646,  646, 
          283,  646,  283,  283,  283,  283,  283,  283,  283,    0, 
          646,  646,    0,  283,  283,  283,  283,  283,  283,  283, 
          646,  646,  283,  646,  646,  646,  646,  646,   52,  283, 
            0,    0,  283,  283,  283,    0,  283,  283,  283,  283, 
          283,  283,  283,  283,  283,  283,  283,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  646,    0, 
            0,    0,    0,    0,    0,  646,  646,  283,    0,    0, 
          283,    0,  646,  283,  283,    0,  283,    0,  283,    0, 
          283,    0,  283,    0,    0,    0,    0,    0,    0,    0, 
          283,    0,  646,  646,    0,  283,  283,  283,  283,  283, 
          283,    0,    0,    0,  283,  520,  283,  283,    0,  283, 
          283,  283,  283,    0,    0,  520,    0,    0,    0,    0, 
            0,    0,  283,  283,  283,  646,  437,  437,  437,    0, 
            0,  437,  437,  437,    0,  437,    0,    0,    0,    0, 
            0,    0,    0,    0,  437,  437,  437,    0,    0,  520, 
            0,    0,    0,    0,  437,  437,    0,  437,  437,  437, 
          437,  437,    0,  520,  520,    0,  116,    0,  520,  437, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  437,  437,  437,  437,  437,  437,  437,  437,  437, 
          437,  437,  437,  437,  437,    0,  520,  437,  437,  437, 
          437,    0,    0,  437,    0,    0,  437,    0,    0,  437, 
            0,  437,    0,  437,  519,  437,    0,  437,  437,  437, 
          437,  437,  437,  437,  519,  437,  437,  437,    0,    0, 
            0,   52,   52,   52,    0,    0,   52,   52,   52,  437, 
           52,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           52,    0,    0,    0,    0,    0,    0,    0,  519,  437, 
            0,    0,   52,   52,   52,   52,   52,    0,    0,    0, 
            0,    0,  519,  519,    0,  115,    0,  519,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  114,    0,    0, 
            0,    0,    0,    0,    0,  519,    0,    0,    0,  128, 
            0,   52,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  647,    0,    0,   99,    0,  101,    0, 
            0,  113,   52,  647,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  114,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  128,    0, 
           99,    0,  101,    0,   52,    0,    0,  520,  520,  520, 
            0,    0,  520,  520,  520,    0,  520,   99,  518,  101, 
          113,    0,  647,    0,    0,    0,  520,  520,    0,    0, 
          306,    0,    0,    0,    0,  520,  520,    0,  520,  520, 
          520,  520,  520,    0,    0,    0,    0,  522,    0,  524, 
          520,  521,  517,    0,    0,    0,    0,    0,    0,    0, 
            0,  521,  520,  520,  520,  520,  520,  520,  520,  520, 
          520,  520,  520,  520,  520,  520,    0,    0,  520,  520, 
          520,  520,    0,  532,  520,    0,    0,  520,    0,    0, 
          520,    0,  520,    0,  520,  521,  520,    0,  520,  520, 
          520,  520,  520,  520,  520,    0,  520,    0,  520,  521, 
          521,    0,  117,    0,  521,    0,  519,  519,  519,    0, 
          520,  519,  519,  519,    0,  519,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  519,  519,    0,    0,    0, 
          520,    0,  521,    0,  519,  519,    0,  519,  519,  519, 
          519,  519,    0,    0,    0,    0,    0,    0,    0,  519, 
          522,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          522,  519,  519,  519,  519,  519,  519,  519,  519,  519, 
          519,  519,  519,  519,  519,    0,    0,  519,  519,  519, 
          519,    0,  531,  519,  114,    0,  519,    0,    0,  519, 
            0,  519,    0,  519,  522,  519,  128,  519,  519,  519, 
          519,  519,  519,  519,    0,  519,    0,  519,  522,  522, 
            0,  118,    0,  522,    0,  647,  647,  647,  113,  519, 
          647,  647,  647,    0,  647,   99,    0,  101,    0,    0, 
            0,  618,    0,    0,  647,  647,    0,    0,    0,  519, 
            0,  522,    0,  647,  647,  518,  647,  647,  647,  647, 
          647,    0,  518,  518,    0,    0,    0,  306,    0,   60, 
            0,    0,    0,  100,  306,  306,    0,    0,  104,   60, 
            0,    0,  127,    0,  522,    0,  524,    0,    0,  517, 
            0,  522,  522,  524,  524,    0,  517,  517,   99,    0, 
          101,    0,  618,    0,    0,  647,    0,  100,  647,    0, 
          647,    0,  104,  521,  521,  521,    0,    0,  521,  521, 
          521,    0,  521,  618,  100,    0,  647,    0,   60,  104, 
            0,  127,  521,  521,    0,    0,    0,    0,    0,    0, 
            0,  521,  521,    0,  521,  521,  521,  521,  521,    0, 
            0,    0,    0,    0,  523,    0,  521,  524,  647,  620, 
            0,    0,    0,  317,    0,    0,    0,  524,  521,  521, 
          521,  521,  521,  521,  521,  521,  521,  521,  521,  521, 
          521,  521,    0,    0,  521,  521,  521,  521,    0,  533, 
          521,    0,    0,  521,    0,    0,  521,    0,  521,    0, 
          521,  524,  521,    0,  521,  521,  521,  521,  521,  521, 
          521,    0,  521,    0,  521,  524,  524,    0,  120,    0, 
          524,    0,  522,  522,  522,    0,  521,  522,  522,  522, 
            0,  522,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  522,  522,    0,    0,    0,  521,    0,  524,    0, 
          522,  522,    0,  522,  522,  522,  522,  522,    0,    0, 
            0,    0,    0,    0,    0,  522,  523,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  523,  522,  522,  522, 
          522,  522,  522,  522,  522,  522,  522,  522,  522,  522, 
          522,    0,    0,  522,  522,  522,  522,    0,  534,  522, 
            0,    0,  522,    0,    0,  522,    0,  522,    0,  522, 
          523,  522,    0,  522,  522,  522,  522,  522,  522,  522, 
            0,  522,    0,  522,  523,  523,    0,  119,    0,  523, 
            0,   60,   60,   60,    0,  522,   60,   60,   60,  127, 
           60,    0,  100,    0,    0,    0,    0,  104,  620,    0, 
           60,   60,    0,    0,    0,  522,    0,  523,    0,   60, 
           60,    0,   60,   60,   60,   60,   60,    0,    0,    0, 
            0,    0,    0,    0,    0,   64,    0,    0,    0,    0, 
            0,    0,    0,    0,  106,   64,    0,    0,    0,    0, 
            0,  523,    0,    0,    0,    0,  620,    0,  523,  523, 
          317,    0,    0,  620,  620,  100,    0,  317,  317,  620, 
          104,   60,    0,    0,   60,    0,    0,    0,  106,  524, 
          524,  524,    0,    0,  524,  524,  524,    0,  524,    0, 
          620,    0,   60,    0,   64,  106,    0,    0,  524,  524, 
            0,    0,    0,    0,    0,    0,    0,  524,  524,    0, 
          524,  524,  524,  524,  524,    0,    0,    0,    0,    0, 
            0,    0,  524,  525,   60,  621,    0,    0,    0,    0, 
            0,    0,    0,  525,  524,  524,  524,  524,  524,  524, 
          524,  524,  524,  524,  524,  524,  524,  524,    0,    0, 
          524,  524,  524,  524,    0,  536,  524,    0,    0,  524, 
            0,    0,  524,    0,  524,    0,  524,  525,  524,    0, 
          524,  524,  524,  524,  524,  524,  524,    0,  524,    0, 
          524,  525,  525,    0,  121,    0,  525,    0,  523,  523, 
          523,    0,  524,  523,  523,  523,    0,  523,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  523,  523,    0, 
            0,    0,  524,    0,  525,    0,  523,  523,    0,  523, 
          523,  523,  523,  523,    0,    0,    0,    0,    0,    0, 
            0,  523,  516,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  516,  523,  523,  523,  523,  523,  523,  523, 
          523,  523,  523,  523,  523,  523,  523,    0,    0,  523, 
          523,  523,  523,    0,  535,  523,    0,    0,  523,    0, 
            0,  523,    0,  523,    0,  523,  516,  523,    0,  523, 
          523,  523,  523,  523,  523,  523,    0,  523,    0,  523, 
          516,  516,    0,  112,    0,  516,    0,   64,   64,   64, 
            0,  523,   64,   64,   64,    0,   64,    0,    0,    0, 
            0,    0,    0,  106,  621,    0,   64,   64,    0,    0, 
            0,  523,    0,  516,    0,   64,   64,    0,   64,   64, 
           64,   64,   64,    0,    0,    0,    0,    0,    0,    0, 
            0,   62,    0,    0,  102,    0,    0,    0,    0,    0, 
            0,   62,    0,    0,    0,   93,    0,    0,    0,    0, 
            0,    0,  621,    0,    0,    0,    0,    0,    0,  621, 
          621,    0,    0,    0,    0,  621,  106,   64,  102,    0, 
           64,    0,    0,    0,    0,  525,  525,  525,    0,   93, 
          525,  525,  525,    0,  525,  102,  621,    0,   64,    0, 
           62,    0,    0,    0,  525,  525,   93,    0,    0,    0, 
            0,    0,    0,  525,  525,    0,  525,  525,  525,  525, 
          525,    0,    0,    0,    0,  525,    0,    0,  525,  515, 
           64,    0,    0,    0,    0,    0,  516,    0,    0,  515, 
          525,  525,  525,  525,  525,  525,  525,  525,  525,  525, 
          525,  525,  525,  525,    0,    0,  525,  525,  525,  525, 
            0,  537,  525,    0,    0,  525,    0,    0,  525,    0, 
          525,    0,  525,  515,  525,    0,  525,  525,  525,  525, 
          525,  525,  525,    0,  525,    0,  525,  515,  515,    0, 
          111,    0,  515,    0,  516,  516,  516,    0,  525,  516, 
          516,  516,    0,  516,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  516,  516,    0,    0,    0,  525,    0, 
          515,    0,  516,  516,    0,  516,  516,  516,  516,  516, 
            0,    0,    0,    0,    0,    0,    0,  516,  518,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  518,  516, 
          516,  516,  516,  516,  516,  516,  516,  516,  516,  516, 
          516,  516,  516,    0,    0,  516,  516,  516,  516,    0, 
          528,  516,    0,    0,  516,    0,    0,  516,    0,  516, 
            0,  516,  518,  516,    0,  516,  516,  516,  516,  516, 
          516,  516,    0,  516,    0,  516,  518,  518,    0,  114, 
            0,  518,    0,   62,   62,   62,    0,  516,   62,   62, 
           62,    0,   62,  102,    0,    0,    0,    0,    0,    0, 
            0,    0,   62,   62,   93,    0,    0,  516,    0,  518, 
            0,   62,   62,    0,   62,   62,   62,   62,   62,    0, 
            0,    0,    0,    0,    0,    0,    0,   53,   92,    0, 
           95,    0,    0,    0,    0,    0,    0,   53,    0,    0, 
            0,  109,  525,    0,    0,    0,    0,    0,    0,  525, 
          525,    0,    0,  516,    0,    0,  102,    0,    0,    0, 
          516,  516,   92,   62,   95,    0,   62,   93,    0,    0, 
            0,  515,  515,  515,    0,  109,  515,  515,  515,   92, 
          515,   95,    0,    0,   62,    0,   53,    0,    0,    0, 
          515,  515,  109,    0,    0,    0,    0,    0,    0,  515, 
          515,    0,  515,  515,  515,  515,  515,    0,    0,  515, 
            0,  518,    0,    0,  515,  646,   62,    0,    0,    0, 
            0,    0,  306,    0,    0,  646,  515,  515,  515,  515, 
          515,  515,  515,  515,  515,  515,  515,  515,  515,  515, 
            0,    0,  515,  515,  515,  515,    0,  527,  515,    0, 
            0,  515,    0,    0,  515,    0,  515,    0,  515,  646, 
          515,    0,  515,  515,  515,  515,  515,  515,  515,    0, 
          515,    0,  515,  646,  646,    0,    0,    0,  646,    0, 
          518,  518,  518,    0,  515,  518,  518,  518,    0,  518, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  518, 
          518,    0,    0,    0,  515,    0,  646,    0,  518,  518, 
            0,  518,  518,  518,  518,  518,    0,    0,    0,    0, 
            0,    0,    0,  518,  306,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  306,  518,  518,  518,  518,  518, 
          518,  518,  518,  518,  518,  518,  518,  518,  518,    0, 
            0,  518,  518,  518,  518,    0,  530,  518,    0,    0, 
          518,    0,    0,  518,    0,  518,    0,  518,  306,  518, 
            0,  518,  518,  518,  518,  518,  518,  518,    0,  518, 
            0,  518,  306,  306,    0,  128,    0,  306,    0,   53, 
           53,   53,    0,  518,   53,   53,   53,   92,   53,   95, 
            0,    0,    0,    0,    0,    0,    0,    0,   53,    0, 
          109,    0,    0,  518,    0,  306,    0,   53,   53,    0, 
           53,   53,   53,   53,   53,    0,    0,    0,    0,    0, 
            0,    0,    0,   55,    0,    0,  108,    0,    0,    0, 
            0,    0,    0,   55,    0,    0,  515,    0,  518,    0, 
            0,    0,    0,  515,  515,  518,  518,    0,    0,  306, 
           92,    0,   95,    0,    0,    0,  306,  306,    0,   53, 
          108,    0,    0,  109,    0,    0,    0,  646,  646,  646, 
            0,    0,  646,  646,  646,    0,  646,  108,    0,    0, 
           53,    0,   55,    0,    0,  646,  646,  646,    0,    0, 
            0,    0,    0,    0,    0,  646,  646,    0,  646,  646, 
          646,  646,  646,    0,    0,    0,    0,  317,    0,    0, 
          646,  317,   53,    0,    0,    0,    0,    0,    0,    0, 
            0,  317,  646,  646,  646,  646,  646,  646,  646,  646, 
          646,  646,  646,  646,  646,  646,    0,    0,  646,  646, 
          646,  646,    0,    0,  646,    0,    0,  646,    0,    0, 
          646,    0,  646,    0,  646,  317,  646,    0,  646,  646, 
          646,  646,  646,  646,  646,    0,  646,  646,  646,  317, 
          317,    0,  127,    0,  317,    0,  306,  306,  306,    0, 
          646,  306,  306,  306,    0,  306,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  306,  306,    0,    0,    0, 
          646,    0,  317,    0,  306,  306,    0,  306,  306,  306, 
          306,  306,    0,    0,    0,    0,    0,    0,    0,  306, 
          458,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          458,  306,  306,  306,  306,  306,  306,  306,  306,  306, 
          306,  306,  306,  306,  306,    0,    0,  306,  306,  306, 
          306,    0,    0,  306,    0,    0,  306,    0,    0,  306, 
            0,  306,    0,  306,  458,  306,    0,  306,  306,  306, 
          306,  306,  306,  306,    0,  306,    0,  306,  458,  458, 
            0,    0,    0,  458,    0,   55,   55,   55,    0,  306, 
           55,   55,   55,    0,   55,  108,    0,    0,    0,    0, 
            0,    0,    0,    0,   55,    0,    0,    0,    0,  306, 
            0,  458,    0,   55,   55,    0,   55,   55,   55,   55, 
           55,    0,    0,    0,    0,    0,    0,    0,    0,   56, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   56, 
            0,    0,  124,    0,  317,    0,    0,    0,    0,    0, 
            0,  317,  317,    0,    0,    0,    0,    0,  108,    0, 
            0,    0,    0,    0,    0,   55,    0,    0,    0,    0, 
            0,    0,    0,  317,  317,  317,    0,    0,  317,  317, 
          317,    0,  317,    0,    0,    0,   55,    0,   56,    0, 
            0,  124,  317,  317,    0,    0,    0,    0,    0,    0, 
            0,  317,  317,    0,  317,  317,  317,  317,  317,    0, 
            0,    0,    0,    0,    0,    0,  317,  328,   55,    0, 
            0,    0,    0,  624,    0,    0,    0,  328,  317,  317, 
          317,  317,  317,  317,  317,  317,  317,  317,  317,  317, 
          317,  317,    0,    0,  317,  317,  317,  317,    0,    0, 
          317,    0,    0,  317,    0,    0,  317,    0,  317,    0, 
          317,  328,  317,    0,  317,  317,  317,  317,  317,  317, 
          317,    0,  317,    0,  317,  328,  328,    0,    0,    0, 
          328,    0,  458,  458,  458,    0,  317,  458,  458,  458, 
            0,  458,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  458,  458,    0,    0,    0,  317,    0,  328,    0, 
          458,  458,    0,  458,  458,  458,  458,  458,    0,    0, 
            0,    0,    0,    0,    0,  458,  264,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  264,  458,  458,  458, 
          458,  458,  458,  458,  458,  458,  458,  458,  458,  458, 
          458,    0,    0,  458,  458,  458,  458,    0,    0,  458, 
            0,    0,  458,    0,    0,  458,    0,  458,    0,  458, 
          264,  458,    0,  458,  458,  458,  458,  458,  458,  458, 
            0,  458,    0,  458,  264,  264,    0,    0,    0,  264, 
            0,   56,   56,   56,    0,  458,   56,   56,   56,  124, 
           56,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           56,    0,  624,    0,    0,  458,    0,  360,    0,   56, 
           56,    0,   56,   56,   56,   56,   56,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          624,    0,    0,    0,    0,    0,    0,  624,  624,    0, 
            0,   56,    0,  620,    0,    0,    0,    0,    0,  328, 
          328,  328,    0,    0,  328,  328,  328,    0,  328,    0, 
            0,    0,   56,    0,  624,   43,    0,    0,  328,  328, 
            0,    0,    0,    0,    0,    0,    0,  328,  328,    0, 
          328,  328,  328,  328,  328,    0,    0,    0,    0,    0, 
            0,    0,  328,  323,   56,    0,    0,    0,    0,    0, 
            0,    0,    0,  323,  328,  328,  328,  328,  328,  328, 
          328,  328,  328,  328,  328,  328,  328,  328,    0,    0, 
          328,  328,  328,  328,    0,    0,  328,    0,    0,  328, 
            0,    0,  328,    0,  328,    0,  328,  323,  328,    0, 
          328,  328,  328,  328,  328,  328,  328,    0,  328,    0, 
          328,  323,  323,    0,    0,    0,  323,    0,  264,  264, 
          264,    0,  328,  264,  264,  264,    0,  264,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  264,  264,    0, 
            0,    0,  328,    0,  323,    0,  264,  264,    0,  264, 
          264,  264,  264,  264,    0,    0,    0,    0,    0,    0, 
            0,  264,  509,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  509,  264,  264,  264,  264,  264,  264,  264, 
          264,  264,  264,  264,  360,  264,  264,    0,    0,  264, 
          264,  360,  360,    0,    0,  264,    0,    0,  264,    0, 
            0,  264,    0,  264,    0,  264,  509,  264,    0,  264, 
          264,  264,  264,  264,  264,  264,    0,  264,    0,  264, 
          509,  509,    0,    0,    0,  509,    0,    0,   43,   43, 
           43,  264,    0,    0,   43,   43,    0,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  264,    0,  509,    0,    0,    0,    0,    0,   43, 
           43,   43,   43,   43,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  123,    0,  125,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  288,    0,    0,    0,    0,    0,    0,   43,    0, 
            0,  288,    0,    0,    0,  323,  323,  323,    0,    0, 
          323,  323,  323,    0,  323,    0,    0,    0,    0,   43, 
            0,  123,    0,  125,  323,  323,    0,    0,    0,    0, 
            0,    0,    0,  323,  323,  288,  323,  323,  323,  323, 
          323,    0,    0,    0,    0,    0,    0,    0,  323,  356, 
          288,   43,    0,  620,    0,  621,    0,    0,    0,  356, 
          323,  323,  323,  323,  323,  323,  323,  323,  323,  323, 
          323,  323,  323,  323,    0,    0,  323,  323,  323,  323, 
            0,    0,  323,    0,    0,  323,    0,    0,  323,    0, 
          323,    0,  323,  356,  323,    0,  323,  323,  323,  323, 
          323,  323,  323,    0,  323,    0,  323,    0,  356,    0, 
            0,    0,  356,    0,  509,  509,  509,    0,  323,  509, 
          509,  509,    0,  509,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  509,  509,    0,    0,    0,  323,    0, 
          356,    0,  509,  509,    0,  509,  509,  509,  509,  509, 
            0,    0,    0,    0,    0,    0,    0,  509,  357,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  357,    0, 
          509,  509,  509,  509,  509,  509,  509,  509,  509,  509, 
          509,  509,  509,    0,    0,  509,  509,  509,  509,    0, 
            0,  509,    0,    0,  509,    0,    0,  509,    0,  509, 
            0,  509,  357,  509,    0,  509,  509,  509,  509,  509, 
          509,  509,    0,  509,    0,  509,    0,  357,    0,    0, 
            0,  357,    0,    0,    0,    0,    0,  509,    0,  123, 
            0,  125,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  620,    0,  621,    0,    0,  509,    0,  357, 
            0,    0,    0,  288,  288,  288,    0,    0,  288,  288, 
          288,    0,  288,    0,  320,    0,    0,    0,    0,    0, 
            0,    0,  288,  288,  320,    0,    0,    0,    0,    0, 
            0,  288,  288,    0,  288,  288,  288,  288,  288,    0, 
          620,    0,  621,    0,    0,    0,    0,  620,  620,  621, 
          621,    0,    0,  620,    0,  621,    0,    0,  320,    0, 
            0,  356,  356,  356,    0,    0,  356,  356,  356,    0, 
          356,    0,    0,  320,  620,    0,  621,  320,    0,    0, 
          356,  356,    0,  288,    0,    0,  288,    0,  288,  356, 
          356,    0,  356,  356,  356,  356,  356,    0,    0,    0, 
            0,    0,    0,    0,  288,  320,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  356,  356,  356,  356, 
          356,  356,  356,  356,  356,  356,  356,  356,  356,  356, 
            0,    0,  356,  356,  356,  356,  288,  242,  356,    0, 
            0,  356,    0,    0,  356,    0,  356,  242,  356,    0, 
          356,    0,  356,  356,  356,  356,  356,  356,  356,    0, 
          356,    0,  356,    0,    0,    0,    0,    0,    0,    0, 
          357,  357,  357,    0,    0,  357,  357,  357,    0,  357, 
            0,  242,    0,    0,    0,    0,    0,    0,    0,  357, 
          357,    0,    0,    0,  356,  242,  242,    0,  357,  357, 
          242,  357,  357,  357,  357,  357,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  357,  357,  357,  357,  357, 
          357,  357,  357,  357,  357,  357,  357,  357,  357,    0, 
            0,  357,  357,  357,  357,    0,    0,  357,    0,    0, 
          357,    0,    0,  357,    0,  357,    0,  357,  239,  357, 
            0,  357,  357,  357,  357,  357,  357,  357,  239,  357, 
            0,  357,    0,    0,    0,    0,  320,  320,  320,    0, 
            0,  320,  320,  320,    0,  320,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  320,  320,    0,    0,    0, 
            0,    0,  239,  357,  320,  320,    0,  320,  320,  320, 
          320,  320,    0,    0,    0,    0,  239,  239,    0,    0, 
            0,  239,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  320,  320,  320,  320,  320,  320,  320,  320,  320, 
          320,  320,  320,  320,  320,    0,    0,  320,  320,  320, 
          320,    0,    0,  320,    0,    0,  320,    0,    0,  320, 
            0,  320,    0,  320,    0,  320,    0,  320,  320,  320, 
          320,  320,  320,  320,    0,  320,    0,  320,  236,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  236,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  242, 
          242,  242,    0,    0,  242,  242,  242,    0,  242,  320, 
            0,    0,    0,    0,    0,    0,    0,    0,  242,  242, 
            0,    0,  236,    0,    0,    0,    0,  242,  242,    0, 
          242,  242,  242,  242,  242,    0,  236,  236,    0,    0, 
            0,  236,  242,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  242,  242,  242,  242,  242, 
          242,  242,  242,  242,  242,    0,  242,  242,    0,    0, 
          242,  242,    0,    0,    0,    0,  242,    0,    0,  242, 
            0,    0,  242,    0,  242,    0,  242,  238,  242,    0, 
          242,  242,  242,  242,  242,  242,  242,  238,  242,    0, 
          242,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  242,    0,    0,    0,    0,    0,    0,    0, 
          239,  239,  239,    0,    0,  239,  239,  239,    0,  239, 
            0,  238,  242,    0,    0,    0,    0,    0,    0,  239, 
          239,    0,    0,    0,    0,  238,  238,    0,  239,  239, 
          238,  239,  239,  239,  239,  239,    0,    0,    0,    0, 
            0,    0,    0,  239,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  239,  239,  239,  239, 
          239,  239,  239,  239,  239,  239,    0,  239,  239,    0, 
            0,  239,  239,    0,    0,    0,    0,  239,    0,    0, 
          239,    0,    0,  239,    0,  239,    0,  239,    0,  239, 
            0,  239,  239,  239,  239,  239,  239,  239,    0,  239, 
            0,  239,  237,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  237,  239,    0,    0,    0,    0,    0,    0, 
          236,  236,  236,    0,    0,  236,  236,  236,    0,  236, 
            0,    0,    0,  239,    0,    0,    0,    0,    0,  236, 
          236,    0,    0,    0,    0,    0,  237,    0,  236,  236, 
            0,  236,  236,  236,  236,  236,    0,    0,    0,    0, 
          237,  237,    0,  236,    0,  237,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  236,  236,  236,  236, 
          236,  236,  236,  236,  236,  236,    0,  236,  236,    0, 
            0,  236,  236,    0,    0,    0,    0,  236,    0,    0, 
          236,    0,    0,  236,    0,  236,    0,  236,    0,  236, 
            0,  236,  236,  236,  236,  236,  236,  236,    0,  236, 
            0,  236,    0,    0,    0,    0,    0,    0,    0,  238, 
          238,  238,    0,  236,  238,  238,  238,    0,  238,    0, 
          240,    0,    0,    0,    0,    0,    0,    0,  238,  238, 
          240,    0,    0,  236,    0,    0,    0,  238,  238,    0, 
          238,  238,  238,  238,  238,    0,    0,    0,    0,    0, 
            0,    0,  238,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  240,  238,  238,  238,  238,  238, 
          238,  238,  238,  238,  238,    0,  238,  238,  240,  240, 
          238,  238,    0,  240,    0,    0,  238,    0,    0,  238, 
            0,    0,  238,    0,  238,    0,  238,    0,  238,    0, 
          238,  238,  238,  238,  238,  238,  238,    0,  238,    0, 
          238,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  238,  438,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  438,    0,    0,    0,    0,    0,    0, 
            0,    0,  238,    0,  237,  237,  237,    0,    0,  237, 
          237,  237,    0,  237,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  237,  237,    0,    0,  103,    0,    0, 
            0,    0,  237,  237,    0,  237,  237,  237,  237,  237, 
            0,    0,  438,    0,  122,    0,  438,  237,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          237,  237,  237,  237,  237,  237,  237,  237,  237,  237, 
            0,  237,  237,    0,  438,  237,  237,    0,    0,    0, 
            0,  237,    0,    0,  237,    0,    0,  237,    0,  237, 
            0,  237,    0,  237,    0,  237,  237,  237,  237,  237, 
          237,  237,    0,  237,    0,  237,  520,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  520,  237,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  237,    0,    0, 
            0,    0,  240,  240,  240,    0,    0,  240,  240,  240, 
           97,  240,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  240,  240,    0,    0,  520,    0,  116,  307,  520, 
          240,  240,    0,  240,  240,  240,  240,  240,    0,    0, 
            0,    0,    0,    0,    0,  240,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  520,  240,  240, 
          240,  240,  240,  240,  240,  240,  240,  240,    0,  240, 
          240,    0,    0,  240,  240,    0,    0,    0,    0,  240, 
            0,  414,  240,    0,    0,  240,    0,  240,    0,  240, 
            0,  240,    0,  240,  240,  240,  240,  240,  240,  240, 
            0,  240,    0,  240,    0,  438,  438,  438,    0,    0, 
            0,  438,  438,    0,  438,  240,    0,    0,    0,    0, 
            0,    0,    0,  438,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  438,  438,  240,  438,  438,  438,  438, 
          438,    0,    0,    0,    0,    0,    0,    0,    0,  519, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  519, 
          438,  438,  438,  438,  438,  438,  438,  438,  438,  438, 
          438,  438,  438,  438,    0,    0,  438,  438,  438,  438, 
            0,    0,    0,    0,    0,  438,    0,    0,    0,    0, 
            0,    0,  438,   96,  438,    0,  438,  438,  438,  438, 
          438,  438,  438,    0,  438,  438,  438,    0,  519,    0, 
          115,    0,  519,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  520,  520, 
          520,    0,    0,    0,  520,  520,    0,  520,  438,    0, 
          519,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  520,  520,    0,  520, 
          520,  520,  520,  520,    0,    0,    0,  521,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  521,    0,    0, 
            0,    0,    0,  520,  520,  520,  520,  520,  520,  520, 
          520,  520,  520,  520,  520,  520,  520,    0,    0,  520, 
          520,  520,  520,    0,  532,    0,    0,    0,  520,    0, 
            0,   98,    0,    0,    0,  520,    0,  520,    0,  520, 
          520,  520,  520,  520,  520,  520,  521,  520,  117,  520, 
          521,    0,    0,    0,    0,  389,  390,  391,  392,  393, 
          394,  395,  396,  397,  398,  399,    0,  400,  401,    0, 
            0,  402,  403,    0,    0,    0,    0,    0,  521,    0, 
            0,  520,    0,    0,    0,    0,    0,  404,    0,  405, 
          522,  406,  407,  408,  409,  410,  411,  412,    0,  413, 
          522,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  519,  519,  519,   99,    0,    0,  519,  519,    0, 
          519,    0,    0,    0,    0,    0,    0,    0,    0,  522, 
            0,  118,    0,  522,    0,    0,    0,    0,    0,  519, 
          519,    0,  519,  519,  519,  519,  519,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  522,    0,    0,    0,    0,  519,  519,  519,  519, 
          519,  519,  519,  519,  519,  519,  519,  519,  519,  519, 
          524,    0,  519,  519,  519,  519,    0,  531,    0,    0, 
          524,  519,    0,    0,    0,    0,    0,    0,  519,    0, 
          519,    0,  519,  519,  519,  519,  519,  519,  519,    0, 
          519,    0,  519,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  101,    0,    0,    0,    0,  521, 
          521,  521,    0,    0,    0,  521,  521,    0,  521,  524, 
            0,  120,    0,  524,  519,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  521,  521,    0, 
          521,  521,  521,  521,  521,    0,    0,    0,    0,    0, 
            0,  524,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  521,  521,  521,  521,  521,  521, 
          521,  521,  521,  521,  521,  521,  521,  521,    0,    0, 
          521,  521,  521,  521,    0,  533,    0,    0,    0,  521, 
            0,    0,    0,    0,    0,    0,  521,    0,  521,    0, 
          521,  521,  521,  521,  521,  521,  521,    0,  521,    0, 
          521,    0,  522,  522,  522,    0,    0,    0,  522,  522, 
            0,  522,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          522,  522,  521,  522,  522,  522,  522,  522,    0,    0, 
            0,  523,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  523,    0,    0,    0,    0,    0,  522,  522,  522, 
          522,  522,  522,  522,  522,  522,  522,  522,  522,  522, 
          522,    0,    0,  522,  522,  522,  522,    0,  534,    0, 
            0,    0,  522,    0,    0,  100,    0,    0,    0,  522, 
            0,  522,    0,  522,  522,  522,  522,  522,  522,  522, 
          523,  522,  119,  522,  523,    0,    0,    0,    0,    0, 
            0,    0,  524,  524,  524,    0,    0,    0,  524,  524, 
            0,  524,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  523,    0,    0,  522,    0,    0,    0,    0, 
          524,  524,    0,  524,  524,  524,  524,  524,    0,    0, 
            0,  525,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  525,    0,    0,    0,    0,    0,  524,  524,  524, 
          524,  524,  524,  524,  524,  524,  524,  524,  524,  524, 
          524,    0,    0,  524,  524,  524,  524,    0,  536,    0, 
            0,    0,  524,    0,    0,  102,    0,    0,    0,  524, 
            0,  524,    0,  524,  524,  524,  524,  524,  524,  524, 
          525,  524,  121,  524,  525, 
      };
   }

   private static final short[] yyTable3() {
      return new short[] {

            0,    0,    0,    0,    0,    0,    0,    0,  642,  600, 
            0,    0,  643,    0,    0,    0,  187,  188,    0,  189, 
          190,  191,  192,  193,  194,  195,    0,  525,  196,  197, 
          524,    0,    0,  198,  199,  200,  201,    0,    0,  516, 
            0,    0,    0,  303,    0,    0,    0,    0,    0,  516, 
          203,  204,    0,  205,  206,  207,  208,  209,  210,  211, 
          212,  213,  214,  215,    0,    0,  216,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,   93,    0,    0,    0,    0,  523,  523, 
          523,    0,    0,    0,  523,  523,    0,  523,  516,    0, 
          112,    0,  516,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  523,  523,    0,  523, 
          523,  523,  523,  523,    0,    0,    0,    0,    0,    0, 
          516,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  523,  523,  523,  523,  523,  523,  523, 
          523,  523,  523,  523,  523,  523,  523,  515,    0,  523, 
          523,  523,  523,    0,  535,    0,    0,  515,  523,    0, 
            0,    0,    0,    0,    0,  523,    0,  523,    0,  523, 
          523,  523,  523,  523,  523,  523,    0,  523,    0,  523, 
            0,    0,    0,    0,    0,    0,    0,    0,  525,  525, 
          525,   92,    0,    0,  525,  525,    0,  525,    0,    0, 
            0,    0,    0,    0,    0,    0,  515,    0,  111,    0, 
          515,  523,    0,    0,    0,    0,  525,  525,    0,  525, 
          525,  525,  525,  525,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  515,    0, 
            0,    0,    0,  525,  525,  525,  525,  525,  525,  525, 
          525,  525,  525,  525,  525,  525,  525,    0,    0,  525, 
          525,  525,  525,    0,  537,    0,    0,    0,  525,    0, 
            0,    0,    0,    0,    0,  525,    0,  525,    0,  525, 
          525,  525,  525,  525,  525,  525,    0,  525,    0,  525, 
            0,  516,  516,  516,    0,    0,    0,  516,  516,    0, 
          516,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  516, 
          516,  525,  516,  516,  516,  516,  516,    0,    0,    0, 
          518,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          518,    0,    0,    0,    0,    0,  516,  516,  516,  516, 
          516,  516,  516,  516,  516,  516,  516,  516,  516,  516, 
            0,    0,  516,  516,  516,  516,    0,  528,    0,    0, 
            0,  516,    0,    0,   95,    0,    0,    0,  516,    0, 
          516,    0,  516,  516,  516,  516,  516,  516,  516,  518, 
          516,  114,  516,  518,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  515, 
          515,  515,    0,    0,    0,  515,  515,    0,  515,    0, 
            0,  518,    0,    0,  516,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  515,  515,    0, 
          515,  515,  515,  515,  515,    0,    0,    0,  306,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  306,    0, 
            0,    0,    0,    0,  515,  515,  515,  515,  515,  515, 
          515,  515,  515,  515,  515,  515,  515,  515,    0,    0, 
          515,  515,  515,  515,    0,  527,    0,    0,    0,  515, 
            0,    0,  109,    0,    0,    0,  515,    0,  515,    0, 
          515,  515,  515,  515,  515,  515,  515,  306,  515,  128, 
          515,  306,    0,    0,    0,    0,    0,    0,    0,    0, 
          613,  608,    0,    0,  614,    0,    0,    0,  187,  188, 
            0,  189,  190,  191,  192,  193,  194,  195,    0,  306, 
          196,  197,  515,    0,    0,  198,  199,  200,  201,    0, 
            0,  317,    0,    0,    0,  303,    0,    0,    0,    0, 
            0,  317,  203,  204,    0,  205,  206,  207,  208,  209, 
          210,  211,  212,  213,  214,  215,    0,    0,  216,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  518,  518,  518,  108,    0,    0,  518,  518, 
            0,  518,    0,    0,    0,    0,    0,    0,    0,    0, 
          317,    0,  127,    0,  317,    0,    0,    0,    0,    0, 
          518,  518,    0,  518,  518,  518,  518,  518,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  317,    0,    0,    0,    0,  518,  518,  518, 
          518,  518,  518,  518,  518,  518,  518,  518,  518,  518, 
          518,    0,    0,  518,  518,  518,  518,    0,  530,    0, 
          234,    0,  518,    0,    0,    0,    0,    0,    0,  518, 
          234,  518,    0,  518,  518,  518,  518,  518,  518,  518, 
            0,  518,    0,  518,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          306,  306,  306,    0,  234,    0,  306,  306,    0,  306, 
            0,    0,    0,    0,    0,  518,    0,    0,  234,  234, 
            0,    0,    0,  234,    0,    0,    0,    0,  306,  306, 
            0,  306,  306,  306,  306,  306,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  306,  306,  306,  306,  306, 
          306,  306,  306,  306,  306,  306,  306,  306,  306,    0, 
            0,  306,  306,  306,  306,    0,    0,    0,    0,    0, 
          306,    0,    0,    0,    0,    0,    0,  306,    0,  306, 
            0,  306,  306,  306,  306,  306,  306,  306,    0,  306, 
            0,  306,    0,  317,  317,  317,    0,    0,    0,  317, 
          317,    0,  317,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  317,  317,  306,  317,  317,  317,  317,  317,    0, 
            0,    0,    0,    0,    0,    0,    0,  235,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  235,  317,  317, 
          317,  317,  317,  317,  317,  317,  317,  317,  317,  317, 
          317,  317,    0,    0,  317,  317,  317,  317,    0,    0, 
            0,    0,    0,  317,    0,    0,    0,    0,    0,    0, 
          317,  235,  317,    0,  317,  317,  317,  317,  317,  317, 
          317,    0,  317,    0,  317,  235,  235,    0,    0,    0, 
          235,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  234,  234,  234,    0,    0,  234,  234,  234, 
            0,  234,    0,    0,    0,    0,  317,    0,    0,    0, 
            0,  234,  234,    0,    0,    0,    0,    0,    0,    0, 
          234,  234,    0,  234,  234,  234,  234,  234,    0,    0, 
            0,    0,    0,    0,    0,  234,  258,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  258,    0,  234,  234, 
          234,  234,  234,  234,  234,  234,  234,  234,    0,  234, 
          234,    0,    0,  234,  234,    0,    0,    0,    0,  234, 
            0,    0,  234,    0,    0,  234,    0,  234,    0,    0, 
          258,  234,    0,    0,    0,  234,  234,  234,  234,  234, 
            0,  234,    0,  234,  258,  258,    0,    0,    0,  258, 
            0,    0,    0,    0,    0,  234,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  660,  600,    0,    0,  661, 
            0,    0,    0,  187,  188,  234,  189,  190,  191,  192, 
          193,  194,  195,    0,    0,  196,  197,    0,    0,    0, 
          198,  199,  200,  201,    0,    0,  259,    0,    0,    0, 
          303,    0,    0,    0,    0,    0,  259,  203,  204,    0, 
          205,  206,  207,  208,  209,  210,  211,  212,  213,  214, 
          215,    0,    0,  216,    0,    0,    0,    0,    0,  235, 
          235,  235,    0,    0,  235,  235,  235,    0,  235,    0, 
          259,    0,    0,    0,    0,    0,    0,    0,  235,  235, 
            0,    0,    0,    0,  259,  259,    0,  235,  235,  259, 
          235,  235,  235,  235,  235,    0,    0,    0,    0,    0, 
            0,    0,  235,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  235,  235,  235,  235,  235, 
          235,  235,  235,  235,  235,    0,  235,  235,    0,    0, 
          235,  235,    0,    0,    0,    0,  235,    0,    0,  235, 
            0,    0,  235,    0,  235,    0,    0,    0,  235,    0, 
            0,    0,  235,  235,  235,  235,  235,    0,  235,    0, 
          235,  245,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  245,  235,    0,    0,    0,    0,    0,  258,  258, 
          258,    0,    0,  258,  258,  258,    0,  258,    0,    0, 
            0,    0,  235,    0,    0,    0,    0,  258,  258,    0, 
            0,    0,    0,    0,    0,  245,  258,  258,    0,  258, 
          258,  258,  258,  258,    0,    0,    0,    0,    0,  245, 
          245,  258,    0,    0,  245,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  258,  258,  258,  258,  258,  258, 
          258,  258,  258,  258,    0,  258,  258,    0,    0,  258, 
          258,    0,    0,    0,    0,  258,    0,    0,  258,    0, 
            0,  258,    0,  258,    0,    0,    0,  258,    0,    0, 
          243,    0,    0,  258,  258,  258,    0,  258,    0,  258, 
          243,    0,    0,    0,    0,    0,    0,    0,  259,  259, 
          259,  258,    0,  259,  259,  259,    0,  259,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  259,  259,    0, 
            0,  258,    0,    0,  243,    0,  259,  259,    0,  259, 
          259,  259,  259,  259,    0,    0,    0,    0,  243,  243, 
            0,  259,    0,  243,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  259,  259,  259,  259,  259,  259, 
          259,  259,  259,  259,    0,  259,  259,    0,    0,  259, 
          259,    0,    0,    0,    0,  259,    0,    0,  259,    0, 
            0,  259,    0,  259,    0,    0,    0,  259,    0,    0, 
            0,    0,    0,  259,  259,  259,    0,  259,    0,  259, 
          244,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          244,  259,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  259,    0,  245,  245,  245,    0,    0,  245,  245, 
          245,    0,  245,    0,  244,    0,    0,    0,    0,    0, 
            0,    0,  245,  245,    0,    0,    0,    0,  244,  244, 
            0,  245,  245,  244,  245,  245,  245,  245,  245,    0, 
            0,    0,    0,    0,    0,    0,  245,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  245, 
          245,  245,  245,  245,  245,  245,  245,  245,  245,    0, 
          245,  245,    0,    0,    0,    0,    0,    0,    0,  248, 
          245,    0,    0,  245,    0,    0,  245,    0,  245,  248, 
            0,    0,  245,    0,    0,    0,    0,    0,  245,  245, 
          245,    0,  245,    0,  245,    0,    0,    0,    0,    0, 
            0,    0,  243,  243,  243,    0,  245,  243,  243,  243, 
            0,  243,    0,  248,    0,    0,    0,    0,    0,    0, 
            0,  243,  243,    0,    0,    0,  245,  248,  248,    0, 
          243,  243,  248,  243,  243,  243,  243,  243,    0,    0, 
            0,    0,    0,    0,    0,  243,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  243,  243, 
          243,  243,  243,  243,  243,  243,  243,  243,    0,  243, 
          243,    0,    0,    0,    0,    0,    0,    0,    0,  243, 
            0,    0,  243,    0,    0,  243,  250,  243,    0,    0, 
            0,    0,    0,    0,    0,    0,  250,  243,  243,  243, 
            0,  243,    0,  243,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  243,    0,    0,    0,    0, 
            0,    0,  244,  244,  244,    0,    0,  244,  244,  244, 
          250,  244,    0,    0,    0,  243,    0,    0,    0,    0, 
            0,  244,  244,    0,  250,  250,    0,    0,    0,  250, 
          244,  244,    0,  244,  244,  244,  244,  244,    0,    0, 
            0,    0,    0,    0,    0,  244,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  244,  244, 
          244,  244,  244,  244,  244,  244,  244,  244,    0,  244, 
          244,    0,    0,    0,    0,    0,    0,    0,    0,  244, 
            0,    0,  244,    0,    0,  244,    0,  244,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  244,  244,  244, 
            0,  244,    0,  244,    0,    0,    0,    0,    0,    0, 
            0,  248,  248,  248,    0,  244,  248,  248,  248,    0, 
          248,    0,  249,    0,    0,    0,    0,    0,    0,    0, 
          248,  248,  249,    0,    0,  244,    0,    0,    0,  248, 
          248,    0,  248,  248,  248,  248,  248,    0,    0,    0, 
            0,    0,    0,    0,  248,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  249,  248,  248,  248, 
          248,  248,  248,  248,  248,  248,  248,    0,  248,  248, 
          249,  249,    0,    0,    0,  249,    0,    0,  248,    0, 
            0,  248,    0,    0,  248,    0,  248,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  248,  248,    0,    0, 
            0,    0,  248,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  248,    0,    0,    0,  250,  250, 
          250,    0,    0,  250,  250,  250,    0,  250,    0,  247, 
            0,    0,    0,    0,  248,    0,    0,  250,  250,  247, 
            0,    0,    0,    0,    0,    0,  250,  250,    0,  250, 
          250,  250,  250,  250,    0,    0,    0,    0,    0,    0, 
            0,  250,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  247,  250,  250,  250,  250,  250,  250, 
          250,  250,  250,  250,    0,  250,  250,  247,  247,    0, 
            0,    0,  247,    0,    0,  250,    0,    0,  250,    0, 
            0,  250,    0,  250,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  250,  250,    0,    0,    0,    0,  250, 
            0,    0,    0,    0,    0,    0,  251,    0,    0,    0, 
            0,  250,    0,    0,    0,    0,  251,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  250,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          251,    0,    0,    0,  249,  249,  249,    0,    0,  249, 
          249,  249,    0,  249,  251,  251,    0,    0,    0,  251, 
            0,    0,    0,  249,  249,    0,    0,    0,    0,    0, 
            0,    0,  249,  249,    0,  249,  249,  249,  249,  249, 
            0,    0,    0,    0,  252,    0,    0,  249,    0,    0, 
            0,    0,    0,    0,  252,    0,    0,    0,    0,    0, 
          249,  249,  249,  249,  249,  249,  249,  249,  249,  249, 
            0,  249,  249,    0,    0,    0,    0,    0,    0,    0, 
            0,  249,    0,    0,  249,    0,    0,  249,  252,  249, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  249, 
          249,    0,  252,  252,    0,  249,    0,  252,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  249,    0,    0, 
            0,  247,  247,  247,    0,    0,  247,  247,  247,    0, 
          247,    0,  253,    0,    0,    0,    0,  249,    0,    0, 
          247,  247,  253,    0,    0,    0,    0,    0,    0,  247, 
          247,    0,  247,  247,  247,  247,  247,    0,    0,    0, 
            0,    0,    0,    0,  247,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  253,  247,  247,  247, 
          247,  247,  247,  247,  247,  247,  247,    0,  247,  247, 
          253,  253,    0,    0,    0,  253,    0,    0,  247,    0, 
            0,  247,    0,    0,  247,    0,  247,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  247,  247,  251,  251, 
          251,    0,  247,  251,  251,  251,    0,  251,    0,  260, 
            0,    0,    0,    0,  247,    0,    0,  251,  251,  260, 
            0,    0,    0,    0,    0,    0,  251,  251,    0,  251, 
          251,  251,  251,  251,  247,    0,    0,    0,    0,    0, 
            0,  251,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  260,    0,    0,    0,    0,    0,    0, 
          251,  251,    0,    0,    0,  251,  251,  260,  260,    0, 
            0,    0,  260,    0,    0,  251,    0,    0,  251,    0, 
            0,  251,    0,  251,    0,    0,  252,  252,  252,    0, 
            0,  252,  252,  252,    0,  252,    0,  254,    0,  251, 
            0,    0,    0,    0,    0,  252,  252,  254,    0,    0, 
            0,  251,    0,    0,  252,  252,    0,  252,  252,  252, 
          252,  252,    0,    0,    0,    0,    0,    0,    0,  252, 
            0,  251,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  254,    0,    0,    0,    0,    0,    0,  252,  252, 
            0,    0,    0,  252,  252,  254,  254,    0,    0,    0, 
          254,    0,    0,  252,    0,    0,  252,    0,    0,  252, 
            0,  252,    0,    0,  253,  253,  253,    0,    0,  253, 
          253,  253,    0,  253,    0,  255,    0,  252,    0,    0, 
            0,    0,    0,  253,  253,  255,    0,    0,    0,  252, 
            0,    0,  253,  253,    0,  253,  253,  253,  253,  253, 
            0,    0,    0,    0,    0,    0,    0,  253,    0,  252, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  255, 
            0,    0,    0,    0,    0,    0,  253,  253,    0,    0, 
            0,  253,  253,  255,  255,    0,    0,    0,  255,    0, 
            0,  253,    0,    0,  253,    0,    0,  253,    0,  253, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  260,  260,  260,    0,  253,  260,  260,  260,    0, 
          260,    0,  261,    0,    0,    0,    0,  253,    0,    0, 
          260,  260,  261,    0,    0,    0,    0,    0,    0,  260, 
          260,    0,  260,  260,  260,  260,  260,  253,    0,    0, 
            0,    0,    0,    0,  260,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  261,    0,    0,    0, 
            0,    0,    0,  260,  260,    0,    0,    0,  260,  260, 
          261,  261,    0,    0,    0,  261,    0,    0,  260,    0, 
            0,  260,    0,    0,  260,    0,  260,    0,    0,  254, 
          254,  254,    0,    0,  254,  254,  254,    0,  254,    0, 
          232,    0,  260,    0,    0,    0,    0,    0,  254,  254, 
          232,    0,    0,    0,  260,    0,    0,  254,  254,    0, 
          254,  254,  254,  254,  254,    0,    0,    0,    0,    0, 
            0,    0,  254,    0,  260,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  232,    0,    0,    0,    0,    0, 
            0,  254,  254,    0,    0,    0,  254,  254,  232,  232, 
            0,    0,    0,  232,    0,    0,  254,    0,    0,  254, 
            0,    0,  254,    0,  254,    0,    0,  255,  255,  255, 
            0,    0,  255,  255,  255,    0,  255,    0,  233,    0, 
          254,    0,    0,    0,    0,    0,  255,  255,  233,    0, 
            0,    0,  254,    0,    0,  255,  255,    0,  255,  255, 
          255,  255,  255,    0,    0,    0,    0,    0,    0,    0, 
          255,    0,  254,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  233,    0,    0,    0,    0,    0,    0,  255, 
          255,    0,    0,    0,  255,  255,  233,  233,    0,    0, 
            0,  233,    0,    0,  255,    0,    0,  255,    0,    0, 
          255,    0,  255,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  261,  261,  261,  231,  255,  261, 
          261,  261,    0,  261,    0,    0,    0,  231,    0,    0, 
          255,    0,    0,  261,  261,    0,    0,    0,    0,    0, 
            0,    0,  261,  261,    0,  261,  261,  261,  261,  261, 
          255,    0,    0,    0,    0,    0,    0,  261,    0,    0, 
            0,  231,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  231,  231,  261,    0,    0, 
            0,  261,  261,    0,    0,    0,    0,    0,    0,    0, 
            0,  261,    0,    0,  261,    0,    0,  261,    0,  261, 
            0,    0,  232,  232,  232,    0,  230,  232,  232,  232, 
            0,  232,    0,    0,    0,  261,  230,    0,    0,    0, 
            0,  232,  232,    0,    0,    0,    0,  261,    0,    0, 
          232,  232,    0,  232,  232,  232,  232,  232,    0,    0, 
            0,    0,    0,    0,    0,  232,    0,  261,    0,    0, 
          230,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  230,  230,    0,    0,  228,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  228,  232, 
            0,    0,  232,    0,    0,  232,    0,  232,    0,    0, 
          233,  233,  233,    0,    0,  233,  233,  233,    0,  233, 
            0,    0,    0,  232,    0,    0,    0,    0,    0,  233, 
          233,    0,  228,    0,    0,  232,    0,    0,  233,  233, 
            0,  233,  233,  233,  233,  233,  228,  228,    0,    0, 
            0,    0,    0,  233,    0,  232,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  233,    0,    0, 
          233,    0,    0,  233,    0,  233,    0,  229,    0,  231, 
          231,  231,    0,    0,  231,  231,  231,  229,  231,    0, 
            0,  233,    0,    0,    0,    0,    0,    0,  231,  231, 
            0,    0,    0,  233,    0,    0,    0,  231,  231,    0, 
          231,  231,  231,  231,  231,    0,    0,    0,    0,    0, 
            0,  229,  231,  233,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  229,  229,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  231,    0,    0,  231, 
            0,    0,  231,    0,  231,    0,  226,    0,  230,  230, 
          230,    0,    0,  230,  230,  230,  226,  230,    0,    0, 
          231,    0,    0,    0,    0,    0,    0,  230,  230,    0, 
            0,    0,  231,    0,    0,    0,  230,  230,    0,  230, 
          230,  230,  230,  230,    0,    0,    0,    0,    0,    0, 
          226,  230,  231,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  226,  226,    0,    0,  227,    0, 
          228,  228,  228,    0,    0,  228,  228,  228,  227,  228, 
            0,    0,    0,    0,    0,  230,    0,    0,  230,  228, 
          228,  230,    0,  230,    0,    0,    0,    0,  228,  228, 
            0,  228,  228,  228,  228,  228,    0,    0,    0,  230, 
            0,    0,  227,  228,    0,    0,    0,    0,    0,    0, 
            0,  230,    0,    0,    0,    0,  227,  227,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  230,    0,    0,    0,    0,    0,  228,    0,    0, 
          228,    0,    0,  228,    0,  228,    0,  222,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  222,    0,    0, 
            0,  228,    0,    0,    0,    0,    0,    0,    0,  229, 
          229,  229,    0,  228,  229,  229,  229,    0,  229,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  229,  229, 
            0,  222,    0,  228,    0,    0,    0,  229,  229,    0, 
          229,  229,  229,  229,  229,  222,  222,    0,    0,    0, 
            0,    0,  229,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  229,    0,    0,  229, 
            0,    0,  229,    0,  229,    0,  224,    0,  226,  226, 
          226,    0,    0,  226,  226,  226,  224,  226,    0,    0, 
          229,    0,    0,    0,    0,    0,    0,  226,  226,    0, 
            0,    0,  229,    0,    0,    0,  226,  226,    0,  226, 
          226,  226,  226,  226,    0,    0,    0,    0,    0,    0, 
          224,  226,  229,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  224,  224,    0,    0,  263,    0, 
          227,  227,  227,    0,    0,  227,  227,  227,  263,  227, 
            0,    0,    0,    0,    0,  226,    0,    0,  226,  227, 
          227,  226,    0,  226,    0,    0,    0,    0,  227,  227, 
            0,  227,  227,  227,  227,  227,    0,    0,    0,  226, 
            0,    0,  263,  227,    0,    0,    0,    0,    0,    0, 
            0,  226,    0,    0,    0,    0,  263,  263,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  226,    0,    0,    0,    0,    0,  227,    0,    0, 
          227,    0,    0,  227,    0,  227,    0,  225,    0,  222, 
          222,  222,    0,    0,  222,  222,  222,  225,  222,    0, 
            0,  227,    0,    0,    0,    0,    0,    0,  222,  222, 
            0,    0,    0,  227,    0,    0,    0,  222,  222,    0, 
          222,  222,  222,  222,  222,    0,    0,    0,    0,    0, 
            0,  225,  222,  227,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  225,  225,    0,    0,  265, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  265, 
            0,    0,    0,    0,    0,    0,  222,    0,    0,  222, 
            0,    0,  222,    0,  222,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          222,    0,    0,  265,    0,    0,    0,    0,  224,  224, 
          224,    0,  222,  224,  224,  224,    0,  224,  265,    0, 
            0,    0,    0,    0,    0,    0,    0,  224,  224,    0, 
            0,    0,  222,    0,    0,    0,  224,  224,    0,  224, 
          224,  224,  224,  224,    0,    0,    0,    0,  223,    0, 
            0,  224,    0,    0,    0,    0,    0,    0,  223,    0, 
            0,    0,    0,    0,  296,    0,    0,    0,    0,    0, 
          263,  263,  263,    0,  296,  263,  263,  263,    0,  263, 
            0,    0,    0,    0,    0,  224,    0,    0,  224,  263, 
          263,  224,  223,  224,    0,    0,    0,    0,  263,  263, 
            0,  263,  263,  263,  263,  263,  223,  223,  289,  224, 
            0,    0,    0,  263,    0,    0,    0,    0,    0,    0, 
            0,  224,    0,  296,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  224,    0,    0,    0,    0,    0,  263,    0,    0, 
          263,    0,    0,  263,    0,  263,    0,  221,    0,  225, 
          225,  225,    0,    0,  225,  225,  225,  221,  225,    0, 
            0,  263,    0,    0,    0,    0,    0,    0,  225,  225, 
            0,    0,    0,  263,    0,    0,    0,  225,  225,    0, 
          225,  225,  225,  225,  225,    0,    0,    0,    0,    0, 
            0,  221,  225,  263,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  221,  221,    0,    0,  290, 
            0,  265,  265,  265,    0,    0,  265,  265,  265,  290, 
          265,    0,    0,    0,    0,    0,  225,    0,    0,  225, 
          265,  265,  225,    0,  225,    0,    0,    0,    0,  265, 
          265,    0,  265,  265,  265,  265,  265,    0,    0,    0, 
          225,    0,    0,  290,  265,    0,    0,    0,    0,    0, 
            0,    0,  225,    0,    0,    0,    0,  294,  290,    0, 
            0,    0,    0,    0,    0,    0,    0,  294,    0,    0, 
            0,    0,  225,    0,    0,    0,    0,    0,  265,    0, 
            0,  265,    0,    0,  265,    0,  265,    0,    0,    0, 
          223,  223,  223,    0,    0,  223,  223,  223,    0,  223, 
            0,  290,  265,    0,    0,    0,  296,  296,  296,  223, 
          223,  296,  296,  296,  265,  296,  294,  295,  223,  223, 
            0,  223,  223,  223,  223,    0,    0,  295,    0,    0, 
            0,    0,    0,  223,  265,    0,    0,  296,  296,  296, 
          296,  296,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  221,    0,    0,    0,    0,    0,    0, 
            0,  291,    0,  221,    0,    0,    0,  223,    0,    0, 
          223,    0,    0,  223,    0,  223,  295,    0,    0,    0, 
            0,    0,    0,  296,    0,    0,  296,    0,    0,    0, 
            0,  223,    0,    0,    0,    0,    0,  265,    0,  221, 
          221,  221,    0,  223,  221,  221,  221,  296,  221,    0, 
           25,    0,  221,    0,    0,    0,    0,    0,  221,  221, 
           25,    0,    0,  223,    0,    0,    0,  221,  221,    0, 
          221,  221,  221,  221,    0,    0,    0,    0,    0,  296, 
            0,    0,  221,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  290,  290,  290,    0,    0,  290,  290,  290,   25, 
          290,    0,    0,    0,    0,    0,  221,    0,    0,  221, 
          290,  290,  221,    0,  221,    0,    0,    0,    0,  290, 
          290,    0,  290,  290,  290,  290,  290,    0,    0,    0, 
          221,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  221,    0,  307,    0,    0,    0,    0,  294, 
          294,  294,    0,    0,  294,  294,  294,    0,  294,    0, 
            0,    0,  221,    0,    0,    0,    0,    0,    0,    0, 
            0,  290,    0,    0,  290,    0,  290,    0,    0,    0, 
          294,  294,  294,  294,  294,    0,    0,    0,    0,    0, 
            0,    0,  290,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  295, 
          295,  295,    0,    0,  295,  295,  295,    0,  295,    0, 
            0,    0,    0,    0,  290,    0,  294,    0,    0,  294, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          295,  295,  295,  295,  295,  221,  221,  221,    0,    0, 
          294,  221,  221,    0,  221,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  221,  221,    0,  221,  221,  221,  221, 
            0,    0,  294,  129,    0,    0,  295,    0,    0,  295, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,   25,   25,   25,    0,    0,    0,   25,   25, 
          295,   25,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  221,    0,    0,    0,    0, 
            0,    0,  129,   25,   25,   25,   25,   25,    0,    0, 
            0,    0,  295,    0,    0,    0,  221,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  514,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  221,    0, 
            0,    0,   25,    0,    0,    0,    0,    0,    0,    0, 
            0,    4,    5,    6,    0,    8,    0,    0,    0,    9, 
           10,    0,    0,   25,   11,    0,   12,   13,   14,   15, 
           16,   17,   18,    0,    0,    0,    0,   19,   20,   21, 
          238,  239,  240,  241,  130,    0,  117,    0,    0,    0, 
            0,    0,    0,   28,    0,   25,  242,  243,  244,    0, 
          245,   35,  246,  247,  248,  249,  284,   40,   41,   42, 
           43,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  250,    0,  130,  130,    0,    0,   46,   47,    0, 
           48,    0,  285,    0,  286,    0,   50,    0,    0,    0, 
            0,    0,    0,    0,  287,    0,    0,    0,    0,   52, 
          288,   54,   55,   56,   57,  517,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,   65,   66,  289,  129, 
          129,  129,  129,  129,  129,  129,  129,  129,  129,  129, 
            0,    0,  129,  129,    0,  129,  129,  129,  129,  129, 
          129,  129,    0,  617,    0,    0,  129,  129,  129,  129, 
          129,  129,  129,  129,    0,  129,    0,    0,    0,    0, 
            0,  129,  129,    0,  129,  129,  129,  129,    0,  129, 
          129,  129,  129,  129,  129,    0,  129,  129,  129,  129, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  514,    0,    0,    0,    0,    0,    0,  514,  514, 
          129,    0,  129,  129,  617,    0,  129,  129,    0,  129, 
            0,  129,    0,    0,    0,  129,    0,    0,    0,    0, 
          129,    0,    0,  129,    0,  617,    0,    0,  129,  129, 
          129,  129,  129,  129,  624,    0,    0,  129,    0,  129, 
          129,    0,  129,  129,  129,  129,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  129,  129,    0,    0,    0, 
          130,  130,  130,  130,  130,  130,  130,  130,  130,  130, 
          130,    0,    0,  130,  130,    0,  130,  130,  130,  130, 
          130,  130,  130,    0,  618,    0,    0,  130,  130,  130, 
          130,  130,  130,  130,  130,    0,  130,    0,    0,    0, 
            0,    0,  130,  130,    0,  130,  130,  130,  130,    0, 
          130,  130,  130,  130,  130,  130,    0,  130,  130,  130, 
          130,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  517,    0,    0,    0,    0,    0,    0,  517, 
          517,  130,    0,  130,  130,  618,    0,  130,  130,    0, 
          130,    0,  130,    0,    0,    0,  130,    0,    0,    0, 
            0,  130,    0,    0,  130,    0,  618,    0,    0,  130, 
          130,  130,  130,  130,  130,  316,    0,    0,  130,    0, 
          130,  130,    0,  130,  130,  130,  130,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  130,  130,    0,  129, 
          129,  129,  129,  129,  129,  129,  129,  129,  129,  129, 
            0,    0,  129,  129,    0,  129,  129,  129,  129,  129, 
          129,  129,    0,  624,    0,    0,  129,  129,  129,  129, 
          129,  129,  129,  130,    0,  129,    0,    0,    0,    0, 
            0,  129,  129,    0,  129,  129,  129,  129,    0,  129, 
          129,  129,  129,  129,  129,    0,  129,  129,  129,  129, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  624,    0,    0,    0,    0,    0,    0,  624,  624, 
          129,    0,  130,  129,  620,    0,  129,  129,    0,  129, 
            0,  129,    0,    0,    0,  129,    0,    0,    0,    0, 
          129,    0,    0,  129,    0,  624,    0,    0,  129,  129, 
          129,  129,  129,  129,  317,    0,    0,  129,    0,  129, 
          129,    0,  129,  129,  129,  129,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  129,  129,    0,    0,    0, 
          130,  130,  130,  130,  130,  130,  130,  130,  130,  130, 
          130,    0,    0,  130,  130,    0,  130,  130,  130,  130, 
          130,  130,  130,    0,    0,    0,    0,  130,  130,  130, 
          130,  130,  130,  130,  361,    0,  130,    0,    0,    0, 
            0,    0,  130,  130,    0,  130,  130,  130,  130,    0, 
          130,  130,  130,  130,  130,  130,    0,  130,  130,  130, 
          130,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  316,    0,    0,    0,    0,    0,    0,  316, 
          316,  130,    0,  361,  130,  621,    0,  130,  130,    0, 
          130,    0,  130,    0,    0,    0,  130,    0,    0,    0, 
            0,  130,    0,    0,  130,    0,    0,    0,    0,  130, 
          130,  130,  130,  130,  130,    0,    0,    0,  130,    0, 
          130,  130,    0,  130,  130,  130,  130,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  130,  130,    0,  130, 
          130,  130,  130,  130,  130,  130,  130,  130,  130,  130, 
            0,    0,  130,  130,    0,  130,  130,  130,  130,  130, 
          130,  130,    0,    0,    0,    0,  130,  130,  130,  130, 
          130,  130,  130,  308,    0,  130,    0,    0,    0,    0, 
            0,  130,  130,    0,  130,  130,  130,  130,    0,  130, 
          130,  130,  130,  130,  130,    0,  130,  130,  130,  130, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  317,    0,    0,    0,    0,    0,    0,  317,  317, 
          130,    0,  308,  130,    0,    0,  130,  130,    0,  130, 
            0,  130,    0,    0,    0,  130,    0,    0,    0,    0, 
          130,    0,    0,  130,    0,    0,    0,    0,  130,  130, 
          130,  130,  130,  130,    0,    0,    0,  130,    0,  130, 
          130,    0,  130,  130,  130,  130,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  130,  130,    0,    0,    0, 
          361,  361,  361,  361,  361,  361,  361,  361,  361,  361, 
          361,    0,  361,  361,  361,  361,  361,  361,  361,  361, 
          361,  361,  361,    0,    0,    0,    0,  361,  361,  361, 
          361,  361,  361,  361,  542,    0,  361,    0,    0,    0, 
            0,    0,  361,  361,    0,  361,  361,  361,  361,    0, 
          361,  361,  361,  361,  361,  361,    0,  361,  361,  361, 
          361,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  361,    0,  542,  361,    0,    0,  361,  361,    0, 
          361,    0,  361,    0,    0,    0,  361,    0,    0,    0, 
            0,    0,    0,    0,  361,    0,    0,    0,    0,  361, 
          361,  361,  361,  361,  361,    0,    0,    0,  361,    0, 
          361,  361,    0,  361,  361,  361,  361,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  361,  361,    0,  308, 
          308,  308,  308,  308,  308,  308,  308,  308,  308,  308, 
            0,    0,  308,  308,    0,  308,  308,  308,  308,  308, 
          308,  308,    0,    0,  646,    0,  308,  308,  308,  308, 
          308,  308,  308,    0,    0,  308,    0,    0,    0,    0, 
            0,  308,  308,    0,  308,  308,  308,  308,    0,  308, 
          308,  308,  308,  308,  308,    0,  308,  308,  308,  308, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  646,    0,    0,    0,    0,    0,    0, 
          308,    0,    0,  308,    0,    0,  308,  308,    0,  308, 
            0,  308,    0,    0,    0,  308,    0,    0,    0,    0, 
            0,    0,    0,  308,    0,   18,    0,    0,  308,  308, 
          308,  308,  308,  308,    0,    0,    0,  308,    0,  308, 
          308,    0,  308,  308,  308,  308,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  308,  308,    0,    0,    0, 
          542,  542,  542,  542,  542,  542,  542,  542,  542,  542, 
          542,    0,    0,  542,  542,    0,  542,  542,  542,  542, 
          542,  542,  542,  646,    0,    0,    0,  542,  542,  542, 
          542,  542,  542,  542,    0,    0,  542,    0,    0,    0, 
            0,    0,  542,  542,    0,  542,  542,  542,  542,    0, 
          542,  542,  542,  542,  542,  542,    0,  542,  542,  542, 
          542,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  646,    0,    0,    0,    0,    0,    0,    0, 
            0,  542,    0,    0,  542,    0,    0,  542,  542,    0, 
          542,    0,  542,    0,    0,    0,  542,    0,    0,    0, 
            0,    0,    0,    0,  542,    0,    0,    0,    0,  542, 
          542,  542,  542,  542,  542,    0,    0,    0,  542,    0, 
          542,  542,    0,  542,  542,  542,  542,    0,    0,    0, 
          646,  646,  646,  646,  646,  646,  542,  542,  646,  646, 
          646,    0,    0,    0,  646,    0,  646,  646,  646,  646, 
          646,  646,  646,  259,    0,    0,    0,  646,  646,  646, 
          646,  646,  646,  646,    0,    0,  646,    0,    0,    0, 
            0,    0,  646,  646,    0,  646,  646,  646,  646,    0, 
          646,  646,  646,  646,  646,  646,    0,  646,  646,  646, 
          646,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  258,    0,    0,    0,    0,    0,    0,    0, 
            0,  646,    0,    0, 
      };
   }

   private static final short[] yyTable4() {
      return new short[] {

          646,    0,    0,  646,  646,    0,  646,    0,  646,    0, 
            0,    0,  646,    0,    0,    0,    0,    0,    0,    0, 
          646,    0,    0,    0,    0,  646,  646,  646,  646,  646, 
          646,    0,    0,    0,  646,    0,  646,  646,    0,  646, 
          646,  646,  646,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  646,  646,    0,  646,  646,  646,  646,  646, 
          646,    0,    0,    0,  646,  646,    0,    0,  310,  646, 
            0,  646,  646,  646,  646,  646,  646,  646,    0,    0, 
            0,    0,  646,  646,  646,  646,  646,  646,  646,    0, 
            0,  646,    0,    0,    0,    0,    0,  646,  646,    0, 
          646,  646,  646,  646,    0,  646,  646,  646,  646,  646, 
          646,    0,  646,  646,  646,  646,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  646,    0,    0,  646, 
            0,    0,  646,  646,    0,  646,    0,  646,    0,    0, 
            0,  646,    0,    0,    0,    0,    0,    0,    0,  646, 
            0,    0,  646,    0,  646,  646,  646,  646,  646,  646, 
            0,    0,    0,  646,    0,  646,  646,    0,  646,  646, 
          646,  646,    0,    0,    0,    0,    4,    5,    6,    0, 
            8,  646,  646,    0,    9,   10,    0,  307,    0,   11, 
            0,   12,   13,   14,   15,   16,   17,   18,    0,    0, 
            0,    0,   19,   20,   21,  238,  239,  240,  241,    0, 
            0,   26,    0,    0,    0,    0,    0,    0,   28,    0, 
            0,  242,  243,  244,    0,  245,   35,  246,  247,  248, 
          249,    0,   40,   41,   42,   43,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  250,    0,    0,  130, 
            0,    0,   46,   47,    0,   48,    0,    0,    0,    0, 
            0,   50,    0,    0,    0,    0,    0,    0,    0,   51, 
            0,    0,    0,    0,   52,   53,   54,   55,   56,   57, 
            0,    0,    0,   58,    0,   59,   60,    0,   61,   62, 
           63,   64,    0,    0,    0,  312,  312,  312,    0,  312, 
            0,   65,   66,  312,  312,    0,    0,    0,  312,    0, 
          312,  312,  312,  312,  312,  312,  312,    0,    0,    0, 
            0,  312,  312,  312,  312,  312,  312,  312,    0,    0, 
          312,    0,    0,    0,    0,    0,    0,  312,    0,    0, 
          312,  312,  312,    0,  312,  312,  312,  312,  312,  312, 
          627,  312,  312,  312,  312,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  312,    0,    0,  312,    0, 
          310,  312,  312,    0,  312,    0,    0,    0,    0,    0, 
          312,    0,    0,    0,    0,    0,    0,    0,  312,    0, 
            0,    0,    0,  312,  312,  312,  312,  312,  312,    0, 
            0,    0,  312,    0,  312,  312,    0,  312,  312,  312, 
          312,    0,    0,    0,    4,    5,    6,    0,    8,    0, 
          312,  312,    9,   10,    0,    0,    0,   11,    0,   12, 
           13,   14,   15,   16,   17,   18,    0,    0,    0,    0, 
           19,   20,   21,  238,  239,  240,  241,    0,    0,   26, 
            0,    0,    0,    0,    0,    0,   28,    0,    0,  242, 
          243,  244,    0,  245,   35,  246,  247,  248,  249,    0, 
           40,   41,   42,   43,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  472,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  250,    0,    0,  130,    0,    0, 
           46,   47,    0,   48,    0,    0,    0,    0,    0,   50, 
            0,    0,    0,    0,    0,    0,    0,   51,    0,    0, 
            0,    0,   52,   53,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   65, 
           66,    0,    0,    4,    5,    6,    0,    8,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  339,  340,  341,  342,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,  118,    0,    0,  343,  344, 
          345,    0,  346,   35,  347,  348,  349,  350,    0,   40, 
            0,    0,  127,  472,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  351,    0,    0,  130,    0,    0,   46, 
           47,    0,   48,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   52,   53,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
            0,    0,    4,    5,    6,    7,    8,    0,   65,   66, 
            9,   10,    0,    0,    0,   11,    0,   12,   13,   14, 
           15,   16,   17,   18,    0,    0,    0,    0,   19,   20, 
           21,   22,   23,   24,   25,    0,    0,   26,    0,    0, 
            0,    0,    0,   27,   28,   29,   30,   31,   32,   33, 
            0,   34,   35,   36,   37,   38,   39,    0,   40,   41, 
           42,   43,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,   44,    0,    0,   45,    0,    0,   46,   47, 
            0,   48,    0,   49,    0,    0,    0,   50,    0,    0, 
            0,    0,    0,    0,    0,   51,    0,    0,    0,    0, 
           52,   53,   54,   55,   56,   57,    0,    0,    0,   58, 
            0,   59,   60,    0,   61,   62,   63,   64,    0,    0, 
            0,    4,    5,    6,    7,  320,    0,   65,   66,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,   15, 
           16,   17,   18,    0,    0,    0,    0,   19,   20,   21, 
           22,   23,   24,   25,    0,    0,   26,    0,    0,    0, 
            0,    0,   27,   28,    0,   30,   31,   32,   33,    0, 
           34,   35,   36,   37,   38,   39,    0,   40,   41,   42, 
           43,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   44,    0,    0,   45,    0,    0,   46,   47,    0, 
           48,    0,   49,    0,    0,    0,   50,    0,    0,    0, 
            0,    0,    0,    0,   51,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,    0,    3, 
            4,    5,    6,    7,    8,    0,   65,   66,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,   15,   16, 
           17,   18,    0,    0,    0,    0,   19,   20,   21,   22, 
           23,   24,   25,    0,    0,   26,    0,    0,    0,    0, 
            0,   27,   28,   29,   30,   31,   32,   33,    0,   34, 
           35,   36,   37,   38,   39,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           44,    0,    0,   45,    0,    0,   46,   47,    0,   48, 
            0,   49,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,   51,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,  319,    4, 
            5,    6,    7,  320,    0,   65,   66,    9,   10,    0, 
            0,    0,   11,    0,   12,   13,   14,   15,   16,   17, 
           18,    0,    0,    0,    0,   19,   20,   21,   22,   23, 
           24,   25,    0,    0,   26,    0,    0,    0,    0,    0, 
           27,   28,    0,   30,   31,   32,   33,    0,   34,   35, 
           36,   37,   38,   39,    0,   40,   41,   42,   43,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   44, 
            0,    0,  321,    0,    0,   46,   47,    0,   48,    0, 
           49,    0,    0,    0,   50,    0,    0,    0,    0,    0, 
            0,    0,   51,    0,    0,    0,    0,   52,   53,   54, 
           55,   56,   57,    0,    0,    0,   58,    0,   59,   60, 
            0,   61,   62,   63,   64,    0,    0,  319,    4,    5, 
            6,    7,  320,    0,   65,   66,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,   15,   16,   17,   18, 
            0,    0,    0,    0,   19,   20,   21,   22,   23,   24, 
           25,    0,    0,   26,    0,    0,    0,    0,    0,   27, 
           28,    0,   30,   31,   32,   33,    0,   34,   35,   36, 
           37,   38,   39,    0,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   44,    0, 
            0,   45,    0,    0,   46,   47,    0,   48,    0,   49, 
            0,    0,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,   51,    0,    0,    0,    0,   52,   53,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,    4,    5,    6, 
            0,    8,    0,   65,   66,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,   15,   16,   17,   18,    0, 
            0,    0,    0,   19,   20,   21,  238,  239,  240,  241, 
            0,    0,  117,    0,    0,    0,    0,    0,    0,   28, 
            0,    0,  242,  243,  244,    0,  245,   35,  246,  247, 
          248,  249,  284,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  250,    0,    0, 
          130,    0,    0,   46,   47,    0,   48,    0,  285,    0, 
          286,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
          287,    0,    0,    0,    0,   52,  288,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,    0,    0,    0,    4,    5,    6, 
            0,    8,   65,   66,  289,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,  108,  109,   17,   18,    0, 
            0,    0,    0,  110,  111,  112,  238,  239,  240,  241, 
            0,    0,  117,    0,    0,    0,    0,    0,    0,   28, 
            0,    0,  242,  243,  244,    0,  245,   35,  246,  247, 
          248,  249,  284,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  250,    0,    0, 
          130,    0,    0,   46,   47,    0,   48,    0,  682,    0, 
          286,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
          287,    0,    0,    0,    0,   52,  288,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,    0,    0,    0,  283,  283,  283, 
            0,  283,   65,   66,  289,  283,  283,    0,    0,    0, 
          283,    0,  283,  283,  283,  283,  283,  283,  283,    0, 
            0,    0,    0,  283,  283,  283,  283,  283,  283,  283, 
            0,    0,  283,    0,    0,    0,    0,    0,    0,  283, 
            0,    0,  283,  283,  283,    0,  283,  283,  283,  283, 
          283,  283,  283,  283,  283,  283,  283,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  283,    0,    0, 
          283,    0,    0,  283,  283,    0,  283,    0,  283,    0, 
          283,    0,  283,    0,    0,    0,    0,    0,    0,    0, 
          283,    0,    0,    0,    0,  283,  283,  283,  283,  283, 
          283,    0,    0,    0,  283,    0,  283,  283,    0,  283, 
          283,  283,  283,    0,    0,    0,    0,    4,    5,    6, 
            0,    8,  283,  283,  283,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,  108,  109,   17,   18,    0, 
            0,    0,    0,  110,  111,  112,  238,  239,  240,  241, 
            0,    0,  117,    0,    0,    0,    0,    0,    0,   28, 
            0,    0,  242,  243,  244,    0,  245,   35,  246,  247, 
          248,  249,  284,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  250,    0,    0, 
          130,    0,    0,   46,   47,    0,   48,    0,  285,    0, 
            0,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
          287,    0,    0,    0,    0,   52,  288,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,    0,    0,    0,    4,    5,    6, 
            0,    8,   65,   66,  289,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,  108,  109,   17,   18,    0, 
            0,    0,    0,  110,  111,  112,  238,  239,  240,  241, 
            0,    0,  117,    0,    0,    0,    0,    0,    0,   28, 
            0,    0,  242,  243,  244,    0,  245,   35,  246,  247, 
          248,  249,  284,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  250,    0,    0, 
          130,    0,    0,   46,   47,    0,   48,    0,    0,    0, 
          286,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
          287,    0,    0,    0,    0,   52,  288,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,    0,    0,    0,    4,    5,    6, 
            0,    8,   65,   66,  289,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,  108,  109,   17,   18,    0, 
            0,    0,    0,  110,  111,  112,  238,  239,  240,  241, 
            0,    0,  117,    0,    0,    0,    0,    0,    0,   28, 
            0,    0,  242,  243,  244,    0,  245,   35,  246,  247, 
          248,  249,  284,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  250,    0,    0, 
          130,    0,    0,   46,   47,    0,   48,    0,  682,    0, 
            0,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
          287,    0,    0,    0,    0,   52,  288,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,    0,    0,    0,    4,    5,    6, 
            0,    8,   65,   66,  289,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,  108,  109,   17,   18,    0, 
            0,    0,    0,  110,  111,  112,  238,  239,  240,  241, 
            0,    0,  117,    0,    0,    0,    0,    0,    0,   28, 
            0,    0,  242,  243,  244,    0,  245,   35,  246,  247, 
          248,  249,  284,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  250,    0,    0, 
          130,    0,    0,   46,   47,    0,   48,    0,    0,    0, 
            0,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
          287,    0,    0,    0,    0,   52,  288,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,    0,    0,    0,    4,    5,    6, 
            7,    8,   65,   66,  289,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,   15,   16,   17,   18,    0, 
            0,    0,    0,   19,   20,   21,   22,   23,   24,   25, 
            0,    0,   26,    0,    0,    0,    0,    0,   27,   28, 
           29,   30,   31,   32,   33,    0,   34,   35,   36,   37, 
           38,   39,    0,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,   44,    0,    0, 
           45,    0,    0,   46,   47,    0,   48,    0,   49,    0, 
            0,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
           51,    0,    0,    0,    0,   52,   53,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,    0,    0,    4,    5,    6,    7, 
            8,    0,   65,   66,    9,   10,    0,    0,    0,   11, 
            0,   12,   13,   14,   15,   16,   17,   18,    0,    0, 
            0,    0,   19,   20,   21,   22,   23,   24,   25,    0, 
            0,   26,    0,    0,    0,    0,    0,   27,   28,    0, 
           30,   31,   32,   33,    0,   34,   35,   36,   37,   38, 
           39,    0,   40,   41,   42,   43,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,   44,    0,    0,   45, 
            0,    0,   46,   47,    0,   48,    0,   49,    0,    0, 
            0,   50,    0,    0,    0,    0,    0,    0,    0,   51, 
            0,    0,    0,    0,   52,   53,   54,   55,   56,   57, 
            0,    0,    0,   58,    0,   59,   60,    0,   61,   62, 
           63,   64,    0,    0,    0,    4,    5,    6,    0,    8, 
            0,   65,   66,    9,   10,    0,    0,    0,   11,    0, 
           12,   13,   14,  108,  109,   17,   18,    0,    0,    0, 
            0,  110,  111,  112,  238,  239,  240,  241,    0,    0, 
          117,    0,    0,    0,    0,    0,    0,   28,    0,    0, 
          242,  243,  244,    0,  245,   35,  246,  247,  248,  249, 
            0,   40,   41,   42,   43,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  250,    0,    0,  130,  507, 
            0,   46,   47,    0,   48,    0,    0,    0,    0,    0, 
           50,    0,    0,    0,    0,    0,    0,    0,  287,    0, 
            0,    0,    0,   52,   53,   54,   55,   56,   57,    0, 
            0,    0,   58,    0,   59,   60,    0,   61,   62,   63, 
           64,    0,    0,    0,    4,    5,    6,    0,    8,    0, 
           65,   66,    9,   10,    0,    0,    0,   11,    0,   12, 
           13,   14,   15,   16,   17,   18,    0,    0,    0,    0, 
           19,   20,   21,  238,  239,  240,  241,    0,    0,  117, 
            0,    0,    0,    0,    0,    0,   28,    0,    0,  242, 
          243,  244,    0,  245,   35,  246,  247,  248,  249,    0, 
           40,   41,   42,   43,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  250,    0,    0,  130,    0,    0, 
           46,   47,    0,   48,    0,  618,    0,    0,    0,   50, 
            0,    0,    0,    0,    0,    0,    0,  287,    0,    0, 
            0,    0,   52,   53,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,    0,    0,    4,    5,    6,    0,    8,    0,   65, 
           66,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  238,  239,  240,  241,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,   28,    0,    0,  242,  243, 
          244,    0,  245,   35,  246,  247,  248,  249,    0,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  250,    0,    0,  130,    0,    0,   46, 
           47,    0,   48,    0,  285,    0,    0,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,  287,    0,    0,    0, 
            0,   52,   53,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
            0,    0,    4,    5,    6,    0,    8,    0,   65,   66, 
            9,   10,    0,    0,    0,   11,    0,   12,   13,   14, 
          108,  109,   17,   18,    0,    0,    0,    0,  110,  111, 
          112,  238,  239,  240,  241,    0,    0,  117,    0,    0, 
            0,    0,    0,    0,   28,    0,    0,  242,  243,  244, 
            0,  245,   35,  246,  247,  248,  249,    0,   40,   41, 
           42,   43,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  250,    0,    0,  130,    0,    0,   46,   47, 
            0,   48,    0,  618,    0,    0,    0,   50,    0,    0, 
            0,    0,    0,    0,    0,  287,    0,    0,    0,    0, 
           52,   53,   54,   55,   56,   57,    0,    0,    0,   58, 
            0,   59,   60,    0,   61,   62,   63,   64,    0,    0, 
            0,    4,    5,    6,    0,    8,    0,   65,   66,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,  108, 
          109,   17,   18,    0,    0,    0,    0,  110,  111,  112, 
          238,  239,  240,  241,    0,    0,  117,    0,    0,    0, 
            0,    0,    0,   28,    0,    0,  242,  243,  244,    0, 
          245,   35,  246,  247,  248,  249,    0,   40,   41,   42, 
           43,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  250,    0,    0,  130,    0,    0,   46,   47,    0, 
           48,    0,  891,    0,    0,    0,   50,    0,    0,    0, 
            0,    0,    0,    0,  287,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,    0,    0, 
            4,    5,    6,    0,    8,    0,   65,   66,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  108,  109, 
           17,   18,    0,    0,    0,    0,  110,  111,  112,  238, 
          239,  240,  241,    0,    0,  117,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  242,  243,  244,    0,  245, 
           35,  246,  247,  248,  249,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          250,    0,    0,  130,    0,    0,   46,   47,    0,   48, 
            0,  682,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,  287,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,    0,  635, 
          635,  635,    0,  635,    0,   65,   66,  635,  635,    0, 
            0,    0,  635,    0,  635,  635,  635,  635,  635,  635, 
          635,    0,    0,    0,    0,  635,  635,  635,  635,  635, 
          635,  635,    0,    0,  635,    0,    0,    0,    0,    0, 
            0,  635,    0,    0,  635,  635,  635,    0,  635,  635, 
          635,  635,  635,  635,    0,  635,  635,  635,  635,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  635, 
            0,    0,  635,  635,    0,  635,  635,    0,  635,    0, 
            0,    0,    0,    0,  635,    0,    0,    0,    0,    0, 
            0,    0,  635,    0,    0,    0,    0,  635,  635,  635, 
          635,  635,  635,    0,    0,    0,  635,    0,  635,  635, 
            0,  635,  635,  635,  635,    0,    0,    0,    4,    5, 
            6,    0,    8,    0,  635,  635,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,   15,   16,   17,   18, 
            0,    0,    0,    0,   19,   20,   21,  238,  239,  240, 
          241,    0,    0,   26,    0,    0,    0,    0,    0,    0, 
           28,    0,    0,  242,  243,  244,    0,  245,   35,  246, 
          247,  248,  249,    0,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  250,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,    0, 
            0,    0,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,   51,    0,    0,    0,    0,   52,   53,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,    4,    5,    6, 
            0,    8,    0,   65,   66,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,  108,  109,   17,   18,    0, 
            0,    0,    0,  110,  111,  112,  238,  239,  240,  241, 
            0,    0,  117,    0,    0,    0,    0,    0,    0,   28, 
            0,    0,  242,  243,  244,    0,  245,   35,  246,  247, 
          248,  249,    0,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  250,    0,    0, 
          130,    0,    0,   46,   47,    0,   48,    0,    0,    0, 
            0,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
          287,    0,    0,    0,    0,   52,   53,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,    0,    0,    4,    5,    6,    0, 
            8,    0,   65,   66,    9,   10,    0,    0,    0,   11, 
            0,   12,   13,   14,   15,   16,   17,   18,    0,    0, 
            0,    0,   19,   20,   21,  238,  239,  240,  241,    0, 
            0,  117,    0,    0,    0,    0,    0,    0,   28,    0, 
            0,  242,  243,  244,    0,  245,   35,  246,  247,  248, 
          249,    0,   40,   41,   42,   43,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  250,    0,    0,  130, 
            0,    0,   46,   47,    0,   48,    0,    0,    0,    0, 
            0,   50,    0,    0,    0,    0,    0,    0,    0,  287, 
            0,    0,    0,    0,   52,   53,   54,   55,   56,   57, 
            0,    0,    0,   58,    0,   59,   60,    0,   61,   62, 
           63,   64,    0,    0,    0,  635,  635,  635,    0,  635, 
            0,   65,   66,  635,  635,    0,    0,    0,  635,    0, 
          635,  635,  635,  635,  635,  635,  635,    0,    0,    0, 
            0,  635,  635,  635,  635,  635,  635,  635,    0,    0, 
          635,    0,    0,    0,    0,    0,    0,  635,    0,    0, 
          635,  635,  635,    0,  635,  635,  635,  635,  635,  635, 
            0,  635,  635,  635,  635,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  635,    0,    0,  635,    0, 
            0,  635,  635,    0,  635,    0,    0,    0,    0,    0, 
          635,    0,    0,    0,    0,    0,    0,    0,  635,    0, 
            0,    0,    0,  635,  635,  635,  635,  635,  635,    0, 
            0,    0,  635,    0,  635,  635,    0,  635,  635,  635, 
          635,    0,    0,    0,    4,    5,    6,    0,    8,    0, 
          635,  635,    9,   10,    0,    0,    0,   11,    0,   12, 
           13,   14,  108,  109,   17,   18,    0,    0,    0,    0, 
          110,  111,  112,  113,  114,  115,  116,    0,    0,  117, 
            0,    0,    0,    0,    0,    0,  118,    0,    0,  119, 
          120,  121,    0,  122,   35,  123,  124,  125,  126,    0, 
           40,    0,    0,  127,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          128,    0,    0,    0,  129,    0,    0,  130,    0,    0, 
           46,   47,    0,   48,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,   52,   53,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,    0,    0,    4,    5,    6,    0,    8,    0,   65, 
           66,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  266,  267,  268,  269,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,  118,    0,    0,  270,  271, 
          272,    0,  273,   35,  274,  275,  276,  277,    0,   40, 
            0,    0,  127,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  278,    0,    0,   45,    0,    0,   46, 
           47,    0,   48,    0,   49,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   52,   53,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
            0,    0,    4,    5,    6,    0,    8,    0,   65,   66, 
            9,   10,    0,    0,    0,   11,    0,   12,   13,   14, 
          108,  109,   17,   18,    0,    0,    0,    0,  110,  111, 
          112,  339,  340,  341,  342,    0,    0,  117,    0,    0, 
            0,    0,    0,    0,  118,    0,    0,  343,  344,  345, 
            0,  346,   35,  347,  348,  349,  350,    0,   40,    0, 
            0,  127,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  351,    0,    0,  428,    0,    0,   46,   47, 
            0,   48,    0,  429,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           52,   53,   54,   55,   56,   57,    0,    0,    0,   58, 
            0,   59,   60,    0,   61,   62,   63,   64,    0,    0, 
            0,    4,    5,    6,    0,    8,    0,   65,   66,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,  108, 
          109,   17,   18,    0,    0,    0,    0,  110,  111,  112, 
          113,  114,  115,  116,    0,    0,  117,    0,    0,    0, 
            0,    0,    0,  118,    0,    0,  119,  120,  121,    0, 
          122,   35,  123,  124,  125,  126,    0,   40,    0,    0, 
          127,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  129,    0,    0,  130,    0,    0,   46,   47,    0, 
           48,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,    0,    0, 
            4,    5,    6,    0,    8,    0,   65,   66,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  108,  109, 
           17,   18,    0,    0,    0,    0,  110,  111,  112,  339, 
          340,  341,  342,    0,    0,  117,    0,    0,    0,    0, 
            0,    0,  118,    0,    0,  343,  344,  345,    0,  346, 
           35,  347,  348,  349,  350,    0,   40,    0,    0,  127, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          351,    0,    0,  428,    0,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,    0,    4, 
            5,    6,    0,    8,    0,   65,   66,    9,   10,    0, 
            0,    0,   11,    0,   12,   13,   14,  108,  109,   17, 
           18,    0,    0,    0,    0,  110,  111,  112,  113,  114, 
          115,  116,    0,    0,  117,    0,    0,    0,    0,    0, 
            0,  118,    0,    0,  119,  120,  121,    0,  954,   35, 
          123,  124,  955,  126,    0,   40,    0,    0,  127,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  956, 
            0,    0,  130,    0,    0,   46,   47,    0,   48,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,   52,   53,   54, 
           55,   56,   57,    0,    0,    0,   58,    0,   59,   60, 
            0,   61,   62,   63,   64,    0,    0,    0,    4,    5, 
            6,    0,    8,    0,   65,   66,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,  108,  109,   17,   18, 
            0,    0,    0,    0,  110,  111,  112,  970,  971,  972, 
          973,    0,    0,  117,    0,    0,    0,    0,    0,    0, 
          118,    0,    0,  974,  975,  976,    0,  977,   35,  978, 
          979,  980,  981,    0,   40,    0,    0,  127,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  982,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,   52,   53,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,   65,   66,  138,  139,  140,  141,  142, 
          143,  144,  145,  146,  147,  148,  149,  150,  151,  152, 
          153,  154,  155,  156,  157,  158,  159,  160,  161,    0, 
            0,  162,  163,  164,  222,  223,  224,  225,  169,  170, 
          171,  172,  173,  174,  175,  176,  177,  178,  179,  180, 
          226,  227,  228,    0,  229,  185,  309,    0,  230,    0, 
            0,    0,  187,  188,    0,  189,  190,  191,  192,  193, 
          194,  195,    0,    0,  196,  197,    0,    0,    0,  198, 
          199,  200,  201,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  203,  204,    0,  205, 
          206,  207,  208,  209,  210,  211,  212,  213,  214,  215, 
            0,    0,  216,   52,  138,  139,  140,  141,  142,  143, 
          144,  145,  146,  147,  148,  149,  150,  151,  152,  153, 
          154,  155,  156,  157,  158,  159,  160,  161,    0,    0, 
          162,  163,  164,  222,  223,  224,  225,  169,  170,  171, 
          172,  173,  174,  175,  176,  177,  178,  179,  180,  226, 
          227,  228,    0,  229,  185,    0,    0,  230,    0,    0, 
            0,  187,  188,    0,  189,  190,  191,  192,  193,  194, 
          195,    0,    0,  196,  197,    0,    0,    0,  198,  199, 
          200,  201,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  203,  204,    0,  205,  206, 
          207,  208,  209,  210,  211,  212,  213,  214,  215,    0, 
            0,  216,   52,  138,  139,  140,  141,  142,  143,  144, 
          145,  146,  147,  148,  149,  150,  151,  152,  153,  154, 
          155,  156,  157,  158,  159,  160,  161,    0,    0,  162, 
          163,  164,  165,  166,  167,  168,  169,  170,  171,  172, 
          173,  174,  175,  176,  177,  178,  179,  180,  181,  182, 
          183,    0,  184,  185,  123,  124,  186,  126,    0,    0, 
          187,  188,    0,  189,  190,  191,  192,  193,  194,  195, 
            0,    0,  196,  197,    0,    0,    0,  198,  199,  200, 
          201,    0,    0,    0,    0,    0,    0,  202,    0,    0, 
            0,    0,    0,    0,  203,  204,    0,  205,  206,  207, 
          208,  209,  210,  211,  212,  213,  214,  215,    0,    0, 
          216,  138,  139,  140,  141,  142,  143,  144,  145,  146, 
          147,  148,  149,  150,  151,  152,  153,  154,  155,  156, 
          157,  158,  159,  160,  161,    0,    0,  162,  163,  164, 
          222,  223,  224,  225,  169,  170,  171,  172,  173,  174, 
          175,  176,  177,  178,  179,  180,  226,  227,  228,    0, 
          229,  185,  359,  360,  230,  361,    0,    0,  187,  188, 
            0,  189,  190,  191,  192,  193,  194,  195,    0,    0, 
          196,  197,    0,    0,    0,  198,  199,  200,  201,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  203,  204,    0,  205,  206,  207,  208,  209, 
          210,  211,  212,  213,  214,  215,    0,    0,  216,  138, 
          139,  140,  141,  142,  143,  144,  145,  146,  147,  148, 
          149,  150,  151,  152,  153,  154,  155,  156,  157,  158, 
          159,  160,  161,    0,    0,  162,  163,  164,  222,  223, 
          224,  225,  169,  170,  171,  172,  173,  174,  175,  176, 
          177,  178,  179,  180,  226,  227,  228,    0,  229,  185, 
            0,    0,  230,    0,    0,    0,  187,  188,    0,  189, 
          190,  191,  192,  193,  194,  195,    0,    0,  196,  197, 
            0,    0,    0,  198,  199,  200,  201,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          203,  204,    0,  205,  206,  207,  208,  209,  210,  211, 
          212,  213,  214,  215,  663,  608,  216,    0,  664,    0, 
            0,    0,  187,  188,    0,  189,  190,  191,  192,  193, 
          194,  195,    0,    0,  196,  197,    0,    0,    0,  198, 
          199,  200,  201,    0,    0,    0,    0,    0,    0,  303, 
            0,    0,    0,    0,    0,    0,  203,  204,    0,  205, 
          206,  207,  208,  209,  210,  211,  212,  213,  214,  215, 
          674,  600,  216,    0,  675,    0,    0,    0,  187,  188, 
            0,  189,  190,  191,  192,  193,  194,  195,    0,    0, 
          196,  197,    0,    0,    0,  198,  199,  200,  201,    0, 
            0,    0,    0,    0,    0,  303,    0,    0,    0,    0, 
            0,    0,  203,  204,    0,  205,  206,  207,  208,  209, 
          210,  211,  212,  213,  214,  215,  677,  608,  216,    0, 
          678,    0,    0,    0,  187,  188,    0,  189,  190,  191, 
          192,  193,  194,  195,    0,    0,  196,  197,    0,    0, 
            0,  198,  199,  200,  201,    0,    0,    0,    0,    0, 
            0,  303,    0,    0,    0,    0,    0,    0,  203,  204, 
            0,  205,  206,  207,  208,  209,  210,  211,  212,  213, 
          214,  215,  705,  600,  216,    0,  706,    0,    0,    0, 
          187,  188,    0,  189,  190,  191,  192,  193,  194,  195, 
            0,    0,  196,  197,    0,    0,    0,  198,  199,  200, 
          201,    0,    0,    0,    0,    0,    0,  303,    0,    0, 
            0,    0,    0,    0,  203,  204,    0,  205,  206,  207, 
          208,  209,  210,  211,  212,  213,  214,  215,  708,  608, 
          216,    0,  709,    0,    0,    0,  187,  188,    0,  189, 
          190,  191,  192,  193,  194,  195,    0,    0,  196,  197, 
            0,    0,    0,  198,  199,  200,  201,    0,    0,    0, 
            0,    0,    0,  303,    0,    0,    0,    0,    0,    0, 
          203,  204,    0,  205,  206,  207,  208,  209,  210,  211, 
          212,  213,  214,  215, 1060,  600,  216,    0, 1061,    0, 
            0,    0,  187,  188,    0,  189,  190,  191,  192,  193, 
          194,  195,    0,    0,  196,  197,    0,    0,    0,  198, 
          199,  200,  201,    0,    0,    0,    0,    0,    0,  303, 
            0,    0,    0,    0,    0,    0,  203,  204,    0,  205, 
          206,  207,  208,  209,  210,  211,  212,  213,  214,  215, 
         1065,  600,  216,    0, 1066,    0,    0,    0,  187,  188, 
            0,  189,  190,  191,  192,  193,  194,  195,    0,    0, 
          196,  197,    0,    0,    0,  198,  199,  200,  201,    0, 
            0,    0,    0,    0,    0,  303,    0,    0,    0,    0, 
            0,    0,  203,  204,    0,  205,  206,  207,  208,  209, 
          210,  211,  212,  213,  214,  215, 1068,  608,  216,    0, 
         1069,    0,    0,    0,  187,  188,    0,  189,  190,  191, 
          192,  193,  194,  195,    0,    0,  196,  197,    0,    0, 
            0,  198,  199,  200,  201,    0,    0,    0,    0,    0, 
            0,  303,    0,    0,    0,    0,    0,    0,  203,  204, 
            0,  205,  206,  207,  208,  209,  210,  211,  212,  213, 
          214,  215,    0,    0,  216, 
      };
   }

   private static final short[] yyCheck1() {
      return new short[] {

           26,   88,   28,   90,   27,    2,    3,    4,    5,   21, 
          295,   32,    9,   10,   11,   21,   74,   14,   15,   16, 
           51,  469,   19,  427,    4,    5,   10,  449,    3,    0, 
            2,    3,  257,   11,   14,  334,    6,  474,  431,  338, 
          489,   15,   16,   45,   49,   19,   10,   11,   45,    0, 
            6,    7,   49,  104,   51,   59,   15,   16,    7,   99, 
           19,  538,    0,   10,    7,   10,   10,  486,  420,   49, 
          489,   27,   10,   59,  104,  419,   10,  421,   27,  528, 
          424,   10,   52,   10,   27,   10,   10,  280,   47,   48, 
           10,   10,  118,  646,  106,  447,   52,   15,   16,   10, 
          106,   19,   99,  564,  448,  849,  112,   10,  925,  662, 
           10,  463,    0,  959,  321,   59,   61,  509,  462,   99, 
          464,   59,   10,  676,   44,  477,  104,   10,   10,   47, 
          136,  128,   59,  130,  478,  839,  556,  598,  129,   59, 
           59,   10,   10,  950,  305,   10,   10,   10,   59,  280, 
          627,   10,   10,   10,  707,  859,  305,  341,   10,   59, 
          345,   44,   10,  650,   10,  652,  253,  341,  512,  362, 
           43,   59,   10,  923,  325,   44,   59,   59,  363,   44, 
           44,  556,  342,   10,  536,   10,   10,   10,   10,   10, 
           59,   59,  279,  537,   59,   59,   10,   10,   44,  359, 
           59,   59,   59,  362,   10,  656,   44,   59,  359,  260, 
          341,   59,   10,  498,  375,  666, 1062,   44,  762,  641, 
           44,  428,   44,   44,  768, 1042,  375,  308,  315,  651, 
          650,  362,  652,   32,   61,   59,   61,  724,   59,   10, 
          237,  690,    2,    3,   10,   10,   59, 1054,  998,    9, 
           10,   11,  328,   10,  127,   15,   16,   10,  310,   19, 
          336,   10,  655,   10, 1008,   10,  353,  264,  265,   10, 
          689,  690,  323,  342,   10,  650,   10,  652,  263,  264, 
           10,   44,  260,  366,  269,   45,   10,  373,   59,    0, 
          359,   51,  378,  323, 1044,  376,  377,   10,   61,   10, 
          264,  265,  779,  300,  724,  302,  303,  699,   44,  306, 
           10,  308,  352,  373,   10,   32,  313,  306,  378,  321, 
          781,  310,  319,   59,  321, 1075,  300,   61,  302,  303, 
          373,   61,  752,  330,  279,  378,  282,   61,  342,  373, 
           32,  300,   32,  302,  378,  323,   59,  319,   59,  724, 
          371,  372,  269,   32,  271,  352,  267,  378,  342,   59, 
           44,  345,  281,   61,  264,   61,  290,  291,  128,  269, 
          130,  775,  352,  359,  262,  263,  264,  338,  815,  267, 
          268,  269,  300,  271,  302,  382,  383,  384,  385,  386, 
          387,  388,  363,  281,  425,  342,  427,  342,  345,  684, 
          431,  432,  290,  291,  703,  293,  294,  295,  296,  297, 
           61,  345,  363,  342,  386,  421,  345,  342,  382,  383, 
          384,  385,  306,  420,  429,  363,  428,  847,  425,  849, 
          427,  428,  429,  338,  431,  432,  363,  434,  341,  359, 
          328,  305,  448,  668,  328,  329,  420,  335,  336,  429, 
          447,  395,  449,  450,  342,  446,    2,    3,  464,  434, 
          342,  341,  434,  460,  915,  421,  463,  915,  424,   10, 
          919,  468,  478,  447,  896,  363,  359,  237,  342,  342, 
          477,  305,  345,  506,  305,    2,    3,    4,    5,  463, 
          359,  359,  448,  264,  359,  473,  474,   14,  538,   45, 
          919,  359,  359,  477,  264,  265,  512,  395,  464,  427, 
          507,  375,  509,  431, 1067,  342,  279,  342,  342,  342, 
           61,  342,  478,  520,   10,  357,  946,  486,   45,  342, 
          950,  537,   49,  847,  819,  359,  342,  509,  359,  536, 
          300,  538,  302,  303,  342,  279,  306,  969,  308,  279, 
          506,  375,  470,  313,  375,  279,  512,  506,  538,  319, 
           44,  321,  536,  506,  290,  291,  525,  310,   10,  305, 
          330,  597,  371,  372,  338,   61,  342,  342,   61,  378, 
          606,  537,   99,  279,  130,  342,   10,  627, 1008,  342, 
          602,  271,  305,  342,   10,  342,  602,  342,  610,   10, 
          338,  342,   10,  615,  610,  305,  342,  491,  342,  615, 
           44,   59,  342,  130,  279,  499,  500,   59,  342,  310, 
          646,   10,  382,  383,  384,  385,  386,  387,  388,  342, 
          627,  654,  946,  517, 1054,  339,  662,   61,  644,  375, 
           44,  647,  342,  640,  641,   61,  342,  627,   59,   44, 
          676,    2,    3,   61,  651,   44,  362,   91,  268,  269, 
          420,  639,  375,   14,  704,  425,  746,  427,  428,  317, 
           59,  431,  432,  317,  434,  375,  640,  308,  309,  904, 
          311,  707,  679,  653, 1036,  910,  338,  447, 1002,  449, 
          450,  237,   10, 1037,   45,  350,  351,  653,  654,   10, 
          460,   10,  699,  463,  342,  654,  345,  704,  468,    2, 
          363,  654,  310,  312,  772,  679,   44,  477,  317,  373, 
          237,   91,   10,  373,  704,   10,  752,  699,  759,  760, 
          689,  263,  264,   44,  765,  766, 1050,  655,  279,  779, 
           45,   59,  626,   10,  775,  376,  377,  507,   59,  509, 
           59,  350,  351,  373,   47,   48,  373,  754,   44,  756, 
          520,  758,  759,  760,  347,  649,  341,  313,  765,  766, 
          341,   59,  262,  319,   59,  321,  536,   44,  775,  130, 
          267,   44,  779,  863,  371,  372,  373,  264,  868,  264, 
           10,  378,  342,  279,   61,  394,  313,   44,   44,  779, 
          797,  342,  319,  800,  321,  783,  317,  363,  362,  396, 
          694,   44,  790,  839,  811,  812,  813,   32,  702,   32, 
          262,  263,  264,  338,   91,  130,  268,  269,  338,  271, 
          338,  737,  738,  859,   91,  352,  800,  815,  349,   59, 
          386,  338,  353,  354,   61,  279,  280,  844,  845,   44, 
          881,  262,  263,  264,   44,  279,  342,  268,  269,   10, 
          271,  941,   91,  279,  264,  338,   10,  328,   44,  386, 
          723,  279,  869,  269,  335,  336,   10,  264,  267,  338, 
          640,  641,  428,  338,  881,  338,  237,  363,  434,  869, 
           44,  651,  306,  306,  328,   44,  983,   44,  895,  896, 
          342,  335,  336,  449,  788,   44,  903,  341,   59,  306, 
          907,  428,  429,   44,  460,   91,   61,  434,  342,  679, 
           10,  363,  468,  903,  345,   59,  342,  264,  362,   58, 
           10,  342,  449,  338,  342,   91,  339,  264,   10,  699, 
         1020,   44,  263,  460,   15,   16,  338,  264,   19,  267, 
          339,  468,  363,  395,  838,  264,  267,   91,  328,  268, 
          269,  958,  313,  509,   44,  335,  336,  264,  319,   59, 
          321,   44,  969,  363,  520,  263,  264,   44,  958,   59, 
          833,  834,  267,   10,  395,   44,   44,   59,   91,  986, 
           44,  988,  509,   44,  754,  359,  756,  994,  758,  759, 
          760,   91, 1028,  520,   61,  765,  766,   59,  313,  264, 
          303,   91,  279,  371,  372,  775,  321,   44,  363,  264, 
          378,  538,  928,  264,  264,  383,   44,  933,  934,  342, 
          936, 1037,  938,   10,   61,  386,   44,  797,  922, 1036, 
          800, 1067,  264,  306, 1041,  308,  309,  310,  311,  363, 
           44,  811,  812,  813,  339,   44,    9,   10,   11,   44, 
          359,  328, 1036,  363,   91,  264,  310,   44,  335,  336, 
          923,  328,  925,  310,  341,  342,  427,  428,  335,  336, 
           44, 1037,   59,  434,  844,  845,  306,  264,  308,  309, 
          310,  311,  312,   48,   44,  641,   44,  317,  449,  328, 
           91,   44,   44,  279,  280,  651,  335,  336,   44,  460, 
          627,  262,  263,  264,   91,   44,  264,  468,  269,  395, 
          340,  881,   44,  428,  641,  271,  345,  420,  348,  349, 
          350,  351,   10,  267,  651,  895,  896,  371,  372,  373, 
         1046, 1047, 1048, 1049,  378,  998,  999,  907,  379,  380, 
          381,  382,  328,  699,  447,  460,   91,  363,  509,  335, 
          336,  341,  306,  468,  308,  309,  310,  311,  312,  520, 
          463,  271,  328,  317,  394,  128,  279,  267,  342,  335, 
          336,   59,  699, 1089,  477,  345,  362,  704,  363, 1042, 
          280, 1044,  264,  345,  328,  712,  340,  269,  264,  271, 
          280,  335,  336,  345,  348,  349,  350,  351,  754,  969, 
          756,   58,  758,   91,   70,  306,  509,  308,  309,  310, 
          311,   54, 1075,  752,  288,  328,  986,    5,  988,  300, 
           91,  302,  335,  336,  994,   10,   91,  754,  328,  756, 
           10,  758,  375,  536, 1039,  335,  336,  520,  328,  340, 
          394,  797,  279,  280,    6,  335,  336,   10,  615,   10, 
          902,  341,  779,  847,  752,  811,  812,  813,  946,   44, 
          563,  564,  362,  851,   44,  948, 1036,  771,   14,  359, 
          797, 1041,  362, 1038,   44,  833,   61, 1062,   -1,   59, 
          641,   44,   -1,   44,  811,  812,  813,   -1,  844,  845, 
          651,  328,   -1,  280,   -1,  598,   59,   -1,  335,  336, 
           61,  264,  265,   -1,  341,  342,   91,  610,  847,   -1, 
          849,   91,  615,  752,   -1,   -1,   -1,  844,  845,  284, 
          285,  286,   -1,   -1,  289,  362,    0,  328,   91,   -1, 
           91,   -1,   -1,   -1,  335,  336,   10,   -1,  699,  895, 
          896,  328,  869,  306,  647,  308,   -1,   -1,  335,  336, 
          317,  907,  655,  656,  341,  658,   -1,  371,  372,  373, 
           -1,   -1,   -1,  666,  378,   -1,   -1,  330,  895,  896, 
           -1,   44,  359,   15,   16,  362,  903,   19,   -1,  267, 
          907,   -1,  349,  328,   -1,   59,  353,  354,  355,  356, 
          335,  336,  280,  754,   -1,  756,  699,  758,   -1,   41, 
           42,   -1,   -1,   -1,   -1,   47,   48,  946,   50,   51, 
           -1,  950,  851,  969,  775,  854,   -1,   -1,   91,  382, 
          383,  384,  385,   -1,  387,  388,   -1,  371,  372,  373, 
          986,  958,  988,   -1,  378,   -1,  797,   -1,  994,  754, 
          328,  756,  969,  758,   -1,   -1,   -1,  335,  336,   -1, 
          811,  812,  813,  341,   -1,   -1,   -1,  328,   -1,  986, 
           -1,  988,  427,  328,  335,  336,  431,  994,   -1, 1008, 
          335,  336,   -1,   -1,  362,   -1,   -1,   -1,  781,  371, 
          372,  373,  797,  844,  845, 1041,  378,  450,   -1,   -1, 
           -1,   -1,  795,   -1,  279,  798,  811,  812,  813,   -1, 
           10,  371,  372,  373,  943,  470,   -1,  946,  378,  948, 
          752,  950,   -1,   -1, 1041, 1054,   -1,  280,  279,  280, 
          485,  486,   -1,   -1,  489,  371,  372,  373,   -1,  844, 
          845,   -1,  378,   -1,  895,  896,  306,   -1,  308,  309, 
          310,  311,  903,  328,  507,   -1,  907,   -1,  328,   59, 
          335,  336,  371,  372,  373,  335,  336,  342,   -1,  378, 
          525,  341,   -1,  528,   -1,  328,   -1,  328,   -1,  371, 
          372,  373,  335,  336,  335,  336,  378,   -1,  341,  359, 
          895,  342,   44,   -1,   44,   -1,   -1,   -1,  262,  263, 
          264,   -1,  907,  267,  268,  269,  359,  271,   -1,  362, 
          306,  362,  308,  309,  310,  311,  279,  281,  969,  851, 
           -1, 1050,  915, 1052,   -1, 1054,  290,  291, 1057,  293, 
          294,  295,  296,  297,   -1,  986,   -1,  988,   -1,   91, 
           -1,   91,   -1,  994,  293,  294,  295,  296,  297,    0, 
           -1,   -1,  284,  285,  286,  287,   -1,  289,   -1,   10, 
           -1,   -1, 1091,  618,   -1,  328,   -1,   -1,  300,   -1, 
          302,  303,  335,  336,   -1,   -1,   -1,   -1,  342,  311, 
           -1,  986,   -1,  988,   -1,   -1,   -1,  640,   -1,  994, 
         1041,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,  363, 
          655,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1, 
           61,  943,   63,  317,  946,   -1,  948,   -1,  950,   -1, 
           -1,   -1,   -1,   -1,   -1,  680,  679,  682,  752,  333, 
          334,  395,   -1,   -1,  689,  690, 1041,   -1,   10,   -1, 
           91,   -1,   -1, 1036,   -1,  349, 1039,  351,   -1,  353, 
          354,  355,  356,   -1,   44,   -1,   10,  389,  390,  391, 
          392,  393,  394,  395,  396,  397,  398,  399,  400,  401, 
          402,  403,  404,  405,  406,  407,  408,  409,  410,  411, 
          412,  413,  414,   -1,   -1,   -1,   -1,   59,  420,   -1, 
           44,   -1,  747,  425,   -1,  427,   -1,   -1,   -1,  431, 
          432,   91,   -1,   -1,   -1,   -1,  306,   61,  308,  309, 
          310,  311,  312,   -1,   -1,  447,    0,  317, 1050,   91, 
         1052,  776, 1054,   -1,   -1, 1057,   10,  279,  328,  279, 
           -1,  463,   -1,  465,  466,  467,  336,   91,  470,   -1, 
          340,  341,   -1,   -1,   -1,  477,   -1,  800,  348,  349, 
          350,  351,   -1,  485,  486,  810,   -1,  489,   -1, 1091, 
           44,  306,  752,  308,  309,  310,  311,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,   58,   59,  328,   -1,  328,   63, 
           -1,  513,  514,  335,  336,  335,  336,   -1,    0,  341, 
           -1,   -1,   -1,  525,  394,  340,  528,   -1,   10,   -1, 
           -1,   -1,  857,  348,  536,   -1,   -1,   -1,   -1,   10, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,  943, 
          271,   59,  946,   -1,  948,   -1,  950,   -1,   -1,  280, 
          281,  282,   44,   -1,   -1,   -1,  891,   -1,   10,  290, 
          291,   -1,  293,  294,  295,  296,  297,   59,   10,   -1, 
           -1,   -1,  842,   91,  305,   -1,   -1,   -1,   59,  849, 
           -1,   -1,   -1,   -1,  919,   -1,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,  618,   59,  339,  279, 
           91,  342,   -1,   -1,  345,  267,  347,   59,  349,   -1, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,  362,  363,   -1,   -1,   -1,   -1,   -1,   -1,   91, 
           -1,   -1,   -1,  655,  375,  279, 1050,   -1, 1052,   91, 
         1054,    0,   -1, 1057,   -1,   -1,   -1,   -1,  328,   -1, 
           -1,   10,   -1,   -1,  395,  335,  336,   -1,  680,   -1, 
          682,   -1,   -1,  943,   -1,  860,  328,  689,  690,  864, 
          950,   -1,   -1,  335,  336,   -1,   -1, 1091,   -1,  341, 
           -1,   -1,   -1,   -1,  328,   44,   -1,   -1,  262,  263, 
          264,  335,  336,  267,  268,  269,   -1,  271,  342,   58, 
           59,   -1,   61,   -1,   63,   -1,   -1,  281,  282,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,  747, 1006,   -1, 1008,   -1, 
         1010,  305,   91,   -1,   -1,   -1,   -1,  759,  760,  267, 
           -1,   -1,   -1,  765,  766,   -1,   -1,   -1,   -1,   -1, 
          324,  325,  774,  775,  776,  329,  330,   -1,  780,   -1, 
          262,  263,  264,   -1,   -1,  339,  268,  269,  342,  271, 
           -1,  345, 1052,  347, 1054,   -1,  267, 1057,   -1,   -1, 
            0,  803,  804,   -1,  806,  807,   -1,   -1,  810,  363, 
           10,  293,  294,  295,  296,  297,   -1, 1077,   -1,   -1, 
          328,  375,   -1,   -1,   -1,  267,   -1,  335,  336,   -1, 
           -1, 1091,   -1,   -1,   -1,  267,   -1,   -1,   -1, 1014, 
           -1,  395, 1017, 1018,   44,   -1, 1021, 1022,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  857,   -1,  328,   58,   59, 
          342,   61,   -1,   63,  335,  336,   -1,   -1,   -1,   -1, 
           -1,  306,  874,  308,  309,  310,  311,  312,   -1,  881, 
           -1,  363,  317,   -1,   -1,   -1,  328,   -1,   -1,  891, 
           -1,   91,   -1,  335,  336,   -1,  328,   -1,   -1,   -1, 
           -1,   -1,   -1,  335,  336,  340, 1081, 1082, 1083, 1084, 
          912,   -1,   10,  395,   -1,  350,  351,  919,  257,  258, 
          259, 1096,  261,  262,  263,  264,  265,  266,  267,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           -1,  280,  281,  282,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,  394, 
          299,   59,   -1,  302,  303,  304,  305,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   44,   -1,  333,  334,  335,  336,  337,  338, 
          339,  340,  341,  342,  343,  344,  345,  346,  347,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  362,  363,   -1,  365,  366,  367,  368, 
          369,  370,   44,   -1, 1036,  374,  375,  376,  377,   91, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,   -1,  392,  393,  394,  395,  257,  258,  259, 
           -1,  261,  262,  263,  264,  265,  266,  267,  268,  269, 
          270,  271,  272,  273,  274,  275,  276,  277,  278,   91, 
          280,  281,  282,  283,  284,  285,  286,  287,  288,  289, 
          290,  291,  292,  293,  294,  295,  296,  297,   -1,  299, 
           59,   -1,  302,  303,  304,  305,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   44,   91,  333,  334,  335,  336,  337,  338,  339, 
          340,  341,  342,  343,  344,  345,  346,  347,  348,  349, 
          350,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,  362,  363,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,    0,  374,  375,  376,  377,   91,  379, 
          380,  381,  382,   10,   -1,   -1,   -1,  365,   -1,  367, 
           -1,  369,  392,  393,  394,  395,   -1,  375,  262,  263, 
          264,   -1,   -1,   -1,  268,  269,   -1,  271,  306,   -1, 
          308,  309,  310,  311,  312,   -1,   -1,   44,   -1,  317, 
          306,   -1,  308,  309,  310,  311,   -1,  279,  280,   -1, 
          328,   58,   59,   -1,   61,   -1,   63,   -1,  336,  262, 
          263,  264,  340,  341,   -1,  268,  269,   -1,  271,   -1, 
          348,  349,  350,  351,  340,   -1,  317,   -1,   -1,   -1, 
           -1,   -1,  348,   -1,   91,   -1,   -1,  279,  280,   -1, 
           -1,   -1,  333,  334,   -1,   -1,  328,   -1,  342,   -1, 
           -1,   -1,   -1,  335,  336,   -1,   -1,   -1,  349,  341, 
          351,   -1,  353,  354,  355,  356,  394,   -1,  359,  363, 
          361,   -1,   -1,   -1,   -1,  483,   -1,   -1,  267,   -1, 
          362,   -1,    0,   -1,   -1,   -1,  328,   -1,   -1,  342, 
           -1,  317,   10,  335,  336,   -1,  322,  323,   -1,  341, 
           -1,  395,   -1,   -1,   -1,   -1,   -1,  333,  334,   -1, 
          363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          362,   -1,   -1,  349,   -1,  351,   44,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,  279,   -1,   -1,  328, 
           58,   59,  395,   61,  552,   63,  335,  336,   -1,   -1, 
           -1,  559,   -1,  306,   -1,  308,  309,  310,  311,  312, 
           -1,   -1,   -1,   -1,  317,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  328,   -1,  340,  341,   -1, 
           -1,   -1,  335,  336,   10,  348,  349,  350,  351,   -1, 
          257,  258,  259,   -1,  261,  262,  263,  264,  265,  266, 
          267,  268,  269,  270,  271,  272,  273,  274,  275,  276, 
          277,  278,   -1,  280,  281,  282,  283,  284,  285,  286, 
          287,  288,  289,  290,  291,  292,  293,  294,  295,  296, 
          297,  394,  299,   59,   -1,  302,  303,  304,  305,  306, 
          307,  308,  309,  310,  311,  312,  313,  314,  315,  316, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   44,   -1,  333,  334,  335,  336, 
          337,   10,  339,  340,  341,  342,  343,  344,  345,  346, 
          347,  348,  349,  350,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,  362,  363,   -1,  365,  366, 
          367,  368,  369,  370,   44,   -1,   -1,  374,  375,  376, 
          377,   91,  379,  380,  381,  382,   -1,   -1,   -1,   -1, 
           59,   -1,   -1,   10,   -1,  392,  393,  394,  395,  257, 
          258,  259,   -1,  261,  262,  263,  264,  265,  266,  267, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,   91,  280,  281,  282,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,  293,  294,  295,  296,  297, 
           -1,  299,   59,   -1,  302,  303,  304,  305,  306,  307, 
          308,  309,  310,  311,  312,  313,  314,  315,  316,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   44,   91,  333,  334,  335,  336,  337, 
           -1,  339,  340,  341,  342,  343,  344,  345,  346,  347, 
          348,  349,  350,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,  362,  363,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,    0,  374,  375,  376,  377, 
           91,  379,  380,  381,  382,   10,  317,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  392,  393,  394,  395,   -1,   -1, 
           -1,   -1,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1, 
          306,   -1,  308,  309,  310,  311,  312,   -1,  349,   44, 
           -1,  317,  353,  354,  355,  356,   -1,   -1,   -1,  279, 
          280,   -1,  328,   58,   59,   -1,   61,   -1,   63,   -1, 
          336,   -1,   -1,   -1,  340,  341,   -1,   -1,   -1,   -1, 
           -1,   -1,  348,  349,  350,  351,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,  279, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  328,   -1, 
           -1,   -1,   -1,   -1,   -1,  335,  336,   -1,   -1,   -1, 
           -1,  341,   -1,   -1,   -1,   -1,   -1,  306,  394,  308, 
          309,  310,  311,  312,   -1,   -1,   -1,   -1,  317,   -1, 
          267,   -1,  362,   -1,    0,   -1,   -1,   -1,  328,  328, 
           -1,   -1,   -1,   -1,   10,  335,  336,  336,   -1,   -1, 
           -1,  340,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348, 
          349,  350,  351,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          317,  318,  319,  320,  321,  322,  323,  324,   44,  326, 
          327,   -1,   -1,   -1,   -1,   -1,  333,  334,  279,   -1, 
           -1,  328,   58,   59,   -1,   -1,   -1,   63,  335,  336, 
           -1,   -1,  349,   -1,  351,  394,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  328,   -1,   -1, 
           -1,   -1,   -1,   -1,  335,  336,   10,   -1,   -1,   -1, 
           -1,   -1,  257,  258,  259,   -1,  261,  262,  263,  264, 
          265,  266,  267,  268,  269,  270,  271,  272,  273,  274, 
          275,  276,  277,  278,   -1,  280,  281,  282,  283,  284, 
          285,  286,  287,  288,  289,  290,  291,  292,  293,  294, 
          295,  296,  297,   -1,  299,   59,   -1,  302,  303,  304, 
          305,  306,  307,  308,  309,  310,  311,  312,  313,  314, 
          315,  316,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   44,   -1,  333,  334, 
          335,  336,  337,   10,  339,  340,  341,  342,  343,  344, 
          345,  346,  347,  348,  349,  350,  351,  352,  353,  354, 
          355,  356,  357,  358,  359,  360,  361,  362,  363,   -1, 
          365,  366,  367,  368,  369,  370,   44,   -1,   -1,  374, 
          375,  376,  377,   91,  379,  380,  381,  382,   -1,   -1, 
           -1,   -1,   59,   -1,   -1,   10,   -1,  392,  393,  394, 
          395,  257,  258,  259,   -1,  261,  262,  263,  264,  265, 
          266,  267,  268,  269,  270,  271,  272,  273,  274,  275, 
          276,  277,  278,   91,  280,  281,  282,  283,  284,  285, 
          286,  287,  288,  289,  290,  291,  292,  293,  294,  295, 
          296,  297,   -1,  299,   59,   -1,  302,  303,  304,  305, 
          306,  307,  308,  309,  310,  311,  312,  313,  314,  315, 
          316,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   44,   91,  333,  334,  335, 
          336,  337,   -1,  339,  340,  341,  342,  343,  344,  345, 
          346,  347,  348,  349,  350,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,  362,  363,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,    0,  374,  375, 
          376,  377,   91,  379,  380,  381,  382,   10,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  392,  393,  394,  395, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  306,   -1,  308,  309,  310,  311,  312,   -1, 
           -1,   44,   -1,  317,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  279,  280,   -1,  328,   58,   59,   -1,   -1,   -1, 
           63,   -1,  336,   -1,   -1,   -1,  340,  341,   -1,   -1, 
           -1,   -1,   -1,   -1,  348,  349,  350,  351,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,  279,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336,   -1, 
           -1,   -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,  306, 
          394,  308,  309,  310,  311,  312,   -1,   -1,   -1,   -1, 
          317,   -1,  267,   -1,  362,   -1,    0,   -1,   -1,   -1, 
          328,  328,   -1,   -1,   -1,   -1,   10,  335,  336,  336, 
           -1,   -1,   -1,  340,  341,   -1,   -1,   -1,   -1,   -1, 
           -1,  348,  349,  350,  351,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  317,  318,  319,  320,  321,  322,  323,   -1, 
           44,  326,  327,   -1,   -1,   -1,   -1,   -1,  333,  334, 
          279,   -1,   -1,  328,   58,   59,   -1,   -1,   -1,   63, 
          335,  336,   -1,   -1,  349,   -1,  351,  394,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  328, 
           -1,   -1,   -1,   -1,   -1,   -1,  335,  336,   10,   -1, 
           -1,   -1,   -1,   -1,  257,  258,  259,   -1,  261,  262, 
          263,  264,  265,  266,  267,  268,  269,  270,  271,  272, 
          273,  274,  275,  276,  277,  278,   -1,  280,  281,  282, 
          283,  284,  285,  286,  287,  288,  289,  290,  291,  292, 
          293,  294,  295,  296,  297,   -1,  299,   59,   -1,  302, 
          303,  304,  305,  306,  307,  308,  309,  310,  311,  312, 
          313,  314,  315,  316,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   44,   -1, 
          333,  334,  335,  336,  337,   10,  339,  340,  341,  342, 
          343,  344,  345,  346,  347,  348,  349,  350,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,  362, 
          363,   -1,  365,  366,  367,  368,  369,  370,   44,   -1, 
           -1,  374,  375,  376,  377,   91,  379,  380,  381,  382, 
           -1,   -1,   -1,   -1,   59,   -1,   -1,   10,   -1,  392, 
          393,  394,  395,  257,  258,  259,   -1,  261,  262,  263, 
          264,  265,  266,  267,  268,  269,  270,  271,  272,  273, 
          274,  275,  276,  277,  278,   91,  280,  281,  282,  283, 
          284,  285,  286,  287,  288,  289,  290,  291,  292,  293, 
          294,  295,  296,  297,   -1,  299,   59,   -1,  302,  303, 
          304,  305,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   44,   91,  333, 
          334,  335,  336,  337,   -1,  339,  340,  341,  342,  343, 
          344,  345,  346,  347,  348,  349,  350,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,  362,  363, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,    0, 
          374,  375,  376,  377,   91,  379,  380,  381,  382,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  392,  393, 
          394,  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  306,   -1,  308,  309,  310,  311, 
          312,   -1,   -1,   44,   -1,  317,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  279,   -1,   -1,  328,   58,   59,   -1, 
           61,   -1,   63,   -1,  336,   -1,   -1,   -1,  340,  341, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           91,   -1,   -1,  279,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335, 
          336,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  306,  394,  308,  309,  310,  311,  312,   -1,   -1, 
           -1,   -1,  317,   -1,  267,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,  328,  328,   -1,   -1,   -1,   -1,   10,  335, 
          336,  336,   -1,   -1,   -1,  340,  341,   -1,   -1,   -1, 
           -1,   -1,   -1,  348,  349,  350,  351,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  279,   -1,   -1,  328,   -1,   59,   -1,   -1, 
           -1,   63,  335,  336,   -1,   -1,   -1,   -1,   -1,  394, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336, 
           10,   -1,   -1,   -1,   -1,   -1,  257,  258,  259,   -1, 
          261,  262,  263,  264,  265,  266,  267,  268,  269,  270, 
          271,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
          281,  282,  283,  284,  285,  286,  287,  288,  289,  290, 
          291,  292,  293,  294,  295,  296,  297,   -1,  299,   59, 
           -1,  302,  303,  304,  305,  306,  307,  308,  309,  310, 
          311,  312,  313,  314,  315,  316,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,  337,   -1,  339,  340, 
          341,  342,  343,  344,  345,  346,  347,  348,  349,  350, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,   -1,  363,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,  375,  376,  377,   -1,  379,  380, 
          381,  382,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  392,  393,  394,  395,  257,  258,  259,   -1,  261, 
          262,  263,  264,  265,  266,  267,  268,  269,  270,  271, 
          272,  273,  274,  275,  276,  277,  278,   -1,  280,  281, 
          282,  283,  284,  285,  286,  287,  288,  289,  290,  291, 
          292,  293,  294,  295,  296,  297,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
          312,  313,  314,  315,  316,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,  337,   -1,  339,  340,   -1, 
          342,  343,  344,  345,  346,  347,  348,  349,  350,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          362,  363,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,    0,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          392,  393,  394,  395,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  306,   -1,  308,  309, 
          310,  311,  312,   -1,   -1,   44,   -1,  317,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  328,   -1, 
           59,   -1,   -1,   -1,   63,   -1,  336,   -1,   -1,   -1, 
          340,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349, 
          350,  351,   -1,   -1,   -1,  257,  258,  259,   -1,  261, 
           -1,   -1,   91,  265,  266,   -1,   -1,   -1,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,  394,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
            0,  313,   -1,   -1,  316,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   59, 
           -1,   -1,  374,   63,  376,  377,   -1,  379,  380,  381, 
          382,  306,   -1,  308,  309,  310,  311,  312,   -1,   -1, 
          392,  393,  317,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  348,  349,  350,  351,   -1,  257,  258, 
          259,   -1,  261,  262,  263,  264,  265,  266,  267,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           -1,  280,  281,  282,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,  394, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,    0,   -1,  333,  334,  335,  336,  337,   -1, 
          339,  340,   10,  342,  343,  344,  345,  346,  347,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  362,  363,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   44,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,   -1,   -1, 
           58,   59,   -1,  392,  393,  394,  395,  257,  258,  259, 
           -1,  261,  262,  263,  264,  265,  266,  267,  268,  269, 
          270,  271,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,  281,  282,  283,  284,  285,  286,  287,  288,  289, 
          290,  291,  292,  293,  294,  295,  296,  297,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,  337,   -1,  339, 
          340,   -1,  342,  343,  344,  345,  346,  347,  348,  349, 
          350,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,    0,  363,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   10,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  392,  393,  394,  395,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   59,   -1,   61,   -1,   63,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   91,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,   -1,   -1,   -1,   -1,  333,  334, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,   -1,   -1,   -1,  349,   44,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,  363,  361,  306,  307,   -1, 
           59,  310,   61,   -1,   63,  314,  315,  375,  317,  318, 
          319,  320,  321,  322,  323,   -1,   -1,  326,  327,   -1, 
           -1,   -1,  331,  332,  333,  334,   -1,  395,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,  348, 
          349,   -1,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,   -1,   -1,  364,   -1,   -1,   -1,  257, 
          258,  259,   -1,  261,  262,  263,  264,  265,  266,   -1, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,   -1,  280,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,  293,  294,  295,  296,  297, 
           -1,  299,   -1,   -1,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,  312,  313,  314,  315,  316,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   63,   -1,  333,  334,  335,  336,  337, 
          338,   10,  340,  341,  342,  343,  344,   -1,  346,   -1, 
          348,  349,  350,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,  362,  363,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   44,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   61,   -1,  392,  393,  394,  395,  257,  258, 
          259,   -1,  261,  262,  263,  264,  265,  266,   -1,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           -1,  280,   91,   -1,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   63,   -1,  333,  334,  335,  336,  337,  338, 
           -1,  340,  341,  342,  343,  344,   -1,  346,   -1,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  362,  363,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,    0,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   10,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  392,  393,  394,  395,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,   -1, 
           -1,   -1,   -1,   59,   -1,   61,   -1,   63,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   -1,   -1,  333,  334,   91,   -1,   -1,   -1,   -1, 
          279,  280,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,  328, 
           -1,   -1,   -1,   10,   -1,   -1,  335,  336,   -1,   -1, 
           -1,   -1,  341,  342,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  362,   -1,   -1,   -1,   44,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,   -1,   -1, 
           -1,   -1,   59,   -1,   61,   -1,   63,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,   -1,  329,  330, 
           -1,   -1,  333,  334,   91,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1, 
      };
   }

   private static final short[] yyCheck2() {
      return new short[] {

           -1,   -1,   -1,   -1,  349,   -1,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,  257,  258,  259, 
           -1,  261,  262,  263,  264,  265,  266,   -1,  268,  269, 
          270,  271,  272,  273,  274,  275,  276,  277,  278,   -1, 
          280,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
          290,  291,  292,  293,  294,  295,  296,  297,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   63,   -1,  333,  334,  335,  336,  337,   -1,   10, 
          340,  341,  342,  343,  344,   -1,  346,   -1,  348,  349, 
          350,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,  362,  363,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   44,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           61,   -1,  392,  393,  394,  395,  257,  258,  259,   -1, 
          261,  262,  263,  264,  265,  266,   -1,  268,  269,  270, 
          271,  272,  273,  274,  275,  276,  277,  278,   -1,  280, 
           91,   -1,  283,  284,  285,  286,  287,  288,  289,  290, 
          291,  292,  293,  294,  295,  296,  297,   -1,  299,   -1, 
           -1,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,  312,  313,  314,  315,  316,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,  337,   -1,   -1,  340, 
          341,  342,  343,  344,   -1,  346,   -1,  348,  349,  350, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,  362,  363,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,    0,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   10,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,  392,  393,  394,  395,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   59,   -1,   61,   -1,   63,   61,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,   -1,  329,  330,   -1, 
           -1,  333,  334,   91,   -1,   -1,   91,   -1,  279,  280, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  349,   -1,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,   -1,  328,   -1,   -1, 
           -1,   10,   -1,   -1,  335,  336,   -1,   -1,   -1,   -1, 
          341,  342,   -1,  306,  307,   -1,   -1,  310,   -1,   -1, 
           -1,  314,  315,   -1,  317,  318,  319,  320,  321,  322, 
          323,  362,   -1,  326,  327,   44,   -1,   -1,  331,  332, 
          333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341,   -1, 
           59,   -1,   61,   -1,   63,  348,  349,   -1,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,   -1, 
           -1,  364,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257, 
          258,  259,   -1,  261,  262,  263,  264,  265,  266,   -1, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,   -1,  280,   -1,  279,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,  293,  294,  295,  296,  297, 
           -1,  299,   -1,   -1,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,  312,  313,  314,  315,  316,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,  328,   -1,  333,  334,  335,  336,  337, 
          335,  336,  340,  341,  342,  343,  344,  342,  346,   -1, 
          348,  349,  350,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,  362,  363,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
            0,  379,  380,  381,  382,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,  392,  393,  394,  395,  257,  258, 
          259,   -1,  261,  262,  263,  264,  265,  266,   -1,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   59, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,  337,   -1, 
           -1,  340,  341,  342,  343,  344,   -1,  346,   -1,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,   -1,  363,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,    0,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   10,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  392,  393,  394,  395,   -1,   -1,  306, 
          307,   -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1, 
          317,  318,  319,  320,  321,  322,  323,   -1,   44,  326, 
          327,   -1,   -1,   -1,  331,  332,  333,  334,   -1,   -1, 
           -1,   -1,   58,   59,  341,   -1,   -1,   63,   -1,   -1, 
           -1,  348,  349,   -1,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,   -1,   -1,  364,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,  262,  263,  264,  265,  266,  267,  268,  269, 
          270,  271,  272,  273,  274,  275,  276,  277,  278,   -1, 
          280,  281,   -1,  283,  284,  285,  286,  287,  288,  289, 
          290,  291,  292,  293,  294,  295,  296,  297,   59,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  328,   -1, 
           -1,   -1,   -1,   -1,   -1,  335,  336,  337,   -1,   -1, 
          340,   -1,  342,  343,  344,   -1,  346,   -1,  348,   -1, 
          350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,  362,  363,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,    0,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,  392,  393,  394,  395,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  280,  281,  282,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   58,   59,   -1,   61,   -1,   63,  305, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   91,  333,  334,  335, 
          336,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,  349,    0,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   10,  361,  362,  363,   -1,   -1, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,  375, 
          271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          281,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44,  395, 
           -1,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   58,   59,   -1,   61,   -1,   63,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   10, 
           -1,  342,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,    0,   -1,   -1,   10,   -1,   10,   -1, 
           -1,   10,  363,   10,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1, 
           44,   -1,   44,   -1,  395,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   61,   91,   61, 
           59,   -1,   59,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           91,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   91,   -1,   91, 
          305,    0,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,   -1,  338,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,  349,   44,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,   -1,  363,   58, 
           59,   -1,   61,   -1,   63,   -1,  262,  263,  264,   -1, 
          375,  267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1, 
          395,   -1,   91,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,   -1,  338,  339,  267,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,  349,   44,  351,  267,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,   -1,  363,   58,   59, 
           -1,   61,   -1,   63,   -1,  262,  263,  264,  267,  375, 
          267,  268,  269,   -1,  271,  279,   -1,  279,   -1,   -1, 
           -1,  280,   -1,   -1,  281,  282,   -1,   -1,   -1,  395, 
           -1,   91,   -1,  290,  291,  328,  293,  294,  295,  296, 
          297,   -1,  335,  336,   -1,   -1,   -1,  328,   -1,    0, 
           -1,   -1,   -1,   10,  335,  336,   -1,   -1,   10,   10, 
           -1,   -1,   10,   -1,  328,   -1,  328,   -1,   -1,  328, 
           -1,  335,  336,  335,  336,   -1,  335,  336,  342,   -1, 
          342,   -1,  341,   -1,   -1,  342,   -1,   44,  345,   -1, 
          347,   -1,   44,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,  362,   61,   -1,  363,   -1,   59,   61, 
           -1,   59,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   91,   -1,  305,    0,  395,   91, 
           -1,   -1,   -1,   91,   -1,   -1,   -1,   10,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,   -1,  338, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1, 
          349,   44,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,  363,   58,   59,   -1,   61,   -1, 
           63,   -1,  262,  263,  264,   -1,  375,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   -1,  395,   -1,   91,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,   -1,  338,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
           44,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   58,   59,   -1,   61,   -1,   63, 
           -1,  262,  263,  264,   -1,  375,  267,  268,  269,  267, 
          271,   -1,  279,   -1,   -1,   -1,   -1,  279,  280,   -1, 
          281,  282,   -1,   -1,   -1,  395,   -1,   91,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,   10,   -1,   -1,   -1,   -1, 
           -1,  328,   -1,   -1,   -1,   -1,  328,   -1,  335,  336, 
          328,   -1,   -1,  335,  336,  342,   -1,  335,  336,  341, 
          342,  342,   -1,   -1,  345,   -1,   -1,   -1,   44,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1, 
          362,   -1,  363,   -1,   59,   61,   -1,   -1,  281,  282, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,    0,  395,   91,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,  338,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,   44,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,   58,   59,   -1,   61,   -1,   63,   -1,  262,  263, 
          264,   -1,  375,  267,  268,  269,   -1,  271,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,  395,   -1,   91,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,  338,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,  349,   44,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,   -1,  363, 
           58,   59,   -1,   61,   -1,   63,   -1,  262,  263,  264, 
           -1,  375,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,  279,  280,   -1,  281,  282,   -1,   -1, 
           -1,  395,   -1,   91,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,    0,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335, 
          336,   -1,   -1,   -1,   -1,  341,  342,  342,   44,   -1, 
          345,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   44, 
          267,  268,  269,   -1,  271,   61,  362,   -1,  363,   -1, 
           59,   -1,   -1,   -1,  281,  282,   61,   -1,   -1,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   91,   -1,   -1,  305,    0, 
          395,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   10, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
           -1,  338,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,  349,   44,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   -1,  363,   58,   59,   -1, 
           61,   -1,   63,   -1,  262,  263,  264,   -1,  375,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,  395,   -1, 
           91,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   -1,  333,  334,  335,  336,   -1, 
          338,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,  349,   44,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   58,   59,   -1,   61, 
           -1,   63,   -1,  262,  263,  264,   -1,  375,  267,  268, 
          269,   -1,  271,  279,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  281,  282,  279,   -1,   -1,  395,   -1,   91, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   10,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1, 
           -1,   10,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335, 
          336,   -1,   -1,  328,   -1,   -1,  342,   -1,   -1,   -1, 
          335,  336,   44,  342,   44,   -1,  345,  342,   -1,   -1, 
           -1,  262,  263,  264,   -1,   44,  267,  268,  269,   61, 
          271,   61,   -1,   -1,  363,   -1,   59,   -1,   -1,   -1, 
          281,  282,   61,   -1,   -1,   -1,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   91, 
           -1,   91,   -1,   -1,  305,    0,  395,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,   10,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,   -1,  338,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349,   44, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,  363,   58,   59,   -1,   -1,   -1,   63,   -1, 
          262,  263,  264,   -1,  375,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,  395,   -1,   91,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,    0,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,  338,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   44,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,   58,   59,   -1,   61,   -1,   63,   -1,  262, 
          263,  264,   -1,  375,  267,  268,  269,  279,  271,  279, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,   -1, 
          279,   -1,   -1,  395,   -1,   91,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,    0,   -1,   -1,   10,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,   -1,   -1,  328,   -1,  328,   -1, 
           -1,   -1,   -1,  335,  336,  335,  336,   -1,   -1,  328, 
          342,   -1,  342,   -1,   -1,   -1,  335,  336,   -1,  342, 
           44,   -1,   -1,  342,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   61,   -1,   -1, 
          363,   -1,   59,   -1,   -1,  280,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   91,   -1,   -1, 
          305,    0,  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,  349,   44,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,  362,  363,   58, 
           59,   -1,   61,   -1,   63,   -1,  262,  263,  264,   -1, 
          375,  267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1, 
          395,   -1,   91,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,  349,   44,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,   -1,  363,   58,   59, 
           -1,   -1,   -1,   63,   -1,  262,  263,  264,   -1,  375, 
          267,  268,  269,   -1,  271,  279,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  281,   -1,   -1,   -1,   -1,  395, 
           -1,   91,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,   -1,   10,   -1,  328,   -1,   -1,   -1,   -1,   -1, 
           -1,  335,  336,   -1,   -1,   -1,   -1,   -1,  342,   -1, 
           -1,   -1,   -1,   -1,   -1,  342,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,   -1,   -1,   -1,  363,   -1,   59,   -1, 
           -1,   59,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  305,    0,  395,   -1, 
           -1,   -1,   -1,   91,   -1,   -1,   -1,   10,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,   -1,   -1, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1, 
          349,   44,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,  363,   58,   59,   -1,   -1,   -1, 
           63,   -1,  262,  263,  264,   -1,  375,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   -1,  395,   -1,   91,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
           44,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   58,   59,   -1,   -1,   -1,   63, 
           -1,  262,  263,  264,   -1,  375,  267,  268,  269,  267, 
          271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          281,   -1,  280,   -1,   -1,  395,   -1,   91,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336,   -1, 
           -1,  342,   -1,  341,   -1,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1, 
           -1,   -1,  363,   -1,  362,   59,   -1,   -1,  281,  282, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,    0,  395,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,   44,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,   58,   59,   -1,   -1,   -1,   63,   -1,  262,  263, 
          264,   -1,  375,  267,  268,  269,   -1,  271,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,  395,   -1,   91,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,  349,   44,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,   -1,  363, 
           58,   59,   -1,   -1,   -1,   63,   -1,   -1,  262,  263, 
          264,  375,   -1,   -1,  268,  269,   -1,  271,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  395,   -1,   91,   -1,   -1,   -1,   -1,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,   10,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,  342,   -1, 
           -1,   10,   -1,   -1,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,  363, 
           -1,   59,   -1,   59,  281,  282,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  290,  291,   44,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,    0, 
           59,  395,   -1,   91,   -1,   91,   -1,   -1,   -1,   10, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,  349,   44,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   -1,  363,   -1,   59,   -1, 
           -1,   -1,   63,   -1,  262,  263,  264,   -1,  375,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,  395,   -1, 
           91,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   -1,  333,  334,  335,  336,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,  349,   44,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   -1,   59,   -1,   -1, 
           -1,   63,   -1,   -1,   -1,   -1,   -1,  375,   -1,  267, 
           -1,  267,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  280,   -1,  280,   -1,   -1,  395,   -1,   91, 
           -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,   -1,    0,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  281,  282,   10,   -1,   -1,   -1,   -1,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
          328,   -1,  328,   -1,   -1,   -1,   -1,  335,  336,  335, 
          336,   -1,   -1,  341,   -1,  341,   -1,   -1,   44,   -1, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1, 
          271,   -1,   -1,   59,  362,   -1,  362,   63,   -1,   -1, 
          281,  282,   -1,  342,   -1,   -1,  345,   -1,  347,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  363,   91,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,  395,    0,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   10,  349,   -1, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,  395,   58,   59,   -1,  290,  291, 
           63,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,    0,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   10,  361, 
           -1,  363,   -1,   -1,   -1,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1, 
           -1,   -1,   44,  395,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1, 
           -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,  349,   -1,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,   -1,  363,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,  395, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   44,   -1,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   58,   59,   -1,   -1, 
           -1,   63,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,   -1,  329,  330,   -1,   -1, 
          333,  334,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,    0,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   10,  361,   -1, 
          363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,   44,  395,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   58,   59,   -1,  290,  291, 
           63,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,   -1,  329,  330,   -1, 
           -1,  333,  334,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   -1,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,  375,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,  395,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   44,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           58,   59,   -1,  305,   -1,   63,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,   -1,  329,  330,   -1, 
           -1,  333,  334,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   -1,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,  375,  267,  268,  269,   -1,  271,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           10,   -1,   -1,  395,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,   -1,  329,  330,   58,   59, 
          333,  334,   -1,   63,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,   -1,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  375,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  395,   -1,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   44,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   59,   -1,   61,   -1,   63,  305,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
           -1,  329,  330,   -1,   91,  333,  334,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,  349,   -1,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,  375,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  395,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           44,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   59,   -1,   61,   10,   63, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   -1,   -1,  333,  334,   -1,   -1,   -1,   -1,  339, 
           -1,   63,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   -1,  262,  263,  264,   -1,   -1, 
           -1,  268,  269,   -1,  271,  375,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  280,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  290,  291,  395,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
           -1,   -1,   -1,   -1,   -1,  342,   -1,   -1,   -1,   -1, 
           -1,   -1,  349,   44,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,  362,  363,   -1,   59,   -1, 
           61,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,   -1,   -1,  268,  269,   -1,  271,  395,   -1, 
           91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,  338,   -1,   -1,   -1,  342,   -1, 
           -1,   44,   -1,   -1,   -1,  349,   -1,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   59,  361,   61,  363, 
           63,   -1,   -1,   -1,   -1,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,   -1,  329,  330,   -1, 
           -1,  333,  334,   -1,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,  395,   -1,   -1,   -1,   -1,   -1,  349,   -1,  351, 
            0,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  262,  263,  264,   44,   -1,   -1,  268,  269,   -1, 
          271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59, 
           -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   91,   -1,   -1,   -1,   -1,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
            0,   -1,  333,  334,  335,  336,   -1,  338,   -1,   -1, 
           10,  342,   -1,   -1,   -1,   -1,   -1,   -1,  349,   -1, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,   -1,   -1,  268,  269,   -1,  271,   59, 
           -1,   61,   -1,   63,  395,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,  338,   -1,   -1,   -1,  342, 
           -1,   -1,   -1,   -1,   -1,   -1,  349,   -1,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,   -1,  262,  263,  264,   -1,   -1,   -1,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          290,  291,  395,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,   -1,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,   -1,  338,   -1, 
           -1,   -1,  342,   -1,   -1,   44,   -1,   -1,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           59,  361,   61,  363,   63,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,   -1,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,  395,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,   -1,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,   -1,  338,   -1, 
           -1,   -1,  342,   -1,   -1,   44,   -1,   -1,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           59,  361,   61,  363,   63, 
      };
   }

   private static final short[] yyCheck3() {
      return new short[] {

           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  306,  307, 
           -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   91,  326,  327, 
          395,   -1,   -1,  331,  332,  333,  334,   -1,   -1,    0, 
           -1,   -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,   10, 
          348,  349,   -1,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,   -1,   -1,  364,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,   -1,   -1,  268,  269,   -1,  271,   59,   -1, 
           61,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,    0,   -1,  333, 
          334,  335,  336,   -1,  338,   -1,   -1,   10,  342,   -1, 
           -1,   -1,   -1,   -1,   -1,  349,   -1,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,   -1,  363, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,   44,   -1,   -1,  268,  269,   -1,  271,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,   61,   -1, 
           63,  395,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,  338,   -1,   -1,   -1,  342,   -1, 
           -1,   -1,   -1,   -1,   -1,  349,   -1,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,   -1,  363, 
           -1,  262,  263,  264,   -1,   -1,   -1,  268,  269,   -1, 
          271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290, 
          291,  395,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,   -1,  338,   -1,   -1, 
           -1,  342,   -1,   -1,   44,   -1,   -1,   -1,  349,   -1, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   59, 
          361,   61,  363,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,   -1,   -1,  268,  269,   -1,  271,   -1, 
           -1,   91,   -1,   -1,  395,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1, 
           -1,   -1,   -1,   -1,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,  338,   -1,   -1,   -1,  342, 
           -1,   -1,   44,   -1,   -1,   -1,  349,   -1,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   59,  361,   61, 
          363,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          306,  307,   -1,   -1,  310,   -1,   -1,   -1,  314,  315, 
           -1,  317,  318,  319,  320,  321,  322,  323,   -1,   91, 
          326,  327,  395,   -1,   -1,  331,  332,  333,  334,   -1, 
           -1,    0,   -1,   -1,   -1,  341,   -1,   -1,   -1,   -1, 
           -1,   10,  348,  349,   -1,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,   -1,   -1,  364,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   44,   -1,   -1,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           59,   -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,   -1,  338,   -1, 
            0,   -1,  342,   -1,   -1,   -1,   -1,   -1,   -1,  349, 
           10,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   44,   -1,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,   -1,  395,   -1,   -1,   58,   59, 
           -1,   -1,   -1,   63,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,   -1,   -1,   -1,   -1, 
          342,   -1,   -1,   -1,   -1,   -1,   -1,  349,   -1,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,   -1,  262,  263,  264,   -1,   -1,   -1,  268, 
          269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  290,  291,  395,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,   -1,   -1, 
           -1,   -1,   -1,  342,   -1,   -1,   -1,   -1,   -1,   -1, 
          349,   44,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,  363,   58,   59,   -1,   -1,   -1, 
           63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,  395,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   -1,   -1,  333,  334,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1, 
           44,  351,   -1,   -1,   -1,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   58,   59,   -1,   -1,   -1,   63, 
           -1,   -1,   -1,   -1,   -1,  375,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  306,  307,   -1,   -1,  310, 
           -1,   -1,   -1,  314,  315,  395,  317,  318,  319,  320, 
          321,  322,  323,   -1,   -1,  326,  327,   -1,   -1,   -1, 
          331,  332,  333,  334,   -1,   -1,    0,   -1,   -1,   -1, 
          341,   -1,   -1,   -1,   -1,   -1,   10,  348,  349,   -1, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1, 
           44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,   -1,   58,   59,   -1,  290,  291,   63, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,   -1,  329,  330,   -1,   -1, 
          333,  334,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,   -1,   -1,  351,   -1, 
           -1,   -1,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,  375,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1, 
           -1,   -1,  395,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,   -1,   -1,   -1,   44,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   58, 
           59,  305,   -1,   -1,   63,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,   -1,  329,  330,   -1,   -1,  333, 
          334,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,   -1,   -1,  351,   -1,   -1, 
            0,   -1,   -1,  357,  358,  359,   -1,  361,   -1,  363, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,  375,   -1,  267,  268,  269,   -1,  271,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,  395,   -1,   -1,   44,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   58,   59, 
           -1,  305,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,   -1,  329,  330,   -1,   -1,  333, 
          334,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,   -1,   -1,  351,   -1,   -1, 
           -1,   -1,   -1,  357,  358,  359,   -1,  361,   -1,  363, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  395,   -1,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   58,   59, 
           -1,  290,  291,   63,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,   -1, 
          329,  330,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   10, 
           -1,   -1,  351,   -1,   -1,   -1,   -1,   -1,  357,  358, 
          359,   -1,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,  375,  267,  268,  269, 
           -1,  271,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   -1,  395,   58,   59,   -1, 
          290,  291,   63,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,    0,  347,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,  357,  358,  359, 
           -1,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  375,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           44,  271,   -1,   -1,   -1,  395,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   58,   59,   -1,   -1,   -1,   63, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  357,  358,  359, 
           -1,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  262,  263,  264,   -1,  375,  267,  268,  269,   -1, 
          271,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   10,   -1,   -1,  395,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,   -1,  329,  330, 
           58,   59,   -1,   -1,   -1,   63,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  357,  358,   -1,   -1, 
           -1,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  375,   -1,   -1,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,    0, 
           -1,   -1,   -1,   -1,  395,   -1,   -1,  281,  282,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,   -1,  329,  330,   58,   59,   -1, 
           -1,   -1,   63,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,  363, 
           -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1, 
           -1,  375,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           44,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,   58,   59,   -1,   -1,   -1,   63, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,    0,   -1,   -1,  305,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
           -1,  329,  330,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   44,  347, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  357, 
          358,   -1,   58,   59,   -1,  363,   -1,   63,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  375,   -1,   -1, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1, 
          271,   -1,    0,   -1,   -1,   -1,   -1,  395,   -1,   -1, 
          281,  282,   10,   -1,   -1,   -1,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,   -1,  329,  330, 
           58,   59,   -1,   -1,   -1,   63,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  357,  358,  262,  263, 
          264,   -1,  363,  267,  268,  269,   -1,  271,   -1,    0, 
           -1,   -1,   -1,   -1,  375,   -1,   -1,  281,  282,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,  395,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1, 
          324,  325,   -1,   -1,   -1,  329,  330,   58,   59,   -1, 
           -1,   -1,   63,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,    0,   -1,  363, 
           -1,   -1,   -1,   -1,   -1,  281,  282,   10,   -1,   -1, 
           -1,  375,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305, 
           -1,  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,  324,  325, 
           -1,   -1,   -1,  329,  330,   58,   59,   -1,   -1,   -1, 
           63,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,   -1,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,   -1,    0,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   10,   -1,   -1,   -1,  375, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1,  395, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,   -1,  324,  325,   -1,   -1, 
           -1,  329,  330,   58,   59,   -1,   -1,   -1,   63,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  262,  263,  264,   -1,  363,  267,  268,  269,   -1, 
          271,   -1,    0,   -1,   -1,   -1,   -1,  375,   -1,   -1, 
          281,  282,   10,   -1,   -1,   -1,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,  395,   -1,   -1, 
           -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   -1, 
           -1,   -1,   -1,  324,  325,   -1,   -1,   -1,  329,  330, 
           58,   59,   -1,   -1,   -1,   63,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1, 
            0,   -1,  363,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           10,   -1,   -1,   -1,  375,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,   -1,  395,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
           -1,  324,  325,   -1,   -1,   -1,  329,  330,   58,   59, 
           -1,   -1,   -1,   63,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,    0,   -1, 
          363,   -1,   -1,   -1,   -1,   -1,  281,  282,   10,   -1, 
           -1,   -1,  375,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,   -1,  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,  324, 
          325,   -1,   -1,   -1,  329,  330,   58,   59,   -1,   -1, 
           -1,   63,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,    0,  363,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   10,   -1,   -1, 
          375,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
          395,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   58,   59,  325,   -1,   -1, 
           -1,  329,  330,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,   -1,  262,  263,  264,   -1,    0,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,  363,   10,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   -1,   -1,  375,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,   -1,  395,   -1,   -1, 
           44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   58,   59,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,  363,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   44,   -1,   -1,  375,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   58,   59,   -1,   -1, 
           -1,   -1,   -1,  305,   -1,  395,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,    0,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   10,  271,   -1, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,  375,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,  305,  395,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,    0,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   10,  271,   -1,   -1, 
          363,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,  375,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           44,  305,  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   58,   59,   -1,   -1,    0,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   10,  271, 
           -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342,  281, 
          282,  345,   -1,  347,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,  363, 
           -1,   -1,   44,  305,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  375,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  395,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,  375,  267,  268,  269,   -1,  271,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   44,   -1,  395,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   58,   59,   -1,   -1,   -1, 
           -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,    0,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   10,  271,   -1,   -1, 
          363,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,  375,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           44,  305,  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   58,   59,   -1,   -1,    0,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   10,  271, 
           -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342,  281, 
          282,  345,   -1,  347,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,  363, 
           -1,   -1,   44,  305,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  375,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  395,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,    0,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   10,  271,   -1, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,  375,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,  305,  395,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          363,   -1,   -1,   44,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,  375,  267,  268,  269,   -1,  271,   59,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,  395,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,    0,   -1, 
           -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1, 
           -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   10,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342,  281, 
          282,  345,   44,  347,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   58,   59,   44,  363, 
           -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  375,   -1,   59,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  395,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,    0,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   10,  271,   -1, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,  375,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,  305,  395,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1,    0, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   10, 
          271,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342, 
          281,  282,  345,   -1,  347,   -1,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
          363,   -1,   -1,   44,  305,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  375,   -1,   -1,   -1,   -1,    0,   59,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1, 
           -1,   -1,  395,   -1,   -1,   -1,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,   44,  363,   -1,   -1,   -1,  262,  263,  264,  281, 
          282,  267,  268,  269,  375,  271,   59,    0,  290,  291, 
           -1,  293,  294,  295,  296,   -1,   -1,   10,   -1,   -1, 
           -1,   -1,   -1,  305,  395,   -1,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,   -1,   10,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   59,   -1,   -1,   -1, 
           -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,   -1, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   44,   -1,  262, 
          263,  264,   -1,  375,  267,  268,  269,  363,  271,   -1, 
            0,   -1,   59,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           10,   -1,   -1,  395,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,   -1,   -1,   -1,   -1,   -1,  395, 
           -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   59, 
          271,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342, 
          281,  282,  345,   -1,  347,   -1,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
          363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  375,   -1,   10,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1, 
           -1,   -1,  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1, 
           -1,   -1,   -1,   -1,  395,   -1,  339,   -1,   -1,  342, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          293,  294,  295,  296,  297,  262,  263,  264,   -1,   -1, 
          363,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
           -1,   -1,  395,   10,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,   -1,  268,  269, 
          363,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  342,   -1,   -1,   -1,   -1, 
           -1,   -1,   59,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,  395,   -1,   -1,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  395,   -1, 
           -1,   -1,  342,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  257,  258,  259,   -1,  261,   -1,   -1,   -1,  265, 
          266,   -1,   -1,  363,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   10,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,   -1,  299,   -1,  395,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,  312,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   59,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,  350,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   91,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  392,  393,  394,  256, 
          257,  258,  259,  260,  261,  262,  263,  264,  265,  266, 
           -1,   -1,  269,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,  280,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   10,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,  298,  299,   -1,  301,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336, 
          337,   -1,   59,  340,  341,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
          357,   -1,   -1,  360,   -1,  362,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   91,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  392,  393,   -1,   -1,   -1, 
          256,  257,  258,  259,  260,  261,  262,  263,  264,  265, 
          266,   -1,   -1,  269,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,  280,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   10,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,   -1,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335, 
          336,  337,   -1,   59,  340,  341,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,  357,   -1,   -1,  360,   -1,  362,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   91,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  392,  393,   -1,  256, 
          257,  258,  259,  260,  261,  262,  263,  264,  265,  266, 
           -1,   -1,  269,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,  280,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   10,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,  298,  299,   -1,  301,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336, 
          337,   -1,   59,  340,  341,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
          357,   -1,   -1,  360,   -1,  362,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   91,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  392,  393,   -1,   -1,   -1, 
          256,  257,  258,  259,  260,  261,  262,  263,  264,  265, 
          266,   -1,   -1,  269,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   10,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,   -1,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335, 
          336,  337,   -1,   59,  340,  341,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,  357,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  392,  393,   -1,  256, 
          257,  258,  259,  260,  261,  262,  263,  264,  265,  266, 
           -1,   -1,  269,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   10,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,  298,  299,   -1,  301,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336, 
          337,   -1,   59,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
          357,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  392,  393,   -1,   -1,   -1, 
          256,  257,  258,  259,  260,  261,  262,  263,  264,  265, 
          266,   -1,  268,  269,  270,  271,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   10,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,   -1,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   59,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  392,  393,   -1,  256, 
          257,  258,  259,  260,  261,  262,  263,  264,  265,  266, 
           -1,   -1,  269,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   10,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,  298,  299,   -1,  301,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   59,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,  362,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  392,  393,   -1,   -1,   -1, 
          256,  257,  258,  259,  260,  261,  262,  263,  264,  265, 
          266,   -1,   -1,  269,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   10,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,   -1,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   59,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,   -1, 
          256,  257,  258,  259,  260,  261,  392,  393,  264,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   10,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,   -1,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   59,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1, 
      };
   }

   private static final short[] yyCheck4() {
      return new short[] {

          340,   -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  392,  393,   -1,  256,  257,  258,  259,  260, 
          261,   -1,   -1,   -1,  265,  266,   -1,   -1,   10,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,  298,  299,   -1, 
          301,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,  363,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,   -1,   -1,   -1,  257,  258,  259,   -1, 
          261,  392,  393,   -1,  265,  266,   -1,   10,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1, 
           -1,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,   -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,   -1,   -1,  257,  258,  259,   -1,  261, 
           -1,  392,  393,  265,  266,   -1,   -1,   -1,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           44,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
          342,  343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1, 
          352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,   -1,   -1,  257,  258,  259,   -1,  261,   -1, 
          392,  393,  265,  266,   -1,   -1,   -1,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   59,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1,  352, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  392, 
          393,   -1,   -1,  257,  258,  259,   -1,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
           -1,   -1,  316,   59,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
           -1,   -1,  257,  258,  259,  260,  261,   -1,  392,  393, 
          265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,  298,  299,  300,  301,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,  313,  314, 
          315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344, 
           -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,   -1, 
           -1,  257,  258,  259,  260,  261,   -1,  392,  393,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,   -1,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,  256, 
          257,  258,  259,  260,  261,   -1,  392,  393,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,  298,  299,  300,  301,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  256,  257, 
          258,  259,  260,  261,   -1,  392,  393,  265,  266,   -1, 
           -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
          298,  299,   -1,  301,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
          348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,  256,  257,  258, 
          259,  260,  261,   -1,  392,  393,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,  298, 
          299,   -1,  301,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,   -1,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,   -1,  392,  393,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
          350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,  392,  393,  394,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
          350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,  392,  393,  394,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
          350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,  392,  393,  394,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,  392,  393,  394,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1,   -1, 
          350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,  392,  393,  394,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,  392,  393,  394,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,   -1,  257,  258,  259, 
          260,  261,  392,  393,  394,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,  298,  299, 
          300,  301,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,   -1,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,  257,  258,  259,  260, 
          261,   -1,  392,  393,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,  298,  299,   -1, 
          301,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,   -1,   -1,  257,  258,  259,   -1,  261, 
           -1,  392,  393,  265,  266,   -1,   -1,   -1,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,  341, 
           -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1, 
          352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,   -1,   -1,  257,  258,  259,   -1,  261,   -1, 
          392,  393,  265,  266,   -1,   -1,   -1,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,   -1,   -1,  257,  258,  259,   -1,  261,   -1,  392, 
          393,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
           -1,   -1,  257,  258,  259,   -1,  261,   -1,  392,  393, 
          265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,  313,  314, 
          315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344, 
           -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,   -1, 
           -1,  257,  258,  259,   -1,  261,   -1,  392,  393,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,   -1, 
          257,  258,  259,   -1,  261,   -1,  392,  393,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,   -1,  257, 
          258,  259,   -1,  261,   -1,  392,  393,  265,  266,   -1, 
           -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
           -1,  299,   -1,   -1,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,  341,   -1,  343,  344,   -1,  346,   -1, 
           -1,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,   -1,  392,  393,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,   -1,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,   -1,  392,  393,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,   -1,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,   -1,  257,  258,  259,   -1, 
          261,   -1,  392,  393,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1, 
           -1,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,   -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,   -1,   -1,  257,  258,  259,   -1,  261, 
           -1,  392,  393,  265,  266,   -1,   -1,   -1,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1, 
          352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,   -1,   -1,  257,  258,  259,   -1,  261,   -1, 
          392,  393,  265,  266,   -1,   -1,   -1,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,   -1,   -1,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          333,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,   -1,   -1,  257,  258,  259,   -1,  261,   -1,  392, 
          393,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
           -1,   -1,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
           -1,   -1,  257,  258,  259,   -1,  261,   -1,  392,  393, 
          265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,  313,   -1, 
           -1,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344, 
           -1,  346,   -1,  348,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,   -1, 
           -1,  257,  258,  259,   -1,  261,   -1,  392,  393,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,   -1,   -1, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,   -1, 
          257,  258,  259,   -1,  261,   -1,  392,  393,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,   -1,   -1,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,   -1,  257, 
          258,  259,   -1,  261,   -1,  392,  393,  265,  266,   -1, 
           -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
           -1,  299,   -1,   -1,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,   -1,   -1,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,   -1,  392,  393,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,   -1,  313,   -1,   -1,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  392,  393,  257,  258,  259,  260,  261, 
          262,  263,  264,  265,  266,  267,  268,  269,  270,  271, 
          272,  273,  274,  275,  276,  277,  278,  279,  280,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,  290,  291, 
          292,  293,  294,  295,  296,  297,  298,  299,  300,  301, 
          302,  303,  304,   -1,  306,  307,  308,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,   -1,   -1,  326,  327,   -1,   -1,   -1,  331, 
          332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
           -1,   -1,  364,  365,  257,  258,  259,  260,  261,  262, 
          263,  264,  265,  266,  267,  268,  269,  270,  271,  272, 
          273,  274,  275,  276,  277,  278,  279,  280,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,  290,  291,  292, 
          293,  294,  295,  296,  297,  298,  299,  300,  301,  302, 
          303,  304,   -1,  306,  307,   -1,   -1,  310,   -1,   -1, 
           -1,  314,  315,   -1,  317,  318,  319,  320,  321,  322, 
          323,   -1,   -1,  326,  327,   -1,   -1,   -1,  331,  332, 
          333,  334,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,   -1, 
           -1,  364,  365,  257,  258,  259,  260,  261,  262,  263, 
          264,  265,  266,  267,  268,  269,  270,  271,  272,  273, 
          274,  275,  276,  277,  278,  279,  280,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,  290,  291,  292,  293, 
          294,  295,  296,  297,  298,  299,  300,  301,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,   -1, 
          314,  315,   -1,  317,  318,  319,  320,  321,  322,  323, 
           -1,   -1,  326,  327,   -1,   -1,   -1,  331,  332,  333, 
          334,   -1,   -1,   -1,   -1,   -1,   -1,  341,   -1,   -1, 
           -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,   -1,   -1, 
          364,  257,  258,  259,  260,  261,  262,  263,  264,  265, 
          266,  267,  268,  269,  270,  271,  272,  273,  274,  275, 
          276,  277,  278,  279,  280,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,  290,  291,  292,  293,  294,  295, 
          296,  297,  298,  299,  300,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,   -1,  314,  315, 
           -1,  317,  318,  319,  320,  321,  322,  323,   -1,   -1, 
          326,  327,   -1,   -1,   -1,  331,  332,  333,  334,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  348,  349,   -1,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,   -1,   -1,  364,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,  267, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,  279,  280,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,  293,  294,  295,  296,  297, 
          298,  299,  300,  301,  302,  303,  304,   -1,  306,  307, 
           -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   -1,  326,  327, 
           -1,   -1,   -1,  331,  332,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          348,  349,   -1,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,  306,  307,  364,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,   -1,   -1,  326,  327,   -1,   -1,   -1,  331, 
          332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          306,  307,  364,   -1,  310,   -1,   -1,   -1,  314,  315, 
           -1,  317,  318,  319,  320,  321,  322,  323,   -1,   -1, 
          326,  327,   -1,   -1,   -1,  331,  332,  333,  334,   -1, 
           -1,   -1,   -1,   -1,   -1,  341,   -1,   -1,   -1,   -1, 
           -1,   -1,  348,  349,   -1,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,  306,  307,  364,   -1, 
          310,   -1,   -1,   -1,  314,  315,   -1,  317,  318,  319, 
          320,  321,  322,  323,   -1,   -1,  326,  327,   -1,   -1, 
           -1,  331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1, 
           -1,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349, 
           -1,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,  306,  307,  364,   -1,  310,   -1,   -1,   -1, 
          314,  315,   -1,  317,  318,  319,  320,  321,  322,  323, 
           -1,   -1,  326,  327,   -1,   -1,   -1,  331,  332,  333, 
          334,   -1,   -1,   -1,   -1,   -1,   -1,  341,   -1,   -1, 
           -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,  306,  307, 
          364,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   -1,  326,  327, 
           -1,   -1,   -1,  331,  332,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,   -1, 
          348,  349,   -1,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,  306,  307,  364,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,   -1,   -1,  326,  327,   -1,   -1,   -1,  331, 
          332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          306,  307,  364,   -1,  310,   -1,   -1,   -1,  314,  315, 
           -1,  317,  318,  319,  320,  321,  322,  323,   -1,   -1, 
          326,  327,   -1,   -1,   -1,  331,  332,  333,  334,   -1, 
           -1,   -1,   -1,   -1,   -1,  341,   -1,   -1,   -1,   -1, 
           -1,   -1,  348,  349,   -1,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,  306,  307,  364,   -1, 
          310,   -1,   -1,   -1,  314,  315,   -1,  317,  318,  319, 
          320,  321,  322,  323,   -1,   -1,  326,  327,   -1,   -1, 
           -1,  331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1, 
           -1,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349, 
           -1,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,   -1,   -1,  364, 
      };
   }

}
